@@ -29,6 +29,7 @@ import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
+
 import javax.jms.IllegalStateException;
 import javax.jms.InvalidDestinationException;
 import javax.jms.JMSException;
@@ -36,6 +37,7 @@ import javax.jms.Message;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageListener;
 import javax.jms.TransactionRolledBackException;
+
 import org.apache.activemq.blob.BlobDownloader;
 import org.apache.activemq.command.ActiveMQBlobMessage;
 import org.apache.activemq.command.ActiveMQDestination;
@@ -54,7 +56,6 @@ import org.apache.activemq.management.JMSConsumerStatsImpl;
 import org.apache.activemq.management.StatsCapable;
 import org.apache.activemq.management.StatsImpl;
 import org.apache.activemq.selector.SelectorParser;
-import org.apache.activemq.thread.Scheduler;
 import org.apache.activemq.transaction.Synchronization;
 import org.apache.activemq.util.Callback;
 import org.apache.activemq.util.IntrospectionSupport;
@@ -91,8 +92,8 @@ import org.slf4j.LoggerFactory;
  * <P>
  * It is a client programming error for a <CODE>MessageListener</CODE> to
  * throw an exception.
- * 
- * 
+ *
+ *
  * @see javax.jms.MessageConsumer
  * @see javax.jms.QueueReceiver
  * @see javax.jms.TopicSubscriber
@@ -109,7 +110,6 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
     }
 
     private static final Logger LOG = LoggerFactory.getLogger(ActiveMQMessageConsumer.class);
-    protected final Scheduler scheduler;
     protected final ActiveMQSession session;
     protected final ConsumerInfo info;
 
@@ -148,14 +148,16 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
     private long lastDeliveredSequenceId;
 
     private IOException failureError;
-    
+
     private long optimizeAckTimestamp = System.currentTimeMillis();
-    private final long optimizeAckTimeout = 300;
+    private long optimizeAcknowledgeTimeOut = 0;
     private long failoverRedeliveryWaitPeriod = 0;
+    private boolean transactedIndividualAck = false;
+    private boolean nonBlockingRedelivery = false;
 
     /**
      * Create a MessageConsumer
-     * 
+     *
      * @param session
      * @param dest
      * @param name
@@ -205,7 +207,6 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         }
 
         this.session = session;
-        this.scheduler = session.getScheduler();
         this.redeliveryPolicy = session.connection.getRedeliveryPolicy();
         setTransformer(session.getTransformer());
 
@@ -222,8 +223,18 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
         // Allows the options on the destination to configure the consumerInfo
         if (dest.getOptions() != null) {
-            Map<String, String> options = new HashMap<String, String>(dest.getOptions());
-            IntrospectionSupport.setProperties(this.info, options, "consumer.");
+            Map<String, Object> options = IntrospectionSupport.extractProperties(
+                new HashMap<String, Object>(dest.getOptions()), "consumer.");
+            IntrospectionSupport.setProperties(this.info, options);
+            if (options.size() > 0) {
+                String msg = "There are " + options.size()
+                    + " consumer options that couldn't be set on the consumer."
+                    + " Check the options are spelled correctly."
+                    + " Unknown parameters=[" + options + "]."
+                    + " This consumer cannot be started.";
+                LOG.warn(msg);
+                throw new ConfigurationException(msg);
+            }
         }
 
         this.info.setDestination(dest);
@@ -244,8 +255,13 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         this.stats = new JMSConsumerStatsImpl(session.getSessionStats(), dest);
         this.optimizeAcknowledge = session.connection.isOptimizeAcknowledge() && session.isAutoAcknowledge()
                                    && !info.isBrowser();
+        if (this.optimizeAcknowledge) {
+            this.optimizeAcknowledgeTimeOut = session.connection.getOptimizeAcknowledgeTimeOut();
+        }
         this.info.setOptimizedAcknowledge(this.optimizeAcknowledge);
         this.failoverRedeliveryWaitPeriod = session.connection.getConsumerFailoverRedeliveryWaitPeriod();
+        this.nonBlockingRedelivery = session.connection.isNonBlockingRedelivery();
+        this.transactedIndividualAck = session.connection.isTransactedIndividualAck() || this.nonBlockingRedelivery;
         if (messageListener != null) {
             setMessageListener(messageListener);
         }
@@ -324,7 +340,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /**
      * Retrieve is a browser
-     * 
+     *
      * @return true if a browser
      */
     protected boolean isBrowser() {
@@ -354,7 +370,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /**
      * Gets this message consumer's message selector expression.
-     * 
+     *
      * @return this message consumer's message selector, or null if no message
      *         selector exists for the message consumer (that is, if the message
      *         selector was not set or was set to null or the empty string)
@@ -368,7 +384,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /**
      * Gets the message consumer's <CODE>MessageListener</CODE>.
-     * 
+     *
      * @return the listener for the message consumer, or null if no listener is
      *         set
      * @throws JMSException if the JMS provider fails to get the message
@@ -389,7 +405,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
      * The effect of calling <CODE>MessageConsumer.setMessageListener</CODE>
      * while messages are being consumed by an existing listener or the consumer
      * is being used to consume messages synchronously is undefined.
-     * 
+     *
      * @param listener the listener to which the messages are to be delivered
      * @throws JMSException if the JMS provider fails to receive the next
      *                 message due to some internal error.
@@ -438,7 +454,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
      * then it it tries to not block at all, it returns a message if it is
      * available - if timeout>0 then it blocks up to timeout amount of time.
      * Expired messages will consumed by this method.
-     * 
+     *
      * @throws JMSException
      * @return null if we timeout or if the consumer is closed.
      */
@@ -454,11 +470,11 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                     if (timeout > 0 && !unconsumedMessages.isClosed()) {
                         timeout = Math.max(deadline - System.currentTimeMillis(), 0);
                     } else {
-                    	if (failureError != null) {
-                    		throw JMSExceptionSupport.create(failureError);
-                    	} else {
-                    		return null;
-                    	}
+                        if (failureError != null) {
+                            throw JMSExceptionSupport.create(failureError);
+                        } else {
+                            return null;
+                        }
                     }
                 } else if (md.getMessage() == null) {
                     return null;
@@ -492,7 +508,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
      * <P>
      * If this <CODE>receive</CODE> is done within a transaction, the consumer
      * retains the message until the transaction commits.
-     * 
+     *
      * @return the next message produced for this message consumer, or null if
      *         this message consumer is concurrently closed
      */
@@ -519,7 +535,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
     private ActiveMQMessage createActiveMQMessage(final MessageDispatch md) throws JMSException {
         ActiveMQMessage m = (ActiveMQMessage)md.getMessage().copy();
         if (m.getDataStructureType()==CommandTypes.ACTIVEMQ_BLOB_MESSAGE) {
-        	((ActiveMQBlobMessage)m).setBlobDownloader(new BlobDownloader(session.getBlobTransferPolicy()));
+            ((ActiveMQBlobMessage)m).setBlobDownloader(new BlobDownloader(session.getBlobTransferPolicy()));
         }
         if (transformer != null) {
             Message transformedMessage = transformer.consumerTransform(session, this, m);
@@ -552,7 +568,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
      * This call blocks until a message arrives, the timeout expires, or this
      * message consumer is closed. A <CODE>timeout</CODE> of zero never
      * expires, and the call blocks indefinitely.
-     * 
+     *
      * @param timeout the timeout value (in milliseconds), a time out of zero
      *                never expires.
      * @return the next message produced for this message consumer, or null if
@@ -564,7 +580,6 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         checkMessageListener();
         if (timeout == 0) {
             return this.receive();
-
         }
 
         sendPullCommand(timeout);
@@ -590,7 +605,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /**
      * Receives the next message if one is immediately available.
-     * 
+     *
      * @return the next message produced for this message consumer, or null if
      *         one is not available
      * @throws JMSException if the JMS provider fails to receive the next
@@ -630,7 +645,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
      * This call blocks until a <CODE>receive</CODE> or message listener in
      * progress has completed. A blocked message consumer <CODE>receive </CODE>
      * call returns null when this message consumer is closed.
-     * 
+     *
      * @throws JMSException if the JMS provider fails to close the consumer due
      *                 to some internal error.
      */
@@ -650,11 +665,14 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                 });
             } else {
                 doClose();
-            } 
+            }
         }
     }
 
     void doClose() throws JMSException {
+        // Store interrupted state and clear so that Transport operations don't
+        // throw InterruptedException and we ensure that resources are clened up.
+        boolean interrupted = Thread.interrupted();
         dispose();
         RemoveInfo removeCommand = info.createRemoveCommand();
         if (LOG.isDebugEnabled()) {
@@ -662,20 +680,22 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         }
         removeCommand.setLastDeliveredSequenceId(lastDeliveredSequenceId);
         this.session.asyncSendPacket(removeCommand);
-    }
-    
+        if (interrupted) {
+            Thread.currentThread().interrupt();
+        }    }
+
     void inProgressClearRequired() {
         inProgressClearRequiredFlag = true;
         // deal with delivered messages async to avoid lock contention with in progress acks
         clearDispatchList = true;
     }
-    
+
     void clearMessagesInProgress() {
         if (inProgressClearRequiredFlag) {
             synchronized (unconsumedMessages.getMutex()) {
                 if (inProgressClearRequiredFlag) {
                     if (LOG.isDebugEnabled()) {
-                        LOG.debug(getConsumerId() + " clearing dispatched list (" + unconsumedMessages.size() + ") on transport interrupt");
+                        LOG.debug(getConsumerId() + " clearing unconsumed list (" + unconsumedMessages.size() + ") on transport interrupt");
                     }
                     // ensure unconsumed are rolledback up front as they may get redelivered to another consumer
                     List<MessageDispatch> list = unconsumedMessages.removeAll();
@@ -701,18 +721,18 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                     if (ack != null) {
                         deliveredMessages.clear();
                         ackCounter = 0;
-            		} else {
-            		    ack = pendingAck;
-            		    pendingAck = null;
-            		}
-            	}
+                    } else {
+                        ack = pendingAck;
+                        pendingAck = null;
+                    }
+                }
             } else if (pendingAck != null && pendingAck.isStandardAck()) {
                 ack = pendingAck;
                 pendingAck = null;
             }
             if (ack != null) {
                 final MessageAck ackToSend = ack;
-                
+
                 if (executorService == null) {
                     executorService = Executors.newSingleThreadExecutor();
                 }
@@ -735,10 +755,10 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     public void dispose() throws JMSException {
         if (!unconsumedMessages.isClosed()) {
-            
+
             // Do we have any acks we need to send out before closing?
             // Ack any delivered messages now.
-            if (!session.getTransacted()) { 
+            if (!session.getTransacted()) {
                 deliverAcks();
                 if (isAutoAcknowledgeBatch()) {
                     acknowledge();
@@ -752,7 +772,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                     Thread.currentThread().interrupt();
                 }
             }
-            
+
             if (session.isClientAcknowledge()) {
                 if (!this.info.isBrowser()) {
                     // rollback duplicates that aren't acknowledged
@@ -830,11 +850,24 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                 deliveredMessages.addFirst(md);
             }
             if (session.getTransacted()) {
-                ackLater(md, MessageAck.DELIVERED_ACK_TYPE);
+                if (transactedIndividualAck) {
+                    immediateIndividualTransactedAck(md);
+                } else {
+                    ackLater(md, MessageAck.DELIVERED_ACK_TYPE);
+                }
             }
         }
     }
-    
+
+    private void immediateIndividualTransactedAck(MessageDispatch md) throws JMSException {
+        // acks accumulate on the broker pending transaction completion to indicate
+        // delivery status
+        registerSync();
+        MessageAck ack = new MessageAck(md, MessageAck.INDIVIDUAL_ACK_TYPE, 1);
+        ack.setTransactionId(session.getTransactionContext().getTransactionId());
+        session.syncSendPacket(ack);
+    }
+
     private void afterMessageIsConsumed(MessageDispatch md, boolean messageExpired) throws JMSException {
         if (unconsumedMessages.isClosed()) {
             return;
@@ -855,14 +888,14 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                         if (!deliveredMessages.isEmpty()) {
                             if (optimizeAcknowledge) {
                                 ackCounter++;
-                                if (ackCounter >= (info.getPrefetchSize() * .65) || System.currentTimeMillis() >= (optimizeAckTimestamp + optimizeAckTimeout)) {
-                                	MessageAck ack = makeAckForAllDeliveredMessages(MessageAck.STANDARD_ACK_TYPE);
-                                	if (ack != null) {
-                            		    deliveredMessages.clear();
-                            		    ackCounter = 0;
-                            		    session.sendAck(ack);
-                            		    optimizeAckTimestamp = System.currentTimeMillis();
-                                	}
+                                if (ackCounter >= (info.getPrefetchSize() * .65) || (optimizeAcknowledgeTimeOut > 0 && System.currentTimeMillis() >= (optimizeAckTimestamp + optimizeAcknowledgeTimeOut))) {
+                                    MessageAck ack = makeAckForAllDeliveredMessages(MessageAck.STANDARD_ACK_TYPE);
+                                    if (ack != null) {
+                                        deliveredMessages.clear();
+                                        ackCounter = 0;
+                                        session.sendAck(ack);
+                                        optimizeAckTimestamp = System.currentTimeMillis();
+                                    }
                                 }
                             } else {
                                 MessageAck ack = makeAckForAllDeliveredMessages(MessageAck.STANDARD_ACK_TYPE);
@@ -885,7 +918,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                 if (messageUnackedByConsumer) {
                     ackLater(md, MessageAck.DELIVERED_ACK_TYPE);
                 }
-            } 
+            }
             else {
                 throw new IllegalStateException("Invalid session state.");
             }
@@ -895,54 +928,32 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
     /**
      * Creates a MessageAck for all messages contained in deliveredMessages.
      * Caller should hold the lock for deliveredMessages.
-     * 
-     * @param type Ack-Type (i.e. MessageAck.STANDARD_ACK_TYPE) 
+     *
+     * @param type Ack-Type (i.e. MessageAck.STANDARD_ACK_TYPE)
      * @return <code>null</code> if nothing to ack.
      */
-	private MessageAck makeAckForAllDeliveredMessages(byte type) {
-		synchronized (deliveredMessages) {
-			if (deliveredMessages.isEmpty())
-				return null;
-			    
-			MessageDispatch md = deliveredMessages.getFirst();
-		    MessageAck ack = new MessageAck(md, type, deliveredMessages.size());
-		    ack.setFirstMessageId(deliveredMessages.getLast().getMessage().getMessageId());
-		    return ack;
-		}
-	}
+    private MessageAck makeAckForAllDeliveredMessages(byte type) {
+        synchronized (deliveredMessages) {
+            if (deliveredMessages.isEmpty())
+                return null;
+
+            MessageDispatch md = deliveredMessages.getFirst();
+            MessageAck ack = new MessageAck(md, type, deliveredMessages.size());
+            ack.setFirstMessageId(deliveredMessages.getLast().getMessage().getMessageId());
+            return ack;
+        }
+    }
 
     private void ackLater(MessageDispatch md, byte ackType) throws JMSException {
 
         // Don't acknowledge now, but we may need to let the broker know the
         // consumer got the message to expand the pre-fetch window
         if (session.getTransacted()) {
-            session.doStartTransaction();
-            if (!synchronizationRegistered) {
-                synchronizationRegistered = true;
-                session.getTransactionContext().addSynchronization(new Synchronization() {
-                    @Override
-                    public void beforeEnd() throws Exception {
-                        acknowledge();
-                        synchronizationRegistered = false;
-                    }
-
-                    @Override
-                    public void afterCommit() throws Exception {
-                        commit();
-                        synchronizationRegistered = false;
-                    }
-
-                    @Override
-                    public void afterRollback() throws Exception {
-                        rollback();
-                        synchronizationRegistered = false;
-                    }
-                });
-            }
+            registerSync();
         }
 
         deliveredCounter++;
-        
+
         MessageAck oldPendingAck = pendingAck;
         pendingAck = new MessageAck(md, ackType, deliveredCounter);
         pendingAck.setTransactionId(session.getTransactionContext().getTransactionId());
@@ -964,7 +975,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                 }
             }
         }
-        
+
         if ((0.5 * info.getPrefetchSize()) <= (deliveredCounter - additionalWindowSize)) {
             session.sendAck(pendingAck);
             pendingAck=null;
@@ -973,10 +984,44 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         }
     }
 
+    private void registerSync() throws JMSException {
+        session.doStartTransaction();
+        if (!synchronizationRegistered) {
+            synchronizationRegistered = true;
+            session.getTransactionContext().addSynchronization(new Synchronization() {
+                @Override
+                public void beforeEnd() throws Exception {
+                    if (transactedIndividualAck) {
+                        clearDispatchList();
+                        waitForRedeliveries();
+                        synchronized(deliveredMessages) {
+                            rollbackOnFailedRecoveryRedelivery();
+                        }
+                    } else {
+                        acknowledge();
+                    }
+                    synchronizationRegistered = false;
+                }
+
+                @Override
+                public void afterCommit() throws Exception {
+                    commit();
+                    synchronizationRegistered = false;
+                }
+
+                @Override
+                public void afterRollback() throws Exception {
+                    rollback();
+                    synchronizationRegistered = false;
+                }
+            });
+        }
+    }
+
     /**
      * Acknowledge all the messages that have been delivered to the client up to
      * this point.
-     * 
+     *
      * @throws JMSException
      */
     public void acknowledge() throws JMSException {
@@ -986,8 +1031,8 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
             // Acknowledge all messages so far.
             MessageAck ack = makeAckForAllDeliveredMessages(MessageAck.STANDARD_ACK_TYPE);
             if (ack == null)
-            	return; // no msgs
-            
+                return; // no msgs
+
             if (session.getTransacted()) {
                 rollbackOnFailedRecoveryRedelivery();
                 session.doStartTransaction();
@@ -995,17 +1040,17 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
             }
             session.sendAck(ack);
             pendingAck = null;
-            
+
             // Adjust the counters
             deliveredCounter = Math.max(0, deliveredCounter - deliveredMessages.size());
             additionalWindowSize = Math.max(0, additionalWindowSize - deliveredMessages.size());
-            
-            if (!session.getTransacted()) {  
+
+            if (!session.getTransacted()) {
                 deliveredMessages.clear();
-            } 
+            }
         }
     }
-    
+
     private void waitForRedeliveries() {
         if (failoverRedeliveryWaitPeriod > 0 && previouslyDeliveredMessages != null) {
             long expiry = System.currentTimeMillis() + failoverRedeliveryWaitPeriod;
@@ -1013,7 +1058,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
             do {
                 numberNotReplayed = 0;
                 synchronized(deliveredMessages) {
-                    if (previouslyDeliveredMessages != null) { 
+                    if (previouslyDeliveredMessages != null) {
                         for (Entry<MessageId, Boolean> entry: previouslyDeliveredMessages.entrySet()) {
                             if (!entry.getValue()) {
                                 numberNotReplayed++;
@@ -1047,17 +1092,17 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                     numberNotReplayed++;
                     if (LOG.isDebugEnabled()) {
                         LOG.debug("previously delivered message has not been replayed in transaction: "
-                                + previouslyDeliveredMessages.transactionId 
+                                + previouslyDeliveredMessages.transactionId
                                 + " , messageId: " + entry.getKey());
                     }
                 }
             }
             if (numberNotReplayed > 0) {
-                String message = "rolling back transaction (" 
+                String message = "rolling back transaction ("
                     + previouslyDeliveredMessages.transactionId + ") post failover recovery. " + numberNotReplayed
                     + " previously delivered message(s) not replayed to consumer: " + this.getConsumerId();
                 LOG.warn(message);
-                throw new TransactionRolledBackException(message);   
+                throw new TransactionRolledBackException(message);
             }
         }
     }
@@ -1098,7 +1143,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                 if (deliveredMessages.isEmpty()) {
                     return;
                 }
-    
+
                 // use initial delay for first redelivery
                 MessageDispatch lastMd = deliveredMessages.getFirst();
                 final int currentRedeliveryCount = lastMd.getMessage().getRedeliveryCounter();
@@ -1108,61 +1153,83 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                     redeliveryDelay = redeliveryPolicy.getInitialRedeliveryDelay();
                 }
                 MessageId firstMsgId = deliveredMessages.getLast().getMessage().getMessageId();
-    
+
                 for (Iterator<MessageDispatch> iter = deliveredMessages.iterator(); iter.hasNext();) {
                     MessageDispatch md = iter.next();
                     md.getMessage().onMessageRolledBack();
                     // ensure we don't filter this as a duplicate
                     session.connection.rollbackDuplicate(this, md.getMessage());
                 }
-    
+
                 if (redeliveryPolicy.getMaximumRedeliveries() != RedeliveryPolicy.NO_MAXIMUM_REDELIVERIES
                     && lastMd.getMessage().getRedeliveryCounter() > redeliveryPolicy.getMaximumRedeliveries()) {
                     // We need to NACK the messages so that they get sent to the
                     // DLQ.
                     // Acknowledge the last message.
-                    
+
                     MessageAck ack = new MessageAck(lastMd, MessageAck.POSION_ACK_TYPE, deliveredMessages.size());
                     ack.setPoisonCause(lastMd.getRollbackCause());
-					ack.setFirstMessageId(firstMsgId);
+                    ack.setFirstMessageId(firstMsgId);
                     session.sendAck(ack,true);
                     // Adjust the window size.
                     additionalWindowSize = Math.max(0, additionalWindowSize - deliveredMessages.size());
                     redeliveryDelay = 0;
                 } else {
-                    
+
                     // only redelivery_ack after first delivery
                     if (currentRedeliveryCount > 0) {
                         MessageAck ack = new MessageAck(lastMd, MessageAck.REDELIVERED_ACK_TYPE, deliveredMessages.size());
                         ack.setFirstMessageId(firstMsgId);
                         session.sendAck(ack,true);
                     }
-    
+
                     // stop the delivery of messages.
-                    unconsumedMessages.stop();
-    
-                    for (Iterator<MessageDispatch> iter = deliveredMessages.iterator(); iter.hasNext();) {
-                        MessageDispatch md = iter.next();
-                        unconsumedMessages.enqueueFirst(md);
-                    }
-    
-                    if (redeliveryDelay > 0 && !unconsumedMessages.isClosed()) {
-                        // Start up the delivery again a little later.
-                        scheduler.executeAfterDelay(new Runnable() {
-                            public void run() {
-                                try {
-                                    if (started.get()) {
-                                        start();
+                    if (nonBlockingRedelivery) {
+                        if (!unconsumedMessages.isClosed()) {
+
+                            final LinkedList<MessageDispatch> pendingRedeliveries =
+                                new LinkedList<MessageDispatch>(deliveredMessages);
+
+                            // Start up the delivery again a little later.
+                            session.getScheduler().executeAfterDelay(new Runnable() {
+                                public void run() {
+                                    try {
+                                        if (!unconsumedMessages.isClosed()) {
+                                            for(MessageDispatch dispatch : pendingRedeliveries) {
+                                                session.dispatch(dispatch);
+                                            }
+                                        }
+                                    } catch (Exception e) {
+                                        session.connection.onAsyncException(e);
                                     }
-                                } catch (JMSException e) {
-                                    session.connection.onAsyncException(e);
                                 }
-                            }
-                        }, redeliveryDelay);
+                            }, redeliveryDelay);
+                        }
+
                     } else {
-                        start();
+                        unconsumedMessages.stop();
+
+                        for (MessageDispatch md : deliveredMessages) {
+                            unconsumedMessages.enqueueFirst(md);
+                        }
+
+                        if (redeliveryDelay > 0 && !unconsumedMessages.isClosed()) {
+                            // Start up the delivery again a little later.
+                            session.getScheduler().executeAfterDelay(new Runnable() {
+                                public void run() {
+                                    try {
+                                        if (started.get()) {
+                                            start();
+                                        }
+                                    } catch (JMSException e) {
+                                        session.connection.onAsyncException(e);
+                                    }
+                                }
+                            }, redeliveryDelay);
+                        } else {
+                            start();
+                        }
                     }
-    
                 }
                 deliveredCounter -= deliveredMessages.size();
                 deliveredMessages.clear();
@@ -1175,13 +1242,13 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /*
      * called with unconsumedMessages && deliveredMessages locked
-     * remove any message not re-delivered as they can't be replayed to this 
+     * remove any message not re-delivered as they can't be replayed to this
      * consumer on rollback
      */
     private void rollbackPreviouslyDeliveredAndNotRedelivered() {
         if (previouslyDeliveredMessages != null) {
             for (Entry<MessageId, Boolean> entry: previouslyDeliveredMessages.entrySet()) {
-                if (!entry.getValue()) {              
+                if (!entry.getValue()) {
                     removeFromDeliveredMessages(entry.getKey());
                 }
             }
@@ -1203,6 +1270,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
             }
         }
     }
+
     /*
      * called with deliveredMessages locked
      */
@@ -1254,10 +1322,9 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                         }
                     } else {
                         if (!session.isTransacted()) {
-                            if (LOG.isDebugEnabled()) {
-                                LOG.debug(getConsumerId() + " ignoring (auto acking) duplicate: " + md.getMessage());
-                            }
-                            MessageAck ack = new MessageAck(md, MessageAck.STANDARD_ACK_TYPE, 1);
+                            LOG.warn("Duplicate dispatch on connection: " + session.getConnection().getConnectionInfo().getConnectionId()
+                                    + " to consumer: "  + getConsumerId() + ", ignoring (auto acking) duplicate: " + md);
+                            MessageAck ack = new MessageAck(md, MessageAck.INDIVIDUAL_ACK_TYPE, 1);
                             session.sendAck(ack);
                         } else {
                             if (LOG.isDebugEnabled()) {
@@ -1274,14 +1341,20 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
                                 }
                             }
                             if (needsPoisonAck) {
-                                LOG.warn("acking duplicate delivery as poison, redelivery must be pending to another"
-                                        + " consumer on this connection, failoverRedeliveryWaitPeriod=" 
-                                        + failoverRedeliveryWaitPeriod + ". Message: " + md);
                                 MessageAck poisonAck = new MessageAck(md, MessageAck.POSION_ACK_TYPE, 1);
                                 poisonAck.setFirstMessageId(md.getMessage().getMessageId());
+                                poisonAck.setPoisonCause(new JMSException("Duplicate dispatch with transacted redeliver pending on another consumer, connection: "
+                                        + session.getConnection().getConnectionInfo().getConnectionId()));
+                                LOG.warn("acking duplicate delivery as poison, redelivery must be pending to another"
+                                        + " consumer on this connection, failoverRedeliveryWaitPeriod="
+                                        + failoverRedeliveryWaitPeriod + ". Message: " + md + ", poisonAck: " + poisonAck);
                                 session.sendAck(poisonAck);
                             } else {
-                                ackLater(md, MessageAck.DELIVERED_ACK_TYPE);
+                                if (transactedIndividualAck) {
+                                    immediateIndividualTransactedAck(md);
+                                } else {
+                                    ackLater(md, MessageAck.DELIVERED_ACK_TYPE);
+                                }
                             }
                         }
                     }
@@ -1299,10 +1372,10 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
     // async (on next call) clear or track delivered as they may be flagged as duplicates if they arrive again
     private void clearDispatchList() {
         if (clearDispatchList) {
-            synchronized (deliveredMessages) {  
+            synchronized (deliveredMessages) {
                 if (clearDispatchList) {
                     if (!deliveredMessages.isEmpty()) {
-                        if (session.isTransacted()) {    
+                        if (session.isTransacted()) {
                             if (LOG.isDebugEnabled()) {
                                 LOG.debug(getConsumerId() + " tracking existing transacted delivered list (" + deliveredMessages.size() + ") on transport interrupt");
                             }
@@ -1352,7 +1425,7 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
 
     /**
      * Delivers a message to the message listener.
-     * 
+     *
      * @return
      * @throws JMSException
      */
@@ -1376,11 +1449,11 @@ public class ActiveMQMessageConsumer implements MessageAvailableConsumer, StatsC
         return lastDeliveredSequenceId;
     }
 
-	public IOException getFailureError() {
-		return failureError;
-	}
+    public IOException getFailureError() {
+        return failureError;
+    }
 
-	public void setFailureError(IOException failureError) {
-		this.failureError = failureError;
-	}
+    public void setFailureError(IOException failureError) {
+        this.failureError = failureError;
+    }
 }
