@@ -26,6 +26,7 @@ import java.util.List;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.atomic.AtomicBoolean;
+
 import javax.jms.BytesMessage;
 import javax.jms.Destination;
 import javax.jms.IllegalStateException;
@@ -53,6 +54,7 @@ import javax.jms.TopicPublisher;
 import javax.jms.TopicSession;
 import javax.jms.TopicSubscriber;
 import javax.jms.TransactionRolledBackException;
+
 import org.apache.activemq.blob.BlobDownloader;
 import org.apache.activemq.blob.BlobTransferPolicy;
 import org.apache.activemq.blob.BlobUploader;
@@ -87,6 +89,7 @@ import org.apache.activemq.thread.Scheduler;
 import org.apache.activemq.transaction.Synchronization;
 import org.apache.activemq.usage.MemoryUsage;
 import org.apache.activemq.util.Callback;
+import org.apache.activemq.util.JMSExceptionSupport;
 import org.apache.activemq.util.LongSequenceGenerator;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -172,21 +175,21 @@ import org.slf4j.LoggerFactory;
  * transactions directly, it is unlikely that many JMS clients will do this.
  * Support for JTA in the JMS API is targeted at systems vendors who will be
  * integrating the JMS API into their application server products.
- * 
- * 
+ *
+ *
  * @see javax.jms.Session
  * @see javax.jms.QueueSession
  * @see javax.jms.TopicSession
  * @see javax.jms.XASession
  */
 public class ActiveMQSession implements Session, QueueSession, TopicSession, StatsCapable, ActiveMQDispatcher {
-	
-	/**
-	 * Only acknowledge an individual message - using message.acknowledge()
-	 * as opposed to CLIENT_ACKNOWLEDGE which 
-	 * acknowledges all messages consumed by a session at when acknowledge()
-	 * is called
-	 */
+
+    /**
+     * Only acknowledge an individual message - using message.acknowledge()
+     * as opposed to CLIENT_ACKNOWLEDGE which
+     * acknowledges all messages consumed by a session at when acknowledge()
+     * is called
+     */
     public static final int INDIVIDUAL_ACKNOWLEDGE = 4;
     public static final int MAX_ACK_CONSTANT = INDIVIDUAL_ACKNOWLEDGE;
 
@@ -197,7 +200,6 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     }
 
     private static final Logger LOG = LoggerFactory.getLogger(ActiveMQSession.class);
-    private final Scheduler scheduler;
     private final ThreadPoolExecutor connectionExecutor;
 
     protected int acknowledgementMode;
@@ -229,7 +231,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Construct the Session
-     * 
+     *
      * @param connection
      * @param sessionId
      * @param acknowledgeMode n.b if transacted - the acknowledgeMode ==
@@ -250,10 +252,9 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
         this.connection.asyncSendPacket(info);
         setTransformer(connection.getTransformer());
         setBlobTransferPolicy(connection.getBlobTransferPolicy());
-        this.scheduler=connection.getScheduler();
         this.connectionExecutor=connection.getExecutor();
         this.executor = new ActiveMQSessionExecutor(this);
-        connection.addSession(this);        
+        connection.addSession(this);
         if (connection.isStarted()) {
             start();
         }
@@ -266,7 +267,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Sets the transaction context of the session.
-     * 
+     *
      * @param transactionContext - provides the means to control a JMS
      *                transaction.
      */
@@ -276,7 +277,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Returns the transaction context of the session.
-     * 
+     *
      * @return transactionContext - session's transaction context.
      */
     public TransactionContext getTransactionContext() {
@@ -285,7 +286,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /*
      * (non-Javadoc)
-     * 
+     *
      * @see org.apache.activemq.management.StatsCapable#getStats()
      */
     public StatsImpl getStats() {
@@ -294,7 +295,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Returns the session's statistics.
-     * 
+     *
      * @return stats - session's statistics.
      */
     public JMSSessionStatsImpl getSessionStats() {
@@ -305,7 +306,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates a <CODE>BytesMessage</CODE> object. A <CODE>BytesMessage</CODE>
      * object is used to send a message containing a stream of uninterpreted
      * bytes.
-     * 
+     *
      * @return the an ActiveMQBytesMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -321,7 +322,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * object is used to send a self-defining set of name-value pairs, where
      * names are <CODE>String</CODE> objects and values are primitive values
      * in the Java programming language.
-     * 
+     *
      * @return an ActiveMQMapMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -338,7 +339,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * <CODE>Message</CODE> object holds all the standard message header
      * information. It can be sent when a message containing only header
      * information is sufficient.
-     * 
+     *
      * @return an ActiveMQMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -353,7 +354,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates an <CODE>ObjectMessage</CODE> object. An
      * <CODE>ObjectMessage</CODE> object is used to send a message that
      * contains a serializable Java object.
-     * 
+     *
      * @return an ActiveMQObjectMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -368,7 +369,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates an initialized <CODE>ObjectMessage</CODE> object. An
      * <CODE>ObjectMessage</CODE> object is used to send a message that
      * contains a serializable Java object.
-     * 
+     *
      * @param object the object to use to initialize this message
      * @return an ActiveMQObjectMessage
      * @throws JMSException if the JMS provider fails to create this message due
@@ -385,7 +386,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates a <CODE>StreamMessage</CODE> object. A
      * <CODE>StreamMessage</CODE> object is used to send a self-defining
      * stream of primitive values in the Java programming language.
-     * 
+     *
      * @return an ActiveMQStreamMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -400,7 +401,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates a <CODE>TextMessage</CODE> object. A <CODE>TextMessage</CODE>
      * object is used to send a message containing a <CODE>String</CODE>
      * object.
-     * 
+     *
      * @return an ActiveMQTextMessage
      * @throws JMSException if the JMS provider fails to create this message due
      *                 to some internal error.
@@ -415,7 +416,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates an initialized <CODE>TextMessage</CODE> object. A
      * <CODE>TextMessage</CODE> object is used to send a message containing a
      * <CODE>String</CODE>.
-     * 
+     *
      * @param text the string used to initialize this message
      * @return an ActiveMQTextMessage
      * @throws JMSException if the JMS provider fails to create this message due
@@ -432,7 +433,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates an initialized <CODE>BlobMessage</CODE> object. A
      * <CODE>BlobMessage</CODE> object is used to send a message containing a
      * <CODE>URL</CODE> which points to some network addressible BLOB.
-     * 
+     *
      * @param url the network addressable URL used to pass directly to the
      *                consumer
      * @return a BlobMessage
@@ -447,7 +448,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Creates an initialized <CODE>BlobMessage</CODE> object. A
      * <CODE>BlobMessage</CODE> object is used to send a message containing a
      * <CODE>URL</CODE> which points to some network addressible BLOB.
-     * 
+     *
      * @param url the network addressable URL used to pass directly to the
      *                consumer
      * @param deletedByBroker indicates whether or not the resource is deleted
@@ -471,7 +472,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * the <CODE>File</CODE> content. Before the message is sent the file
      * conent will be uploaded to the broker or some other remote repository
      * depending on the {@link #getBlobTransferPolicy()}.
-     * 
+     *
      * @param file the file to be uploaded to some remote repo (or the broker)
      *                depending on the strategy
      * @return a BlobMessage
@@ -494,7 +495,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * the <CODE>File</CODE> content. Before the message is sent the file
      * conent will be uploaded to the broker or some other remote repository
      * depending on the {@link #getBlobTransferPolicy()}.
-     * 
+     *
      * @param in the stream to be uploaded to some remote repo (or the broker)
      *                depending on the strategy
      * @return a BlobMessage
@@ -512,7 +513,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Indicates whether the session is in transacted mode.
-     * 
+     *
      * @return true if the session is in transacted mode
      * @throws JMSException if there is some internal error.
      */
@@ -525,7 +526,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Returns the acknowledgement mode of the session. The acknowledgement mode
      * is set at the time that the session is created. If the session is
      * transacted, the acknowledgement mode is ignored.
-     * 
+     *
      * @return If the session is not transacted, returns the current
      *         acknowledgement mode for the session. If the session is
      *         transacted, returns SESSION_TRANSACTED.
@@ -541,7 +542,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Commits all messages done in this transaction and releases any locks
      * currently held.
-     * 
+     *
      * @throws JMSException if the JMS provider fails to commit the transaction
      *                 due to some internal error.
      * @throws TransactionRolledBackException if the transaction is rolled back
@@ -563,7 +564,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Rolls back any messages done in this transaction and releases any locks
      * currently held.
-     * 
+     *
      * @throws JMSException if the JMS provider fails to roll back the
      *                 transaction due to some internal error.
      * @throws javax.jms.IllegalStateException if the method is not called by a
@@ -575,7 +576,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
             throw new javax.jms.IllegalStateException("Not a transacted session");
         }
         if (LOG.isDebugEnabled()) {
-            LOG.debug(getSessionId() + " Transaction Rollback");
+            LOG.debug(getSessionId() + " Transaction Rollback, txid:"  + transactionContext.getTransactionId());
         }
         transactionContext.rollback();
     }
@@ -604,7 +605,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Invoking any other <CODE>Session</CODE> method on a closed session must
      * throw a <CODE> JMSException.IllegalStateException</CODE>. Closing a
      * closed session must <I>not </I> throw an exception.
-     * 
+     *
      * @throws JMSException if the JMS provider fails to close the session due
      *                 to some internal error.
      */
@@ -636,28 +637,36 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     }
 
     private void doClose() throws JMSException {
+        boolean interrupted = Thread.interrupted();
         dispose();
         RemoveInfo removeCommand = info.createRemoveCommand();
         removeCommand.setLastDeliveredSequenceId(lastDeliveredSequenceId);
         connection.asyncSendPacket(removeCommand);
+        if (interrupted) {
+            Thread.currentThread().interrupt();
+        }
     }
 
     void clearMessagesInProgress() {
-        executor.clearMessagesInProgress();        
+        executor.clearMessagesInProgress();
         // we are called from inside the transport reconnection logic
         // which involves us clearing all the connections' consumers
-        // dispatch and delivered lists. So rather than trying to 
-        // grab a mutex (which could be already owned by the message 
-        // listener calling the send or an ack) we allow it to complete in 
-        // a separate thread via the scheduler and notify us via 
+        // dispatch and delivered lists. So rather than trying to
+        // grab a mutex (which could be already owned by the message
+        // listener calling the send or an ack) we allow it to complete in
+        // a separate thread via the scheduler and notify us via
         // connection.transportInterruptionProcessingComplete()
         //
         for (final ActiveMQMessageConsumer consumer : consumers) {
             consumer.inProgressClearRequired();
-            scheduler.executeAfterDelay(new Runnable() {
-                public void run() {
-                    consumer.clearMessagesInProgress();
-                }}, 0l);
+            try {
+                connection.getScheduler().executeAfterDelay(new Runnable() {
+                    public void run() {
+                        consumer.clearMessagesInProgress();
+                    }}, 0l);
+            } catch (JMSException e) {
+                connection.onClientInternalException(e);
+            }
         }
     }
 
@@ -714,7 +723,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Check if the session is closed. It is used for ensuring that the session
      * is open before performing various operations.
-     * 
+     *
      * @throws IllegalStateException if the Session is closed
      */
     protected void checkClosed() throws IllegalStateException {
@@ -723,6 +732,15 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
         }
     }
 
+    /**
+     * Checks if the session is closed.
+     *
+     * @return true if the session is closed, false otherwise.
+     */
+    public boolean isClosed() {
+        return closed;
+    }
+
     /**
      * Stops message delivery in this session, and restarts message delivery
      * with the oldest unacknowledged message.
@@ -740,7 +758,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * that had been previously delivered. Redelivered messages do not have to
      * be delivered in exactly their original delivery order.
      * </UL>
-     * 
+     *
      * @throws JMSException if the JMS provider fails to stop and restart
      *                 message delivery due to some internal error.
      * @throws IllegalStateException if the method is called by a transacted
@@ -762,7 +780,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Returns the session's distinguished message listener (optional).
-     * 
+     *
      * @return the message listener associated with this session
      * @throws JMSException if the JMS provider fails to get the message
      *                 listener due to an internal error.
@@ -783,7 +801,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * messages are still supported.
      * <P>
      * This is an expert facility not used by regular JMS clients.
-     * 
+     *
      * @param listener the message listener to associate with this session
      * @throws JMSException if the JMS provider fails to set the message
      *                 listener due to an internal error.
@@ -803,7 +821,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Optional operation, intended to be used only by Application Servers, not
      * by ordinary JMS clients.
-     * 
+     *
      * @see javax.jms.ServerSession
      */
     public void run() {
@@ -867,7 +885,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
                                 ack.setFirstMessageId(md.getMessage().getMessageId());
                                 asyncSendPacket(ack);
                             } else {
-                                
+
                                 MessageAck ack = new MessageAck(md, MessageAck.REDELIVERED_ACK_TYPE, 1);
                                 ack.setFirstMessageId(md.getMessage().getMessageId());
                                 asyncSendPacket(ack);
@@ -878,7 +896,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
                                 for (int i = 0; i < redeliveryCounter; i++) {
                                     redeliveryDelay = redeliveryPolicy.getNextRedeliveryDelay(redeliveryDelay);
                                 }
-                                scheduler.executeAfterDelay(new Runnable() {
+                                connection.getScheduler().executeAfterDelay(new Runnable() {
 
                                     public void run() {
                                         ((ActiveMQDispatcher)md.getConsumer()).dispatch(md);
@@ -907,7 +925,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * a destination. Since <CODE>Queue </CODE> and <CODE>Topic</CODE> both
      * inherit from <CODE>Destination</CODE>, they can be used in the
      * destination parameter to create a <CODE>MessageProducer</CODE> object.
-     * 
+     *
      * @param destination the <CODE>Destination</CODE> to send to, or null if
      *                this is a producer which does not have a specified
      *                destination.
@@ -933,7 +951,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * Since <CODE>Queue</CODE> and <CODE> Topic</CODE> both inherit from
      * <CODE>Destination</CODE>, they can be used in the destination
      * parameter to create a <CODE>MessageConsumer</CODE>.
-     * 
+     *
      * @param destination the <CODE>Destination</CODE> to access.
      * @return the MessageConsumer
      * @throws JMSException if the session fails to create a consumer due to
@@ -955,7 +973,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * <P>
      * A client uses a <CODE>MessageConsumer</CODE> object to receive messages
      * that have been sent to a destination.
-     * 
+     *
      * @param destination the <CODE>Destination</CODE> to access
      * @param messageSelector only messages with properties matching the message
      *                selector expression are delivered. A value of null or an
@@ -1038,7 +1056,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * inhibit the delivery of messages published by its own connection. The
      * default value for this attribute is False. The <CODE>noLocal</CODE>
      * value must be supported by destinations that are topics.
-     * 
+     *
      * @param destination the <CODE>Destination</CODE> to access
      * @param messageSelector only messages with properties matching the message
      *                selector expression are delivered. A value of null or an
@@ -1130,7 +1148,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * initiated by the JMS API. The one exception is the creation of temporary
      * queues, which is accomplished with the <CODE>createTemporaryQueue</CODE>
      * method.
-     * 
+     *
      * @param queueName the name of this <CODE>Queue</CODE>
      * @return a <CODE>Queue</CODE> with the given name
      * @throws JMSException if the session fails to create a queue due to some
@@ -1158,7 +1176,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * initiated by the JMS API. The one exception is the creation of temporary
      * topics, which is accomplished with the <CODE>createTemporaryTopic</CODE>
      * method.
-     * 
+     *
      * @param topicName the name of this <CODE>Topic</CODE>
      * @return a <CODE>Topic</CODE> with the given name
      * @throws JMSException if the session fails to create a topic due to some
@@ -1176,7 +1194,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueBrowser</CODE> object to peek at the messages on
      * the specified queue.
-     * 
+     *
      * @param queue the <CODE>queue</CODE> to access
      * @exception InvalidDestinationException if an invalid destination is
      *                    specified
@@ -1207,7 +1225,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * The subscriber <CODE>NoLocal</CODE> attribute allows a subscriber to
      * inhibit the delivery of messages published by its own connection. The
      * default value for this attribute is false.
-     * 
+     *
      * @param topic the non-temporary <CODE>Topic</CODE> to subscribe to
      * @param name the name used to identify this subscription
      * @return the TopicSubscriber
@@ -1245,7 +1263,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * durable <CODE>TopicSubscriber</CODE> with the same name and a new topic
      * and/or message selector. Changing a durable subscriber is equivalent to
      * unsubscribing (deleting) the old one and creating a new one.
-     * 
+     *
      * @param topic the non-temporary <CODE>Topic</CODE> to subscribe to
      * @param name the name used to identify this subscription
      * @param messageSelector only messages with properties matching the message
@@ -1264,6 +1282,11 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     public TopicSubscriber createDurableSubscriber(Topic topic, String name, String messageSelector, boolean noLocal) throws JMSException {
         checkClosed();
 
+        if (isIndividualAcknowledge()) {
+            throw JMSExceptionSupport.create("Cannot create a durable consumer for a Session in "+
+                                             "INDIVIDUAL_ACKNOWLEDGE mode.", null);
+        }
+
         if (topic instanceof CustomDestination) {
             CustomDestination customDestination = (CustomDestination)topic;
             return customDestination.createDurableSubscriber(this, name, messageSelector, noLocal);
@@ -1280,7 +1303,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueBrowser</CODE> object to peek at the messages on
      * the specified queue.
-     * 
+     *
      * @param queue the <CODE>queue</CODE> to access
      * @return the Queue Browser
      * @throws JMSException if the session fails to create a browser due to some
@@ -1297,7 +1320,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueBrowser</CODE> object to peek at the messages on
      * the specified queue using a message selector.
-     * 
+     *
      * @param queue the <CODE>queue</CODE> to access
      * @param messageSelector only messages with properties matching the message
      *                selector expression are delivered. A value of null or an
@@ -1319,7 +1342,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>TemporaryQueue</CODE> object. Its lifetime will be that
      * of the <CODE>Connection</CODE> unless it is deleted earlier.
-     * 
+     *
      * @return a temporary queue identity
      * @throws JMSException if the session fails to create a temporary queue due
      *                 to some internal error.
@@ -1333,7 +1356,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>TemporaryTopic</CODE> object. Its lifetime will be that
      * of the <CODE>Connection</CODE> unless it is deleted earlier.
-     * 
+     *
      * @return a temporary topic identity
      * @throws JMSException if the session fails to create a temporary topic due
      *                 to some internal error.
@@ -1347,7 +1370,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueReceiver</CODE> object to receive messages from
      * the specified queue.
-     * 
+     *
      * @param queue the <CODE>Queue</CODE> to access
      * @return
      * @throws JMSException if the session fails to create a receiver due to
@@ -1363,7 +1386,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueReceiver</CODE> object to receive messages from
      * the specified queue using a message selector.
-     * 
+     *
      * @param queue the <CODE>Queue</CODE> to access
      * @param messageSelector only messages with properties matching the message
      *                selector expression are delivered. A value of null or an
@@ -1391,7 +1414,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     /**
      * Creates a <CODE>QueueSender</CODE> object to send messages to the
      * specified queue.
-     * 
+     *
      * @param queue the <CODE>Queue</CODE> to access, or null if this is an
      *                unidentified producer
      * @return QueueSender
@@ -1422,7 +1445,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * The subscriber <CODE>NoLocal</CODE> attribute allows a subscriber to
      * inhibit the delivery of messages published by its own connection. The
      * default value for this attribute is false.
-     * 
+     *
      * @param topic the <CODE>Topic</CODE> to subscribe to
      * @return TopicSubscriber
      * @throws JMSException if the session fails to create a subscriber due to
@@ -1453,7 +1476,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * The subscriber <CODE>NoLocal</CODE> attribute allows a subscriber to
      * inhibit the delivery of messages published by its own connection. The
      * default value for this attribute is false.
-     * 
+     *
      * @param topic the <CODE>Topic</CODE> to subscribe to
      * @param messageSelector only messages with properties matching the message
      *                selector expression are delivered. A value of null or an
@@ -1487,7 +1510,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * on a topic. Each time a client creates a <CODE>TopicPublisher</CODE> on
      * a topic, it defines a new sequence of messages that have no ordering
      * relationship with the messages it has previously sent.
-     * 
+     *
      * @param topic the <CODE>Topic</CODE> to publish to, or null if this is
      *                an unidentified producer
      * @return TopicPublisher
@@ -1517,7 +1540,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * <CODE>TopicSubscriber</CODE> for the subscription, or while a consumed
      * message is part of a pending transaction or has not been acknowledged in
      * the session.
-     * 
+     *
      * @param name the name used to identify this subscription
      * @throws JMSException if the session fails to unsubscribe to the durable
      *                 subscription due to some internal error.
@@ -1558,7 +1581,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * group, thereby acknowledging all messages consumed by the session.)
      * <P>
      * Messages that have been received but not acknowledged may be redelivered.
-     * 
+     *
      * @throws JMSException if the JMS provider fails to acknowledge the
      *                 messages due to some internal error.
      * @throws javax.jms.IllegalStateException if this method is called on a
@@ -1574,7 +1597,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Add a message consumer.
-     * 
+     *
      * @param consumer - message consumer.
      * @throws JMSException
      */
@@ -1588,7 +1611,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Remove the message consumer.
-     * 
+     *
      * @param consumer - consumer to be removed.
      * @throws JMSException
      */
@@ -1603,7 +1626,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Adds a message producer.
-     * 
+     *
      * @param producer - message producer to be added.
      * @throws JMSException
      */
@@ -1614,7 +1637,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Removes a message producer.
-     * 
+     *
      * @param producer - message producer to be removed.
      * @throws JMSException
      */
@@ -1625,7 +1648,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Start this Session.
-     * 
+     *
      * @throws JMSException
      */
     protected void start() throws JMSException {
@@ -1639,7 +1662,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Stops this session.
-     * 
+     *
      * @throws JMSException
      */
     protected void stop() throws JMSException {
@@ -1655,7 +1678,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Returns the session id.
-     * 
+     *
      * @return value - session id.
      */
     protected SessionId getSessionId() {
@@ -1678,7 +1701,8 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Sends the message for dispatch by the broker.
-     * 
+     *
+     *
      * @param producer - message producer.
      * @param destination - message destination.
      * @param message - message to be sent.
@@ -1686,10 +1710,11 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
      * @param priority - message priority.
      * @param timeToLive - message expiration.
      * @param producerWindow
+     * @param onComplete
      * @throws JMSException
      */
     protected void send(ActiveMQMessageProducer producer, ActiveMQDestination destination, Message message, int deliveryMode, int priority, long timeToLive,
-                        MemoryUsage producerWindow, int sendTimeout) throws JMSException {
+                        MemoryUsage producerWindow, int sendTimeout, AsyncCallback onComplete) throws JMSException {
 
         checkClosed();
         if (destination.isTemporary() && connection.isDeleted(destination)) {
@@ -1740,7 +1765,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
             if (LOG.isTraceEnabled()) {
                 LOG.trace(getSessionId() + " sending message: " + msg);
             }
-            if (sendTimeout <= 0 && !msg.isResponseRequired() && !connection.isAlwaysSyncSend() && (!msg.isPersistent() || connection.isUseAsyncSend() || txid != null)) {
+            if (onComplete==null && sendTimeout <= 0 && !msg.isResponseRequired() && !connection.isAlwaysSyncSend() && (!msg.isPersistent() || connection.isUseAsyncSend() || txid != null)) {
                 this.connection.asyncSendPacket(msg);
                 if (producerWindow != null) {
                     // Since we defer lots of the marshaling till we hit the
@@ -1754,10 +1779,10 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
                     producerWindow.increaseUsage(size);
                 }
             } else {
-                if (sendTimeout > 0) {
+                if (sendTimeout > 0 && onComplete==null) {
                     this.connection.syncSendPacket(msg,sendTimeout);
                 }else {
-                    this.connection.syncSendPacket(msg);
+                    this.connection.syncSendPacket(msg, onComplete);
                 }
             }
 
@@ -1766,7 +1791,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Send TransactionInfo to indicate transaction has started
-     * 
+     *
      * @throws JMSException if some internal error occurs
      */
     protected void doStartTransaction() throws JMSException {
@@ -1777,7 +1802,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Checks whether the session has unconsumed messages.
-     * 
+     *
      * @return true - if there are unconsumed messages.
      */
     public boolean hasUncomsumedMessages() {
@@ -1786,7 +1811,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Checks whether the session uses transactions.
-     * 
+     *
      * @return true - if the session uses transactions.
      */
     public boolean isTransacted() {
@@ -1795,7 +1820,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Checks whether the session used client acknowledgment.
-     * 
+     *
      * @return true - if the session uses client acknowledgment.
      */
     protected boolean isClientAcknowledge() {
@@ -1804,7 +1829,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Checks whether the session used auto acknowledgment.
-     * 
+     *
      * @return true - if the session uses client acknowledgment.
      */
     public boolean isAutoAcknowledge() {
@@ -1813,20 +1838,20 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Checks whether the session used dup ok acknowledgment.
-     * 
+     *
      * @return true - if the session uses client acknowledgment.
      */
     public boolean isDupsOkAcknowledge() {
         return acknowledgementMode == Session.DUPS_OK_ACKNOWLEDGE;
     }
-    
+
     public boolean isIndividualAcknowledge(){
-    	return acknowledgementMode == ActiveMQSession.INDIVIDUAL_ACKNOWLEDGE;
+        return acknowledgementMode == ActiveMQSession.INDIVIDUAL_ACKNOWLEDGE;
     }
 
     /**
      * Returns the message delivery listener.
-     * 
+     *
      * @return deliveryListener - message delivery listener.
      */
     public DeliveryListener getDeliveryListener() {
@@ -1835,7 +1860,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Sets the message delivery listener.
-     * 
+     *
      * @param deliveryListener - message delivery listener.
      */
     public void setDeliveryListener(DeliveryListener deliveryListener) {
@@ -1844,7 +1869,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Returns the SessionInfo bean.
-     * 
+     *
      * @return info - SessionInfo bean.
      * @throws JMSException
      */
@@ -1855,7 +1880,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Send the asynchronus command.
-     * 
+     *
      * @param command - command to be executed.
      * @throws JMSException
      */
@@ -1865,7 +1890,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
 
     /**
      * Send the synchronus command.
-     * 
+     *
      * @param command - command to be executed.
      * @return Response
      * @throws JMSException
@@ -1948,7 +1973,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
         this.blobTransferPolicy = blobTransferPolicy;
     }
 
-    public List getUnconsumedMessages() {
+    public List<MessageDispatch> getUnconsumedMessages() {
         return executor.getUnconsumedMessages();
     }
 
@@ -2010,7 +2035,7 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
         }
         return false;
     }
-    
+
     /**
      * highest sequence id of the last message delivered by this session.
      * Passed to the broker in the close command, maintained by dispose()
@@ -2019,11 +2044,11 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
     public long getLastDeliveredSequenceId() {
         return lastDeliveredSequenceId;
     }
-    
+
     protected void sendAck(MessageAck ack) throws JMSException {
         sendAck(ack,false);
     }
-    
+
     protected void sendAck(MessageAck ack, boolean lazy) throws JMSException {
         if (lazy || connection.isSendAcksAsync() || getTransacted()) {
             asyncSendPacket(ack);
@@ -2031,11 +2056,11 @@ public class ActiveMQSession implements Session, QueueSession, TopicSession, Sta
             syncSendPacket(ack);
         }
     }
-    
-    protected Scheduler getScheduler() {
-        return this.scheduler;
+
+    protected Scheduler getScheduler() throws JMSException {
+        return this.connection.getScheduler();
     }
-    
+
     protected ThreadPoolExecutor getConnectionExecutor() {
         return this.connectionExecutor;
     }
