@@ -40,8 +40,6 @@ import org.slf4j.LoggerFactory;
 /**
  * This broker filter handles tracking the state of the broker for purposes of
  * publishing advisory messages to advisory consumers.
- * 
- * 
  */
 public class AdvisoryBroker extends BrokerFilter {
 
@@ -52,10 +50,11 @@ public class AdvisoryBroker extends BrokerFilter {
     protected final ConcurrentHashMap<ConsumerId, ConsumerInfo> consumers = new ConcurrentHashMap<ConsumerId, ConsumerInfo>();
     protected final ConcurrentHashMap<ProducerId, ProducerInfo> producers = new ConcurrentHashMap<ProducerId, ProducerInfo>();
     protected final ConcurrentHashMap<ActiveMQDestination, DestinationInfo> destinations = new ConcurrentHashMap<ActiveMQDestination, DestinationInfo>();
+    protected final ConcurrentHashMap<BrokerInfo, ActiveMQMessage> networkBridges = new ConcurrentHashMap<BrokerInfo, ActiveMQMessage>();
     protected final ProducerId advisoryProducerId = new ProducerId();
-    
+
     private final LongSequenceGenerator messageIdGenerator = new LongSequenceGenerator();
-    
+
     public AdvisoryBroker(Broker next) {
         super(next);
         advisoryProducerId.setConnectionId(ID_GENERATOR.generateId());
@@ -77,7 +76,7 @@ public class AdvisoryBroker extends BrokerFilter {
     @Override
     public Subscription addConsumer(ConnectionContext context, ConsumerInfo info) throws Exception {
         Subscription answer = super.addConsumer(context, info);
-        
+
         // Don't advise advisory topics.
         if (!AdvisorySupport.isAdvisoryTopic(info.getDestination())) {
             ActiveMQTopic topic = AdvisorySupport.getConsumerAdvisoryTopic(info.getDestination());
@@ -95,15 +94,23 @@ public class AdvisoryBroker extends BrokerFilter {
                 }
             }
 
-            // We need to replay all the previously collected destination
-            // objects
-            // for this newly added consumer.
-            if (AdvisorySupport.isDestinationAdvisoryTopic(info.getDestination())) {
-                // Replay the destinations.
-                for (Iterator<DestinationInfo> iter = destinations.values().iterator(); iter.hasNext();) {
-                    DestinationInfo value = iter.next();
-                    ActiveMQTopic topic = AdvisorySupport.getDestinationAdvisoryTopic(value.getDestination());
-                    fireAdvisory(context, topic, value, info.getConsumerId());
+            // We check here whether the Destination is Temporary Destination specific or not since we
+            // can avoid sending advisory messages to the consumer if it only wants Temporary Destination
+            // notifications.  If its not just temporary destination related destinations then we have
+            // to send them all, a composite destination could want both.
+            if (AdvisorySupport.isTempDestinationAdvisoryTopic(info.getDestination())) {
+                // Replay the temporary destinations.
+                for (DestinationInfo destination : destinations.values()) {
+                    if (destination.getDestination().isTemporary()) {
+                        ActiveMQTopic topic = AdvisorySupport.getDestinationAdvisoryTopic(destination.getDestination());
+                        fireAdvisory(context, topic, destination, info.getConsumerId());
+                    }
+                }
+            } else if (AdvisorySupport.isDestinationAdvisoryTopic(info.getDestination())) {
+                // Replay all the destinations.
+                for (DestinationInfo destination : destinations.values()) {
+                    ActiveMQTopic topic = AdvisorySupport.getDestinationAdvisoryTopic(destination.getDestination());
+                    fireAdvisory(context, topic, destination, info.getConsumerId());
                 }
             }
 
@@ -124,6 +131,15 @@ public class AdvisoryBroker extends BrokerFilter {
                     fireConsumerAdvisory(context,value.getDestination(), topic, value, info.getConsumerId());
                 }
             }
+
+            // Replay network bridges
+            if (AdvisorySupport.isNetworkBridgeAdvisoryTopic(info.getDestination())) {
+                for (Iterator<BrokerInfo> iter = networkBridges.keySet().iterator(); iter.hasNext();) {
+                    BrokerInfo key = iter.next();
+                    ActiveMQTopic topic = AdvisorySupport.getNetworkBridgeAdvisoryTopic();
+                    fireAdvisory(context, topic, key, null, networkBridges.get(key));
+                }
+            }
         }
         return answer;
     }
@@ -173,42 +189,41 @@ public class AdvisoryBroker extends BrokerFilter {
         super.removeDestination(context, destination, timeout);
         DestinationInfo info = destinations.remove(destination);
         if (info != null) {
+            // ensure we don't modify (and loose/overwrite) an in-flight add advisory, so duplicate
+            info = info.copy();
             info.setDestination(destination);
             info.setOperationType(DestinationInfo.REMOVE_OPERATION_TYPE);
             ActiveMQTopic topic = AdvisorySupport.getDestinationAdvisoryTopic(destination);
             fireAdvisory(context, topic, info);
-            try {
-                next.removeDestination(context, AdvisorySupport.getConsumerAdvisoryTopic(info.getDestination()), -1);
-            } catch (Exception expectedIfDestinationDidNotExistYet) {                
-            }
-            try {
-                next.removeDestination(context, AdvisorySupport.getProducerAdvisoryTopic(info.getDestination()), -1);
-            } catch (Exception expectedIfDestinationDidNotExistYet) {
+            ActiveMQTopic[] advisoryDestinations = AdvisorySupport.getAllDestinationAdvisoryTopics(destination);
+            for(ActiveMQTopic advisoryDestination : advisoryDestinations) {
+                try {
+                    next.removeDestination(context, advisoryDestination, -1);
+                } catch (Exception expectedIfDestinationDidNotExistYet) {
+                }
             }
         }
-
     }
 
     @Override
     public void removeDestinationInfo(ConnectionContext context, DestinationInfo destInfo) throws Exception {
-        super.removeDestinationInfo(context, destInfo);   
+        super.removeDestinationInfo(context, destInfo);
         DestinationInfo info = destinations.remove(destInfo.getDestination());
         if (info != null) {
+            // ensure we don't modify (and loose/overwrite) an in-flight add advisory, so duplicate
+            info = info.copy();
             info.setDestination(destInfo.getDestination());
             info.setOperationType(DestinationInfo.REMOVE_OPERATION_TYPE);
             ActiveMQTopic topic = AdvisorySupport.getDestinationAdvisoryTopic(destInfo.getDestination());
             fireAdvisory(context, topic, info);
-            try {
-                next.removeDestination(context, AdvisorySupport.getConsumerAdvisoryTopic(info.getDestination()), -1);
-            } catch (Exception expectedIfDestinationDidNotExistYet) {
-            }
-            try {
-                next.removeDestination(context, AdvisorySupport.getProducerAdvisoryTopic(info.getDestination()), -1);
-            
-            } catch (Exception expectedIfDestinationDidNotExistYet) {
+            ActiveMQTopic[] advisoryDestinations = AdvisorySupport.getAllDestinationAdvisoryTopics(destInfo.getDestination());
+            for(ActiveMQTopic advisoryDestination : advisoryDestinations) {
+                try {
+                    next.removeDestination(context, advisoryDestination, -1);
+                } catch (Exception expectedIfDestinationDidNotExistYet) {
+                }
             }
         }
-
     }
 
     @Override
@@ -230,7 +245,7 @@ public class AdvisoryBroker extends BrokerFilter {
             ActiveMQTopic topic = AdvisorySupport.getConsumerAdvisoryTopic(dest);
             consumers.remove(info.getConsumerId());
             if (!dest.isTemporary() || destinations.containsKey(dest)) {
-            	fireConsumerAdvisory(context,dest, topic, info.createRemoveCommand());
+                fireConsumerAdvisory(context,dest, topic, info.createRemoveCommand());
             }
         }
     }
@@ -266,7 +281,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("expired", e);
         }
     }
-    
+
     @Override
     public void messageConsumed(ConnectionContext context, MessageReference messageReference) {
         super.messageConsumed(context, messageReference);
@@ -281,7 +296,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("consumed", e);
         }
     }
-    
+
     @Override
     public void messageDelivered(ConnectionContext context, MessageReference messageReference) {
         super.messageDelivered(context, messageReference);
@@ -296,7 +311,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("delivered", e);
         }
     }
-    
+
     @Override
     public void messageDiscarded(ConnectionContext context, Subscription sub, MessageReference messageReference) {
         super.messageDiscarded(context, sub, messageReference);
@@ -316,7 +331,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("discarded", e);
         }
     }
-    
+
     @Override
     public void slowConsumer(ConnectionContext context, Destination destination,Subscription subs) {
         super.slowConsumer(context, destination,subs);
@@ -329,7 +344,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("slow consumer", e);
         }
     }
-    
+
     @Override
     public void fastProducer(ConnectionContext context,ProducerInfo producerInfo) {
         super.fastProducer(context, producerInfo);
@@ -342,7 +357,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("fast producer", e);
         }
     }
-    
+
     @Override
     public void isFull(ConnectionContext context, Destination destination, Usage usage) {
         super.isFull(context, destination, usage);
@@ -359,13 +374,13 @@ public class AdvisoryBroker extends BrokerFilter {
             }
         }
     }
-    
+
     @Override
-    public void nowMasterBroker() {   
+    public void nowMasterBroker() {
         super.nowMasterBroker();
         try {
             ActiveMQTopic topic = AdvisorySupport.getMasterBrokerAdvisoryTopic();
-            ActiveMQMessage advisoryMessage = new ActiveMQMessage();                       
+            ActiveMQMessage advisoryMessage = new ActiveMQMessage();
             ConnectionContext context = new ConnectionContext();
             context.setSecurityContext(SecurityContext.BROKER_SECURITY_CONTEXT);
             context.setBroker(getBrokerService().getBroker());
@@ -374,7 +389,7 @@ public class AdvisoryBroker extends BrokerFilter {
             handleFireFailure("now master broker", e);
         }
     }
-    
+
     @Override
     public void sendToDeadLetterQueue(ConnectionContext context, MessageReference messageReference,
                                       Subscription subscription){
@@ -388,16 +403,18 @@ public class AdvisoryBroker extends BrokerFilter {
             }
         } catch (Exception e) {
             handleFireFailure("add to DLQ", e);
-        } 
+        }
     }
 
     @Override
-    public void networkBridgeStarted(BrokerInfo brokerInfo, boolean createdByDuplex) {
+    public void networkBridgeStarted(BrokerInfo brokerInfo, boolean createdByDuplex, String remoteIp) {
         try {
          if (brokerInfo != null) {
              ActiveMQMessage advisoryMessage = new ActiveMQMessage();
              advisoryMessage.setBooleanProperty("started", true);
              advisoryMessage.setBooleanProperty("createdByDuplex", createdByDuplex);
+             advisoryMessage.setStringProperty("remoteIp", remoteIp);
+             networkBridges.putIfAbsent(brokerInfo, advisoryMessage);
 
              ActiveMQTopic topic = AdvisorySupport.getNetworkBridgeAdvisoryTopic();
 
@@ -417,6 +434,7 @@ public class AdvisoryBroker extends BrokerFilter {
          if (brokerInfo != null) {
              ActiveMQMessage advisoryMessage = new ActiveMQMessage();
              advisoryMessage.setBooleanProperty("started", false);
+             networkBridges.remove(brokerInfo);
 
              ActiveMQTopic topic = AdvisorySupport.getNetworkBridgeAdvisoryTopic();
 
@@ -460,7 +478,7 @@ public class AdvisoryBroker extends BrokerFilter {
             }
         }
         advisoryMessage.setIntProperty(AdvisorySupport.MSG_PROPERTY_CONSUMER_COUNT, count);
-        
+
         fireAdvisory(context, topic, command, targetConsumerId, advisoryMessage);
     }
 
@@ -489,13 +507,13 @@ public class AdvisoryBroker extends BrokerFilter {
             advisoryMessage.setStringProperty(AdvisorySupport.MSG_PROPERTY_ORIGIN_BROKER_NAME, getBrokerName());
             String id = getBrokerId() != null ? getBrokerId().getValue() : "NOT_SET";
             advisoryMessage.setStringProperty(AdvisorySupport.MSG_PROPERTY_ORIGIN_BROKER_ID, id);
-            
+
             String url = getBrokerService().getVmConnectorURI().toString();
             if (getBrokerService().getDefaultSocketURIString() != null) {
                 url = getBrokerService().getDefaultSocketURIString();
             }
             advisoryMessage.setStringProperty(AdvisorySupport.MSG_PROPERTY_ORIGIN_BROKER_URL, url);
-            
+
             //set the data structure
             advisoryMessage.setDataStructure(command);
             advisoryMessage.setPersistent(false);
