@@ -16,11 +16,14 @@
  */
 package org.apache.activemq.advisory;
 
+import java.util.ArrayList;
+
 import javax.jms.Destination;
 import javax.jms.JMSException;
 import org.apache.activemq.ActiveMQMessageTransformation;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQTopic;
+
 public final class AdvisorySupport {
     public static final String ADVISORY_TOPIC_PREFIX = "ActiveMQ.Advisory.";
     public static final ActiveMQTopic CONNECTION_ADVISORY_TOPIC = new ActiveMQTopic(ADVISORY_TOPIC_PREFIX
@@ -59,7 +62,10 @@ public final class AdvisorySupport {
     public static final String MSG_PROPERTY_MESSAGE_ID = "orignalMessageId";
     public static final String MSG_PROPERTY_CONSUMER_COUNT = "consumerCount";
     public static final String MSG_PROPERTY_DISCARDED_COUNT = "discardedCount";
-    
+
+    public static final ActiveMQTopic ALL_DESTINATIONS_COMPOSITE_ADVISORY_TOPIC = new ActiveMQTopic(
+            TOPIC_ADVISORY_TOPIC.getPhysicalName() + "," + QUEUE_ADVISORY_TOPIC.getPhysicalName() + "," +
+                    TEMP_QUEUE_ADVISORY_TOPIC.getPhysicalName() + "," + TEMP_TOPIC_ADVISORY_TOPIC.getPhysicalName());
     public static final ActiveMQTopic TEMP_DESTINATION_COMPOSITE_ADVISORY_TOPIC = new ActiveMQTopic(
             TEMP_QUEUE_ADVISORY_TOPIC.getPhysicalName() + "," + TEMP_TOPIC_ADVISORY_TOPIC.getPhysicalName());
     private static final ActiveMQTopic AGENT_TOPIC_DESTINATION = new ActiveMQTopic(AGENT_TOPIC);
@@ -71,6 +77,28 @@ public final class AdvisorySupport {
         return CONNECTION_ADVISORY_TOPIC;
     }
 
+    public static ActiveMQTopic[] getAllDestinationAdvisoryTopics(Destination destination) throws JMSException {
+        return getAllDestinationAdvisoryTopics(ActiveMQMessageTransformation.transformDestination(destination));
+    }
+
+    public static ActiveMQTopic[] getAllDestinationAdvisoryTopics(ActiveMQDestination destination) throws JMSException {
+        ArrayList<ActiveMQTopic> result = new ArrayList<ActiveMQTopic>();
+
+        result.add(getConsumerAdvisoryTopic(destination));
+        result.add(getProducerAdvisoryTopic(destination));
+        result.add(getExpiredMessageTopic(destination));
+        result.add(getNoConsumersAdvisoryTopic(destination));
+        result.add(getSlowConsumerAdvisoryTopic(destination));
+        result.add(getFastProducerAdvisoryTopic(destination));
+        result.add(getMessageDiscardedAdvisoryTopic(destination));
+        result.add(getMessageDeliveredAdvisoryTopic(destination));
+        result.add(getMessageConsumedAdvisoryTopic(destination));
+        result.add(getMessageDLQdAdvisoryTopic(destination));
+        result.add(getFullAdvisoryTopic(destination));
+
+        return result.toArray(new ActiveMQTopic[0]);
+    }
+
     public static ActiveMQTopic getConsumerAdvisoryTopic(Destination destination) throws JMSException {
         return getConsumerAdvisoryTopic(ActiveMQMessageTransformation.transformDestination(destination));
     }
@@ -120,6 +148,17 @@ public final class AdvisorySupport {
         return new ActiveMQTopic(name);
     }
 
+    public static ActiveMQTopic getNoConsumersAdvisoryTopic(Destination destination) throws JMSException {
+        return getExpiredMessageTopic(ActiveMQMessageTransformation.transformDestination(destination));
+    }
+
+    public static ActiveMQTopic getNoConsumersAdvisoryTopic(ActiveMQDestination destination) {
+        if (destination.isQueue()) {
+            return getNoQueueConsumersAdvisoryTopic(destination);
+        }
+        return getNoTopicConsumersAdvisoryTopic(destination);
+    }
+
     public static ActiveMQTopic getNoTopicConsumersAdvisoryTopic(Destination destination) throws JMSException {
         return getNoTopicConsumersAdvisoryTopic(ActiveMQMessageTransformation.transformDestination(destination));
     }
@@ -187,17 +226,13 @@ public final class AdvisorySupport {
                 + destination.getPhysicalName();
         return new ActiveMQTopic(name);
     }
-    
+
     public static ActiveMQTopic getMessageDLQdAdvisoryTopic(ActiveMQDestination destination) {
         String name = MESSAGE_DLQ_TOPIC_PREFIX + destination.getDestinationTypeAsString() + "."
                 + destination.getPhysicalName();
         return new ActiveMQTopic(name);
     }
 
-    public static ActiveMQTopic getMasterBrokerAdvisoryTopic(Destination destination) throws JMSException {
-        return getMasterBrokerAdvisoryTopic(ActiveMQMessageTransformation.transformDestination(destination));
-    }
-
     public static ActiveMQTopic getMasterBrokerAdvisoryTopic() {
         return new ActiveMQTopic(MASTER_BROKER_TOPIC_PREFIX);
     }
@@ -222,16 +257,16 @@ public final class AdvisorySupport {
 
     public static ActiveMQTopic getDestinationAdvisoryTopic(ActiveMQDestination destination) {
         switch (destination.getDestinationType()) {
-        case ActiveMQDestination.QUEUE_TYPE:
-            return QUEUE_ADVISORY_TOPIC;
-        case ActiveMQDestination.TOPIC_TYPE:
-            return TOPIC_ADVISORY_TOPIC;
-        case ActiveMQDestination.TEMP_QUEUE_TYPE:
-            return TEMP_QUEUE_ADVISORY_TOPIC;
-        case ActiveMQDestination.TEMP_TOPIC_TYPE:
-            return TEMP_TOPIC_ADVISORY_TOPIC;
-        default:
-            throw new RuntimeException("Unknown destination type: " + destination.getDestinationType());
+            case ActiveMQDestination.QUEUE_TYPE:
+                return QUEUE_ADVISORY_TOPIC;
+            case ActiveMQDestination.TOPIC_TYPE:
+                return TOPIC_ADVISORY_TOPIC;
+            case ActiveMQDestination.TEMP_QUEUE_TYPE:
+                return TEMP_QUEUE_ADVISORY_TOPIC;
+            case ActiveMQDestination.TEMP_TOPIC_TYPE:
+                return TEMP_TOPIC_ADVISORY_TOPIC;
+            default:
+                throw new RuntimeException("Unknown destination type: " + destination.getDestinationType());
         }
     }
 
@@ -239,6 +274,20 @@ public final class AdvisorySupport {
         return isDestinationAdvisoryTopic(ActiveMQMessageTransformation.transformDestination(destination));
     }
 
+    public static boolean isTempDestinationAdvisoryTopic(ActiveMQDestination destination) {
+        if (destination.isComposite()) {
+            ActiveMQDestination[] compositeDestinations = destination.getCompositeDestinations();
+            for (int i = 0; i < compositeDestinations.length; i++) {
+                if (!isTempDestinationAdvisoryTopic(compositeDestinations[i])) {
+                    return false;
+                }
+            }
+            return true;
+        } else {
+            return destination.equals(TEMP_QUEUE_ADVISORY_TOPIC) || destination.equals(TEMP_TOPIC_ADVISORY_TOPIC);
+        }
+    }
+
     public static boolean isDestinationAdvisoryTopic(ActiveMQDestination destination) {
         if (destination.isComposite()) {
             ActiveMQDestination[] compositeDestinations = destination.getCompositeDestinations();
@@ -259,17 +308,20 @@ public final class AdvisorySupport {
     }
 
     public static boolean isAdvisoryTopic(ActiveMQDestination destination) {
-        if (destination.isComposite()) {
-            ActiveMQDestination[] compositeDestinations = destination.getCompositeDestinations();
-            for (int i = 0; i < compositeDestinations.length; i++) {
-                if (isAdvisoryTopic(compositeDestinations[i])) {
-                    return true;
+        if (destination != null) {
+            if (destination.isComposite()) {
+                ActiveMQDestination[] compositeDestinations = destination.getCompositeDestinations();
+                for (int i = 0; i < compositeDestinations.length; i++) {
+                    if (isAdvisoryTopic(compositeDestinations[i])) {
+                        return true;
+                    }
                 }
+                return false;
+            } else {
+                return destination.isTopic() && destination.getPhysicalName().startsWith(ADVISORY_TOPIC_PREFIX);
             }
-            return false;
-        } else {
-            return destination.isTopic() && destination.getPhysicalName().startsWith(ADVISORY_TOPIC_PREFIX);
         }
+        return false;
     }
 
     public static boolean isConnectionAdvisoryTopic(Destination destination) throws JMSException {
@@ -452,6 +504,24 @@ public final class AdvisorySupport {
         }
     }
 
+    public static boolean isNetworkBridgeAdvisoryTopic(Destination destination) throws JMSException {
+        return isNetworkBridgeAdvisoryTopic(ActiveMQMessageTransformation.transformDestination(destination));
+    }
+
+    public static boolean isNetworkBridgeAdvisoryTopic(ActiveMQDestination destination) {
+        if (destination.isComposite()) {
+            ActiveMQDestination[] compositeDestinations = destination.getCompositeDestinations();
+            for (int i = 0; i < compositeDestinations.length; i++) {
+                if (isNetworkBridgeAdvisoryTopic(compositeDestinations[i])) {
+                    return true;
+                }
+            }
+            return false;
+        } else {
+            return destination.isTopic() && destination.getPhysicalName().startsWith(NETWORK_BRIDGE_TOPIC_PREFIX);
+        }
+    }
+
     /**
      * Returns the agent topic which is used to send commands to the broker
      */
