@@ -20,10 +20,11 @@ import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Iterator;
-import java.util.Map;
+import java.util.LinkedList;
 import java.util.StringTokenizer;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.regex.Pattern;
+
 import javax.management.ObjectName;
 import org.apache.activemq.broker.jmx.ManagedTransportConnector;
 import org.apache.activemq.broker.jmx.ManagementContext;
@@ -39,7 +40,6 @@ import org.apache.activemq.transport.TransportFactory;
 import org.apache.activemq.transport.TransportServer;
 import org.apache.activemq.transport.discovery.DiscoveryAgent;
 import org.apache.activemq.transport.discovery.DiscoveryAgentFactory;
-import org.apache.activemq.util.MDCHelper;
 import org.apache.activemq.util.ServiceStopper;
 import org.apache.activemq.util.ServiceSupport;
 import org.slf4j.Logger;
@@ -47,7 +47,7 @@ import org.slf4j.LoggerFactory;
 
 /**
  * @org.apache.xbean.XBean
- * 
+ *
  */
 public class TransportConnector implements Connector, BrokerServiceAware {
 
@@ -73,6 +73,11 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     private boolean rebalanceClusterClients;
     private boolean updateClusterClientsOnRemove = false;
     private String updateClusterFilter;
+    private boolean auditNetworkProducers = false;
+    private int maximumProducersAllowedPerConnection = Integer.MAX_VALUE;
+    private int maximumConsumersAllowedPerConnection  = Integer.MAX_VALUE;
+
+    LinkedList<String> peerBrokers = new LinkedList<String>();
 
     public TransportConnector() {
     }
@@ -118,6 +123,9 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         rc.setRebalanceClusterClients(isRebalanceClusterClients());
         rc.setUpdateClusterFilter(getUpdateClusterFilter());
         rc.setUpdateClusterClientsOnRemove(isUpdateClusterClientsOnRemove());
+        rc.setAuditNetworkProducers(isAuditNetworkProducers());
+        rc.setMaximumConsumersAllowedPerConnection(getMaximumConsumersAllowedPerConnection());
+        rc.setMaximumProducersAllowedPerConnection(getMaximumProducersAllowedPerConnection());
         return rc;
     }
 
@@ -130,7 +138,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     }
 
     /**
-     * 
+     *
      * @deprecated use the {@link #setBrokerService(BrokerService)} method
      *             instead.
      */
@@ -168,7 +176,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
      * {@link TransportServer} configured via the
      * {@link #setServer(TransportServer)} method. This value is used to lazy
      * create a {@link TransportServer} instance
-     * 
+     *
      * @param uri
      */
     public void setUri(URI uri) {
@@ -208,20 +216,19 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         brokerInfo.setBrokerId(broker.getBrokerId());
         brokerInfo.setPeerBrokerInfos(broker.getPeerBrokerInfos());
         brokerInfo.setFaultTolerantConfiguration(broker.isFaultTolerantConfiguration());
-        brokerInfo.setBrokerURL(getServer().getConnectURI().toString());
-        final Map context = MDCHelper.getCopyOfContextMap();
+        brokerInfo.setBrokerURL(broker.getBrokerService().getDefaultSocketURIString());
         getServer().setAcceptListener(new TransportAcceptListener() {
             public void onAccept(final Transport transport) {
                 try {
                     DefaultThreadPools.getDefaultTaskRunnerFactory().execute(new Runnable() {
                         public void run() {
-                            MDCHelper.setContextMap(context);
                             try {
                                 Connection connection = createConnection(transport);
                                 connection.start();
                             } catch (Exception e) {
+                                String remoteHost = transport.getRemoteAddress();
                                 ServiceSupport.dispose(transport);
-                                onAcceptError(e);
+                                onAcceptError(e, remoteHost);
                             }
                         }
                     });
@@ -259,8 +266,11 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     }
 
     public String getPublishableConnectString() throws Exception {
+        return getPublishableConnectString(getConnectUri());
+    }
+
+    public String getPublishableConnectString(URI theConnectURI) throws Exception {
         String publishableConnectString = null;
-        URI theConnectURI = getConnectUri();
         if (theConnectURI != null) {
             publishableConnectString = theConnectURI.toString();
             // strip off server side query parameters which may not be compatible to
@@ -283,7 +293,6 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         }
         if (server != null) {
             ss.stop(server);
-            server = null;
         }
         if (this.statusDector != null) {
             this.statusDector.stop();
@@ -293,6 +302,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
             TransportConnection c = iter.next();
             ss.stop(c);
         }
+        server = null;
         ss.throwFirstException();
         LOG.info("Connector " + getName() + " Stopped");
     }
@@ -328,7 +338,13 @@ public class TransportConnector implements Connector, BrokerServiceAware {
 
     protected DiscoveryAgent createDiscoveryAgent() throws IOException {
         if (discoveryUri != null) {
-            return DiscoveryAgentFactory.createDiscoveryAgent(discoveryUri);
+            DiscoveryAgent agent = DiscoveryAgentFactory.createDiscoveryAgent(discoveryUri);
+
+            if( agent!=null && agent instanceof BrokerServiceAware ) {
+                ((BrokerServiceAware)agent).setBrokerService(brokerService);
+            }
+
+            return agent;
         }
         return null;
     }
@@ -392,41 +408,60 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     protected ConnectionControl getConnectionControl() {
         boolean rebalance = isRebalanceClusterClients();
         String connectedBrokers = "";
-        String self = "";
+        String separator = "";
 
         if (isUpdateClusterClients()) {
-            if (brokerService.getDefaultSocketURIString() != null) {
-                self += brokerService.getDefaultSocketURIString();
-                self += ",";
-            }
-            if (rebalance == false) {
-                connectedBrokers += self;
-            }
-            if (this.broker.getPeerBrokerInfos() != null) {
-                for (BrokerInfo info : this.broker.getPeerBrokerInfos()) {
-                    if (isMatchesClusterFilter(info.getBrokerName())) {
-                        connectedBrokers += info.getBrokerURL();
-                        connectedBrokers += ",";
-                    }
+            synchronized (peerBrokers) {
+                for (String uri : getPeerBrokers()) {
+                    connectedBrokers += separator + uri;
+                    separator = ",";
+                }
+
+                if (rebalance) {
+                    String shuffle = getPeerBrokers().removeFirst();
+                    getPeerBrokers().addLast(shuffle);
                 }
             }
-            if (rebalance) {
-                connectedBrokers += self;
-            }
         }
-
         ConnectionControl control = new ConnectionControl();
         control.setConnectedBrokers(connectedBrokers);
         control.setRebalanceConnection(rebalance);
         return control;
 
     }
+    
+    public void addPeerBroker(BrokerInfo info) {
+        if (isMatchesClusterFilter(info.getBrokerName())) {
+            synchronized (peerBrokers) {
+                getPeerBrokers().addLast(info.getBrokerURL());
+            }
+        }
+    }
+    
+    public void removePeerBroker(BrokerInfo info) {
+        synchronized (peerBrokers) {
+            getPeerBrokers().remove(info.getBrokerURL());
+        }
+    }
+
+    public LinkedList<String> getPeerBrokers() {
+        synchronized (peerBrokers) {
+            if (peerBrokers.isEmpty()) {
+                peerBrokers.add(brokerService.getDefaultSocketURIString());
+            }
+            return peerBrokers;
+        }
+    }
 
     public void updateClientClusterInfo() {
+
         if (isRebalanceClusterClients() || isUpdateClusterClients()) {
             ConnectionControl control = getConnectionControl();
             for (Connection c : this.connections) {
                 c.updateClient(control);
+                if (isRebalanceClusterClients()) {
+                    control = getConnectionControl();
+                }
             }
         }
     }
@@ -522,7 +557,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     public void setRebalanceClusterClients(boolean rebalanceClusterClients) {
         this.rebalanceClusterClients = rebalanceClusterClients;
     }
- 
+
     /**
      * @return the updateClusterClientsOnRemove
      */
@@ -536,7 +571,7 @@ public class TransportConnector implements Connector, BrokerServiceAware {
     public void setUpdateClusterClientsOnRemove(boolean updateClusterClientsOnRemove) {
         this.updateClusterClientsOnRemove = updateClusterClientsOnRemove;
     }
-    
+
     /**
      * @return the updateClusterFilter
      */
@@ -552,4 +587,37 @@ public class TransportConnector implements Connector, BrokerServiceAware {
         this.updateClusterFilter = updateClusterFilter;
     }
 
+    public int connectionCount() {
+        return connections.size();
+    }
+
+    public boolean isAuditNetworkProducers() {
+        return auditNetworkProducers;
+    }
+
+    /**
+     * Enable a producer audit on network connections, Traps the case of a missing send reply and resend.
+     * Note: does not work with conduit=false, networked composite destinations or networked virtual topics
+     * @param auditNetworkProducers
+     */
+    public void setAuditNetworkProducers(boolean auditNetworkProducers) {
+        this.auditNetworkProducers = auditNetworkProducers;
+    }
+
+    public int getMaximumProducersAllowedPerConnection() {
+        return maximumProducersAllowedPerConnection;
+    }
+
+    public void setMaximumProducersAllowedPerConnection(int maximumProducersAllowedPerConnection) {
+        this.maximumProducersAllowedPerConnection = maximumProducersAllowedPerConnection;
+    }
+
+    public int getMaximumConsumersAllowedPerConnection() {
+        return maximumConsumersAllowedPerConnection;
+    }
+
+    public void setMaximumConsumersAllowedPerConnection(int maximumConsumersAllowedPerConnection) {
+        this.maximumConsumersAllowedPerConnection = maximumConsumersAllowedPerConnection;
+    }
+
 }
