@@ -64,7 +64,9 @@ public class ManagementContext implements Service {
     private JMXConnectorServer connectorServer;
     private ObjectName namingServiceObjectName;
     private Registry registry;
+    private ServerSocket registrySocket;
     private final List<ObjectName> registeredMBeanNames = new CopyOnWriteArrayList<ObjectName>();
+    private boolean allowRemoteAddressInMBeanNames = true;
 
     public ManagementContext() {
         this(null);
@@ -145,6 +147,13 @@ public class ManagementContext implements Service {
                 }
             }
             beanServer = null;
+            if(registrySocket!=null) {
+                try {
+                    registrySocket.close();
+                } catch (IOException e) {
+                }
+                registrySocket = null;
+            }
         }
     }
 
@@ -301,7 +310,7 @@ public class ManagementContext implements Service {
         return result;
     }
     
-    public Set queryNames(ObjectName name, QueryExp query) throws Exception{
+    public Set<ObjectName>  queryNames(ObjectName name, QueryExp query) throws Exception{
         return getMBeanServer().queryNames(name, query);
     }
     
@@ -418,12 +427,13 @@ public class ManagementContext implements Service {
                 registry = LocateRegistry.createRegistry(connectorPort, null, new RMIServerSocketFactory() {
                     public ServerSocket createServerSocket(int port)
                             throws IOException {
-                        ServerSocket result = new ServerSocket(port);
-                        result.setReuseAddress(true);
-                        return result;
+                        registrySocket = new ServerSocket(port);
+                        registrySocket.setReuseAddress(true);
+                        return registrySocket;
                     }});
             }
             namingServiceObjectName = ObjectName.getInstance("naming:type=rmiregistry");
+
             // Do not use the createMBean as the mx4j jar may not be in the
             // same class loader than the server
             Class cl = Class.forName("mx4j.tools.naming.NamingService");
@@ -486,6 +496,9 @@ public class ManagementContext implements Service {
         return createConnector;
     }
 
+    /**
+     * @org.apache.xbean.Property propertyEditor="org.apache.activemq.util.BooleanEditor"
+     */
     public void setCreateConnector(boolean createConnector) {
         this.createConnector = createConnector;
     }
@@ -513,4 +526,12 @@ public class ManagementContext implements Service {
     public void setEnvironment(Map environment) {
         this.environment = environment;
     }
+
+    public boolean isAllowRemoteAddressInMBeanNames() {
+        return allowRemoteAddressInMBeanNames;
+    }
+
+    public void setAllowRemoteAddressInMBeanNames(boolean allowRemoteAddressInMBeanNames) {
+        this.allowRemoteAddressInMBeanNames = allowRemoteAddressInMBeanNames;
+    }
 }
