@@ -23,15 +23,19 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
+
 import javax.jms.JMSException;
 import org.apache.activemq.broker.ConnectionContext;
 import org.apache.activemq.broker.ConsumerBrokerExchange;
 import org.apache.activemq.broker.DestinationAlreadyExistsException;
 import org.apache.activemq.broker.ProducerBrokerExchange;
+import org.apache.activemq.broker.TransportConnection;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ConsumerControl;
 import org.apache.activemq.command.ConsumerId;
 import org.apache.activemq.command.ConsumerInfo;
+import org.apache.activemq.command.DestinationInfo;
 import org.apache.activemq.command.Message;
 import org.apache.activemq.command.MessageAck;
 import org.apache.activemq.command.MessageDispatchNotification;
@@ -48,7 +52,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * 
+ *
  */
 public abstract class AbstractRegion implements Region {
 
@@ -63,7 +67,7 @@ public abstract class AbstractRegion implements Region {
     protected final RegionBroker broker;
     protected boolean autoCreateDestinations = true;
     protected final TaskRunnerFactory taskRunnerFactory;
-    protected final Object destinationsMutex = new Object();
+    protected final ReentrantReadWriteLock destinationsLock = new ReentrantReadWriteLock();
     protected final Map<ConsumerId, Object> consumerChangeMutexMap = new HashMap<ConsumerId, Object>();
     protected boolean started;
 
@@ -76,7 +80,7 @@ public abstract class AbstractRegion implements Region {
         this.destinationStatistics = destinationStatistics;
         this.usageManager = memoryManager;
         this.taskRunnerFactory = taskRunnerFactory;
-        if (broker == null) {
+        if (destinationFactory == null) {
             throw new IllegalArgumentException("null destinationFactory");
         }
         this.destinationFactory = destinationFactory;
@@ -94,32 +98,42 @@ public abstract class AbstractRegion implements Region {
             context.setSecurityContext(SecurityContext.BROKER_SECURITY_CONTEXT);
             context.getBroker().addDestination(context, dest, false);
         }
-        synchronized (destinationsMutex) {
+        destinationsLock.readLock().lock();
+        try{
             for (Iterator<Destination> i = destinations.values().iterator(); i.hasNext();) {
                 Destination dest = i.next();
                 dest.start();
             }
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
     public void stop() throws Exception {
         started = false;
-        synchronized (destinationsMutex) {
+        destinationsLock.readLock().lock();
+        try{
             for (Iterator<Destination> i = destinations.values().iterator(); i.hasNext();) {
                 Destination dest = i.next();
                 dest.stop();
             }
+        } finally {
+            destinationsLock.readLock().unlock();
         }
         destinations.clear();
     }
 
     public Destination addDestination(ConnectionContext context, ActiveMQDestination destination,
             boolean createIfTemporary) throws Exception {
-        LOG.debug(broker.getBrokerName() + " adding destination: " + destination);
-        synchronized (destinationsMutex) {
+
+        destinationsLock.writeLock().lock();
+        try {
             Destination dest = destinations.get(destination);
             if (dest == null) {
                 if (destination.isTemporary() == false || createIfTemporary) {
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug(broker.getBrokerName() + " adding destination: " + destination);
+                    }
                     dest = createDestination(context, destination);
                     // intercept if there is a valid interceptor defined
                     DestinationInterceptor destinationInterceptor = broker.getDestinationInterceptor();
@@ -136,6 +150,8 @@ public abstract class AbstractRegion implements Region {
                 }
             }
             return dest;
+        } finally {
+            destinationsLock.writeLock().unlock();
         }
     }
 
@@ -181,9 +197,12 @@ public abstract class AbstractRegion implements Region {
             // dropping the subscription.
         }
 
-        LOG.debug("Removing destination: " + destination);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug(broker.getBrokerName() + " removing destination: " + destination);
+        }
 
-        synchronized (destinationsMutex) {
+        destinationsLock.writeLock().lock();
+        try {
             Destination dest = destinations.remove(destination);
             if (dest != null) {
                 // timeout<0 or we timed out, we now force any remaining
@@ -202,31 +221,45 @@ public abstract class AbstractRegion implements Region {
                 }
 
             } else {
-                LOG.debug("Destination doesn't exist: " + dest);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Cannot remove a destination that doesn't exist: " + destination);
+                }
             }
+        } finally {
+            destinationsLock.writeLock().unlock();
         }
     }
 
     /**
      * Provide an exact or wildcard lookup of destinations in the region
-     * 
+     *
      * @return a set of matching destination objects.
      */
+    @SuppressWarnings("unchecked")
     public Set<Destination> getDestinations(ActiveMQDestination destination) {
-        synchronized (destinationsMutex) {
+        destinationsLock.readLock().lock();
+        try{
             return destinationMap.get(destination);
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
     public Map<ActiveMQDestination, Destination> getDestinationMap() {
-        synchronized (destinationsMutex) {
-            return new HashMap<ActiveMQDestination, Destination>(destinations);
+        destinationsLock.readLock().lock();
+        try{
+            return destinations;
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
+    @SuppressWarnings("unchecked")
     public Subscription addConsumer(ConnectionContext context, ConsumerInfo info) throws Exception {
-        LOG.debug(broker.getBrokerName() + " adding consumer: " + info.getConsumerId() + " for destination: "
-                + info.getDestination());
+        if (LOG.isDebugEnabled()) {
+            LOG.debug(broker.getBrokerName() + " adding consumer: " + info.getConsumerId() + " for destination: "
+                    + info.getDestination());
+        }
         ActiveMQDestination destination = info.getDestination();
         if (destination != null && !destination.isPattern() && !destination.isComposite()) {
             // lets auto-create the destination
@@ -244,8 +277,7 @@ public abstract class AbstractRegion implements Region {
         synchronized (addGuard) {
             Subscription o = subscriptions.get(info.getConsumerId());
             if (o != null) {
-                LOG
-                        .warn("A duplicate subscription was detected. Clients may be misbehaving. Later warnings you may see about subscription removal are a consequence of this.");
+                LOG.warn("A duplicate subscription was detected. Clients may be misbehaving. Later warnings you may see about subscription removal are a consequence of this.");
                 return o;
             }
 
@@ -279,11 +311,13 @@ public abstract class AbstractRegion implements Region {
             // Add the subscription to all the matching queues.
             // But copy the matches first - to prevent deadlocks
             List<Destination> addList = new ArrayList<Destination>();
-            synchronized (destinationsMutex) {
-                for (Iterator iter = destinationMap.get(info.getDestination()).iterator(); iter.hasNext();) {
-                    Destination dest = (Destination) iter.next();
+            destinationsLock.readLock().lock();
+            try {
+                for (Destination dest : (Set<Destination>) destinationMap.get(info.getDestination())) {
                     addList.add(dest);
                 }
+            } finally {
+                destinationsLock.readLock().unlock();
             }
 
             for (Destination dest : addList) {
@@ -300,9 +334,10 @@ public abstract class AbstractRegion implements Region {
 
     /**
      * Get all the Destinations that are in storage
-     * 
+     *
      * @return Set of all stored destinations
      */
+    @SuppressWarnings("rawtypes")
     public Set getDurableDestinations() {
         return destinationFactory.getDestinations();
     }
@@ -312,15 +347,21 @@ public abstract class AbstractRegion implements Region {
      */
     protected Set<ActiveMQDestination> getInactiveDestinations() {
         Set<ActiveMQDestination> inactiveDests = destinationFactory.getDestinations();
-        synchronized (destinationsMutex) {
+        destinationsLock.readLock().lock();
+        try {
             inactiveDests.removeAll(destinations.keySet());
+        } finally {
+            destinationsLock.readLock().unlock();
         }
         return inactiveDests;
     }
 
+    @SuppressWarnings("unchecked")
     public void removeConsumer(ConnectionContext context, ConsumerInfo info) throws Exception {
-        LOG.debug(broker.getBrokerName() + " removing consumer: " + info.getConsumerId() + " for destination: "
-                + info.getDestination());
+        if (LOG.isDebugEnabled()) {
+            LOG.debug(broker.getBrokerName() + " removing consumer: " + info.getConsumerId() + " for destination: "
+                    + info.getDestination());
+        }
 
         Subscription sub = subscriptions.remove(info.getConsumerId());
         // The sub could be removed elsewhere - see ConnectionSplitBroker
@@ -328,12 +369,13 @@ public abstract class AbstractRegion implements Region {
 
             // remove the subscription from all the matching queues.
             List<Destination> removeList = new ArrayList<Destination>();
-            synchronized (destinationsMutex) {
-                for (Iterator iter = destinationMap.get(info.getDestination()).iterator(); iter.hasNext();) {
-                    Destination dest = (Destination) iter.next();
+            destinationsLock.readLock().lock();
+            try {
+                for (Destination dest : (Set<Destination>) destinationMap.get(info.getDestination())) {
                     removeList.add(dest);
-
                 }
+            } finally {
+                destinationsLock.readLock().unlock();
             }
             for (Destination dest : removeList) {
                 dest.removeSubscription(context, sub, info.getLastDeliveredSequenceId());
@@ -374,7 +416,9 @@ public abstract class AbstractRegion implements Region {
                     LOG.warn("Ack for non existent subscription, ack:" + ack);
                     throw new IllegalArgumentException("The subscription does not exist: " + ack.getConsumerId());
                 } else {
-                    LOG.debug("Ack for non existent subscription in recovery, ack:" + ack);
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("Ack for non existent subscription in recovery, ack:" + ack);
+                    }
                     return;
                 }
             }
@@ -393,9 +437,14 @@ public abstract class AbstractRegion implements Region {
 
     protected Destination lookup(ConnectionContext context, ActiveMQDestination destination,boolean createTemporary) throws Exception {
         Destination dest = null;
-        synchronized (destinationsMutex) {
+
+        destinationsLock.readLock().lock();
+        try {
             dest = destinations.get(destination);
+        } finally {
+            destinationsLock.readLock().unlock();
         }
+
         if (dest == null) {
             if (isAutoCreateDestinations()) {
                 // Try to auto create the destination... re-invoke broker
@@ -409,10 +458,14 @@ public abstract class AbstractRegion implements Region {
                     // this error
                 }
                 // We should now have the dest created.
-                synchronized (destinationsMutex) {
+                destinationsLock.readLock().lock();
+                try {
                     dest = destinations.get(destination);
+                } finally {
+                    destinationsLock.readLock().unlock();
                 }
             }
+
             if (dest == null) {
                 throw new JMSException("The destination " + destination + " does not exist.");
             }
@@ -440,9 +493,13 @@ public abstract class AbstractRegion implements Region {
     protected void processDispatchNotificationViaDestination(MessageDispatchNotification messageDispatchNotification)
             throws Exception {
         Destination dest = null;
-        synchronized (destinationsMutex) {
+        destinationsLock.readLock().lock();
+        try {
             dest = destinations.get(messageDispatchNotification.getDestination());
+        } finally {
+            destinationsLock.readLock().unlock();
         }
+
         if (dest != null) {
             dest.processDispatchNotification(messageDispatchNotification);
         } else {
@@ -454,15 +511,17 @@ public abstract class AbstractRegion implements Region {
     }
 
     public void gc() {
-        for (Iterator<Subscription> iter = subscriptions.values().iterator(); iter.hasNext();) {
-            Subscription sub = iter.next();
+        for (Subscription sub : subscriptions.values()) {
             sub.gc();
         }
-        synchronized (destinationsMutex) {
-            for (Iterator<Destination> iter = destinations.values().iterator(); iter.hasNext();) {
-                Destination dest = iter.next();
+
+        destinationsLock.readLock().lock();
+        try {
+            for (Destination dest : destinations.values()) {
                 dest.gc();
             }
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
@@ -481,29 +540,35 @@ public abstract class AbstractRegion implements Region {
         this.autoCreateDestinations = autoCreateDestinations;
     }
 
+    @SuppressWarnings("unchecked")
     public void addProducer(ConnectionContext context, ProducerInfo info) throws Exception {
-        synchronized (destinationsMutex) {
-            for (Iterator iter = destinationMap.get(info.getDestination()).iterator(); iter.hasNext();) {
-                Destination dest = (Destination) iter.next();
+        destinationsLock.readLock().lock();
+        try {
+            for (Destination dest : (Set<Destination>) destinationMap.get(info.getDestination())) {
                 dest.addProducer(context, info);
             }
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
     /**
      * Removes a Producer.
-     * 
+     *
      * @param context
      *            the environment the operation is being executed under.
      * @throws Exception
      *             TODO
      */
+    @SuppressWarnings("unchecked")
     public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception {
-        synchronized (destinationsMutex) {
-            for (Iterator iter = destinationMap.get(info.getDestination()).iterator(); iter.hasNext();) {
-                Destination dest = (Destination) iter.next();
+        destinationsLock.readLock().lock();
+        try {
+            for (Destination dest : (Set<Destination>) destinationMap.get(info.getDestination())) {
                 dest.removeProducer(context, info);
             }
+        } finally {
+            destinationsLock.readLock().unlock();
         }
     }
 
