@@ -17,6 +17,7 @@
 package org.apache.activemq.broker.region;
 
 import java.io.IOException;
+import java.util.List;
 import javax.jms.ResourceAllocationException;
 import org.apache.activemq.advisory.AdvisorySupport;
 import org.apache.activemq.broker.Broker;
@@ -30,15 +31,18 @@ import org.apache.activemq.command.ActiveMQTopic;
 import org.apache.activemq.command.Message;
 import org.apache.activemq.command.MessageDispatchNotification;
 import org.apache.activemq.command.ProducerInfo;
+import org.apache.activemq.filter.NonCachedMessageEvaluationContext;
+import org.apache.activemq.security.SecurityContext;
 import org.apache.activemq.state.ProducerState;
 import org.apache.activemq.store.MessageStore;
+import org.apache.activemq.thread.Scheduler;
 import org.apache.activemq.usage.MemoryUsage;
 import org.apache.activemq.usage.SystemUsage;
 import org.apache.activemq.usage.Usage;
 import org.slf4j.Logger;
 
 /**
- * 
+ *
  */
 public abstract class BaseDestination implements Destination {
     /**
@@ -58,6 +62,7 @@ public abstract class BaseDestination implements Destination {
     protected SystemUsage systemUsage;
     protected MemoryUsage memoryUsage;
     private boolean producerFlowControl = true;
+    private boolean alwaysRetroactive = false;
     protected boolean warnOnProducerFlowControl = true;
     protected long blockedProducerWarningInterval = DEFAULT_BLOCKED_PRODUCER_WARNING_INTERVAL;
 
@@ -88,11 +93,19 @@ public abstract class BaseDestination implements Destination {
     private boolean prioritizedMessages;
     private long inactiveTimoutBeforeGC = DEFAULT_INACTIVE_TIMEOUT_BEFORE_GC;
     private boolean gcIfInactive;
+    private boolean gcWithNetworkConsumers;
     private long lastActiveTime=0l;
     private boolean reduceMemoryFootprint = false;
+    protected final Scheduler scheduler;
+    private boolean disposed = false;
+    private boolean doOptimzeMessageStorage = true;
+    /*
+     * percentage of in-flight messages above which optimize message store is disabled
+     */
+    private int optimizeMessageStoreInFlightLimit = 10;
 
     /**
-     * @param broker
+     * @param brokerService
      * @param store
      * @param destination
      * @param parentStats
@@ -110,11 +123,12 @@ public abstract class BaseDestination implements Destination {
         this.memoryUsage = this.systemUsage.getMemoryUsage();
         this.memoryUsage.setUsagePortion(1.0f);
         this.regionBroker = brokerService.getRegionBroker();
+        this.scheduler = brokerService.getBroker().getScheduler();
     }
 
     /**
      * initialize the destination
-     * 
+     *
      * @throws Exception
      */
     public void initialize() throws Exception {
@@ -139,11 +153,19 @@ public abstract class BaseDestination implements Destination {
         this.producerFlowControl = producerFlowControl;
     }
 
+    public boolean isAlwaysRetroactive() {
+        return alwaysRetroactive;
+    }
+
+    public void setAlwaysRetroactive(boolean alwaysRetroactive) {
+        this.alwaysRetroactive = alwaysRetroactive;
+    }
+
     /**
      * Set's the interval at which warnings about producers being blocked by
      * resource usage will be triggered. Values of 0 or less will disable
      * warnings
-     * 
+     *
      * @param blockedProducerWarningInterval the interval at which warning about
      *            blocked producers will be triggered.
      */
@@ -152,7 +174,7 @@ public abstract class BaseDestination implements Destination {
     }
 
     /**
-     * 
+     *
      * @return the interval at which warning about blocked producers will be
      *         triggered.
      */
@@ -210,7 +232,7 @@ public abstract class BaseDestination implements Destination {
     public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception {
         destinationStatistics.getProducers().decrement();
     }
-    
+
     public void addSubscription(ConnectionContext context, Subscription sub) throws Exception{
         destinationStatistics.getConsumers().increment();
         this.lastActiveTime=0l;
@@ -241,8 +263,13 @@ public abstract class BaseDestination implements Destination {
         return store;
     }
 
-    public final boolean isActive() {
-        return destinationStatistics.getConsumers().getCount() != 0 || destinationStatistics.getProducers().getCount() != 0;
+    public boolean isActive() {
+        boolean isActive = destinationStatistics.getConsumers().getCount() != 0 ||
+                           destinationStatistics.getProducers().getCount() != 0;
+        if (isActive && isGcWithNetworkConsumers() && destinationStatistics.getConsumers().getCount() != 0) {
+            isActive = hasRegularConsumers(getConsumers());
+        }
+        return isActive;
     }
 
     public int getMaxPageSize() {
@@ -407,7 +434,7 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * set the dead letter strategy
-     * 
+     *
      * @param deadLetterStrategy
      */
     public void setDeadLetterStrategy(DeadLetterStrategy deadLetterStrategy) {
@@ -424,7 +451,7 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * called when message is consumed
-     * 
+     *
      * @param context
      * @param messageReference
      */
@@ -436,7 +463,7 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * Called when message is delivered to the broker
-     * 
+     *
      * @param context
      * @param messageReference
      */
@@ -449,7 +476,7 @@ public abstract class BaseDestination implements Destination {
     /**
      * Called when a message is discarded - e.g. running low on memory This will
      * happen only if the policy is enabled - e.g. non durable topics
-     * 
+     *
      * @param context
      * @param messageReference
      */
@@ -461,7 +488,7 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * Called when there is a slow consumer
-     * 
+     *
      * @param context
      * @param subs
      */
@@ -476,7 +503,7 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * Called to notify a producer is too fast
-     * 
+     *
      * @param context
      * @param producerInfo
      */
@@ -488,11 +515,11 @@ public abstract class BaseDestination implements Destination {
 
     /**
      * Called when a Usage reaches a limit
-     * 
+     *
      * @param context
      * @param usage
      */
-    public void isFull(ConnectionContext context, Usage usage) {
+    public void isFull(ConnectionContext context, Usage<?> usage) {
         if (advisoryWhenFull) {
             broker.isFull(context, this, usage);
         }
@@ -505,6 +532,11 @@ public abstract class BaseDestination implements Destination {
         }
         this.destinationStatistics.setParent(null);
         this.memoryUsage.stop();
+        this.disposed = true;
+    }
+
+    public boolean isDisposed() {
+        return this.disposed;
     }
 
     /**
@@ -572,15 +604,15 @@ public abstract class BaseDestination implements Destination {
     protected final void waitForSpace(ConnectionContext context, Usage<?> usage, String warning) throws IOException, InterruptedException, ResourceAllocationException {
         waitForSpace(context, usage, 100, warning);
     }
-    
+
     protected final void waitForSpace(ConnectionContext context, Usage<?> usage, int highWaterMark, String warning) throws IOException, InterruptedException, ResourceAllocationException {
-        if (systemUsage.isSendFailIfNoSpace()) {
-            getLog().debug("sendFailIfNoSpace, forcing exception on send: " + warning);
+        if (!context.isNetworkConnection() && systemUsage.isSendFailIfNoSpace()) {
+            getLog().debug("sendFailIfNoSpace, forcing exception on send, usage:  " + usage + ": " + warning);
             throw new ResourceAllocationException(warning);
         }
-        if (systemUsage.getSendFailIfNoSpaceAfterTimeout() != 0) {
+        if (!context.isNetworkConnection() && systemUsage.getSendFailIfNoSpaceAfterTimeout() != 0) {
             if (!usage.waitForSpace(systemUsage.getSendFailIfNoSpaceAfterTimeout(), highWaterMark)) {
-                getLog().debug("sendFailIfNoSpaceAfterTimeout expired, forcing exception on send: " + warning);
+                getLog().debug("sendFailIfNoSpaceAfterTimeout expired, forcing exception on send, usage: " + usage + ": " + warning);
                 throw new ResourceAllocationException(warning);
             }
         } else {
@@ -590,10 +622,10 @@ public abstract class BaseDestination implements Destination {
                 if (context.getStopping().get()) {
                     throw new IOException("Connection closed, send aborted.");
                 }
-    
+
                 long now = System.currentTimeMillis();
                 if (now >= nextWarn) {
-                    getLog().info(warning + " (blocking for: " + (now - start) / 1000 + "s)");
+                    getLog().info("" + usage + ": " + warning + " (blocking for: " + (now - start) / 1000 + "s)");
                     nextWarn = now + blockedProducerWarningInterval;
                 }
             }
@@ -610,7 +642,7 @@ public abstract class BaseDestination implements Destination {
         return this.slowConsumerStrategy;
     }
 
-   
+
     public boolean isPrioritizedMessages() {
         return this.prioritizedMessages;
     }
@@ -649,7 +681,19 @@ public abstract class BaseDestination implements Destination {
     public void setGcIfInactive(boolean gcIfInactive) {
         this.gcIfInactive = gcIfInactive;
     }
-    
+
+    /**
+     * Indicate if it is ok to gc destinations that have only network consumers
+     * @param gcWithNetworkConsumers
+     */
+    public void setGcWithNetworkConsumers(boolean gcWithNetworkConsumers) {
+        this.gcWithNetworkConsumers = gcWithNetworkConsumers;
+    }
+
+    public boolean isGcWithNetworkConsumers() {
+        return gcWithNetworkConsumers;
+    }
+
     public void markForGC(long timeStamp) {
         if (isGcIfInactive() && this.lastActiveTime == 0 && isActive() == false
                 && destinationStatistics.messages.getCount() == 0 && getInactiveTimoutBeforeGC() > 0l) {
@@ -674,4 +718,42 @@ public abstract class BaseDestination implements Destination {
     protected boolean isReduceMemoryFootprint() {
         return this.reduceMemoryFootprint;
     }
+
+    public boolean isDoOptimzeMessageStorage() {
+        return doOptimzeMessageStorage;
+    }
+
+    public void setDoOptimzeMessageStorage(boolean doOptimzeMessageStorage) {
+        this.doOptimzeMessageStorage = doOptimzeMessageStorage;
+    }
+
+    public int getOptimizeMessageStoreInFlightLimit() {
+        return optimizeMessageStoreInFlightLimit;
+    }
+
+    public void setOptimizeMessageStoreInFlightLimit(int optimizeMessageStoreInFlightLimit) {
+        this.optimizeMessageStoreInFlightLimit = optimizeMessageStoreInFlightLimit;
+    }
+
+
+    public abstract List<Subscription> getConsumers();
+
+    protected boolean hasRegularConsumers(List<Subscription> consumers) {
+        boolean hasRegularConsumers = false;
+        for (Subscription subscription: consumers) {
+            if (!subscription.getConsumerInfo().isNetworkSubscription()) {
+                hasRegularConsumers = true;
+                break;
+            }
+        }
+        return hasRegularConsumers;
+    }
+
+    protected ConnectionContext createConnectionContext() {
+        ConnectionContext answer = new ConnectionContext(new NonCachedMessageEvaluationContext());
+        answer.setBroker(this.broker);
+        answer.getMessageEvaluationContext().setDestination(getActiveMQDestination());
+        answer.setSecurityContext(SecurityContext.BROKER_SECURITY_CONTEXT);
+        return answer;
+    }
 }
