@@ -16,20 +16,6 @@
  */
 package org.apache.activemq.broker.region;
 
-import java.io.IOException;
-import java.net.URI;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.CopyOnWriteArrayList;
-import java.util.concurrent.ThreadPoolExecutor;
-import javax.jms.InvalidClientIDException;
-import javax.jms.JMSException;
-import org.apache.activemq.advisory.AdvisorySupport;
 import org.apache.activemq.broker.Broker;
 import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.broker.Connection;
@@ -40,23 +26,7 @@ import org.apache.activemq.broker.ProducerBrokerExchange;
 import org.apache.activemq.broker.TransportConnector;
 import org.apache.activemq.broker.region.policy.DeadLetterStrategy;
 import org.apache.activemq.broker.region.policy.PolicyMap;
-import org.apache.activemq.command.ActiveMQDestination;
-import org.apache.activemq.command.BrokerId;
-import org.apache.activemq.command.BrokerInfo;
-import org.apache.activemq.command.ConnectionId;
-import org.apache.activemq.command.ConnectionInfo;
-import org.apache.activemq.command.ConsumerControl;
-import org.apache.activemq.command.ConsumerInfo;
-import org.apache.activemq.command.DestinationInfo;
-import org.apache.activemq.command.Message;
-import org.apache.activemq.command.MessageAck;
-import org.apache.activemq.command.MessageDispatch;
-import org.apache.activemq.command.MessageDispatchNotification;
-import org.apache.activemq.command.MessagePull;
-import org.apache.activemq.command.ProducerInfo;
-import org.apache.activemq.command.RemoveSubscriptionInfo;
-import org.apache.activemq.command.Response;
-import org.apache.activemq.command.TransactionId;
+import org.apache.activemq.command.*;
 import org.apache.activemq.state.ConnectionState;
 import org.apache.activemq.store.kahadb.plist.PListStore;
 import org.apache.activemq.thread.Scheduler;
@@ -70,10 +40,25 @@ import org.apache.activemq.util.ServiceStopper;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.jms.InvalidClientIDException;
+import javax.jms.JMSException;
+import java.io.IOException;
+import java.net.URI;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.ThreadPoolExecutor;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
+
 /**
  * Routes Broker operations to the correct messaging regions for processing.
- * 
- * 
+ *
+ *
  */
 public class RegionBroker extends EmptyBroker {
     public static final String ORIGINAL_EXPIRATION = "originalExpiration";
@@ -104,7 +89,9 @@ public class RegionBroker extends EmptyBroker {
     private ConnectionContext adminConnectionContext;
     private final Scheduler scheduler;
     private final ThreadPoolExecutor executor;
-    
+    private boolean allowTempAutoCreationOnSend;
+
+    private final ReentrantReadWriteLock inactiveDestinationsPurgeLock = new ReentrantReadWriteLock();
     private final Runnable purgeInactiveDestinationsTask = new Runnable() {
         public void run() {
             purgeInactiveDestinations();
@@ -130,7 +117,7 @@ public class RegionBroker extends EmptyBroker {
 
     @Override
     public Map<ActiveMQDestination, Destination> getDestinationMap() {
-        Map<ActiveMQDestination, Destination> answer = getQueueRegion().getDestinationMap();
+        Map<ActiveMQDestination, Destination> answer = new HashMap<ActiveMQDestination, Destination>(getQueueRegion().getDestinationMap());
         answer.putAll(getTopicRegion().getDestinationMap());
         return answer;
     }
@@ -152,6 +139,7 @@ public class RegionBroker extends EmptyBroker {
     }
 
     @Override
+    @SuppressWarnings("rawtypes")
     public Broker getAdaptor(Class type) {
         if (type.isInstance(this)) {
             return this;
@@ -232,17 +220,8 @@ public class RegionBroker extends EmptyBroker {
         synchronized (clientIdSet) {
             ConnectionContext oldContext = clientIdSet.get(clientId);
             if (oldContext != null) {
-            	if (context.isFaultTolerant() || context.isNetworkConnection()){
-            		//remove the old connection
-            		try{
-            			removeConnection(oldContext, info, new Exception("remove stale client"));
-            		}catch(Exception e){
-            			LOG.warn("Failed to remove stale connection ",e);
-            		}
-            	}else{
                 throw new InvalidClientIDException("Broker: " + getBrokerName() + " - Client: " + clientId + " already connected from "
                                                    + oldContext.getConnection().getRemoteAddress());
-            	}
             } else {
                 clientIdSet.put(clientId, context);
             }
@@ -284,7 +263,7 @@ public class RegionBroker extends EmptyBroker {
     }
 
     @Override
-    public Destination addDestination(ConnectionContext context, ActiveMQDestination destination,boolean create) throws Exception {
+    public Destination addDestination(ConnectionContext context, ActiveMQDestination destination, boolean createIfTemp) throws Exception {
 
         Destination answer;
 
@@ -293,6 +272,12 @@ public class RegionBroker extends EmptyBroker {
             return answer;
         }
 
+     synchronized (destinations) {
+        answer = destinations.get(destination);
+        if (answer != null) {
+            return answer;
+        }
+
         switch (destination.getDestinationType()) {
         case ActiveMQDestination.QUEUE_TYPE:
             answer = queueRegion.addDestination(context, destination,true);
@@ -301,10 +286,10 @@ public class RegionBroker extends EmptyBroker {
             answer = topicRegion.addDestination(context, destination,true);
             break;
         case ActiveMQDestination.TEMP_QUEUE_TYPE:
-            answer = tempQueueRegion.addDestination(context, destination,create);
+            answer = tempQueueRegion.addDestination(context, destination, createIfTemp);
             break;
         case ActiveMQDestination.TEMP_TOPIC_TYPE:
-            answer = tempTopicRegion.addDestination(context, destination,create);
+            answer = tempTopicRegion.addDestination(context, destination, createIfTemp);
             break;
         default:
             throw createUnknownDestinationTypeException(destination);
@@ -312,6 +297,7 @@ public class RegionBroker extends EmptyBroker {
 
         destinations.put(destination, answer);
         return answer;
+     }
 
     }
 
@@ -322,11 +308,9 @@ public class RegionBroker extends EmptyBroker {
             switch (destination.getDestinationType()) {
             case ActiveMQDestination.QUEUE_TYPE:
                 queueRegion.removeDestination(context, destination, timeout);
-                removeAdvisoryTopics("Queue.", context, destination, timeout);
                 break;
             case ActiveMQDestination.TOPIC_TYPE:
                 topicRegion.removeDestination(context, destination, timeout);
-                removeAdvisoryTopics("Topic.", context, destination, timeout);
                 break;
             case ActiveMQDestination.TEMP_QUEUE_TYPE:
                 tempQueueRegion.removeDestination(context, destination, timeout);
@@ -343,25 +327,6 @@ public class RegionBroker extends EmptyBroker {
 
     }
 
-    public void removeAdvisoryTopics(String destinationType, ConnectionContext context, ActiveMQDestination destination, long timeout) throws Exception {
-        if (this.brokerService.isAdvisorySupport()) {
-            String producerAdvisoryTopic = AdvisorySupport.PRODUCER_ADVISORY_TOPIC_PREFIX + destinationType + destination.getPhysicalName();
-            String consumerAdvisoryTopic = AdvisorySupport.CONSUMER_ADVISORY_TOPIC_PREFIX + destinationType + destination.getPhysicalName();
-
-            ActiveMQDestination dests[] = getDestinations();
-            for (ActiveMQDestination dest: dests) {
-                String name = dest.getPhysicalName();
-                if ( name.equals(producerAdvisoryTopic) || name.equals(consumerAdvisoryTopic) ) {
-                    try {
-                        removeDestination(context, dest, timeout);
-                    } catch (JMSException ignore) {
-                        // at least ignore the Unknown Destination Type JMSException
-                    }
-                }
-            }
-        }
-    }
-
     @Override
     public void addDestinationInfo(ConnectionContext context, DestinationInfo info) throws Exception {
         addDestination(context, info.getDestination(),true);
@@ -388,12 +353,12 @@ public class RegionBroker extends EmptyBroker {
     @Override
     public void addProducer(ConnectionContext context, ProducerInfo info) throws Exception {
         ActiveMQDestination destination = info.getDestination();
-        synchronized (purgeInactiveDestinationsTask) {
-            if (destination != null) {
-
+        if (destination != null) {
+            inactiveDestinationsPurgeLock.readLock().lock();
+            try {
                 // This seems to cause the destination to be added but without
                 // advisories firing...
-                context.getBroker().addDestination(context, destination, false);
+                context.getBroker().addDestination(context, destination, isAllowTempAutoCreationOnSend());
                 switch (destination.getDestinationType()) {
                 case ActiveMQDestination.QUEUE_TYPE:
                     queueRegion.addProducer(context, info);
@@ -408,6 +373,8 @@ public class RegionBroker extends EmptyBroker {
                     tempTopicRegion.addProducer(context, info);
                     break;
                 }
+            } finally {
+                inactiveDestinationsPurgeLock.readLock().unlock();
             }
         }
     }
@@ -415,8 +382,9 @@ public class RegionBroker extends EmptyBroker {
     @Override
     public void removeProducer(ConnectionContext context, ProducerInfo info) throws Exception {
         ActiveMQDestination destination = info.getDestination();
-        synchronized (purgeInactiveDestinationsTask) {
-            if (destination != null) {
+        if (destination != null) {
+            inactiveDestinationsPurgeLock.readLock().lock();
+            try {
                 switch (destination.getDestinationType()) {
                 case ActiveMQDestination.QUEUE_TYPE:
                     queueRegion.removeProducer(context, info);
@@ -431,6 +399,8 @@ public class RegionBroker extends EmptyBroker {
                     tempTopicRegion.removeProducer(context, info);
                     break;
                 }
+            } finally {
+                inactiveDestinationsPurgeLock.readLock().unlock();
             }
         }
     }
@@ -441,7 +411,8 @@ public class RegionBroker extends EmptyBroker {
         if (destinationInterceptor != null) {
             destinationInterceptor.create(this, context, destination);
         }
-        synchronized (purgeInactiveDestinationsTask) {
+        inactiveDestinationsPurgeLock.readLock().lock();
+        try {
             switch (destination.getDestinationType()) {
             case ActiveMQDestination.QUEUE_TYPE:
                 return queueRegion.addConsumer(context, info);
@@ -458,13 +429,16 @@ public class RegionBroker extends EmptyBroker {
             default:
                 throw createUnknownDestinationTypeException(destination);
             }
+        } finally {
+            inactiveDestinationsPurgeLock.readLock().unlock();
         }
     }
 
     @Override
     public void removeConsumer(ConnectionContext context, ConsumerInfo info) throws Exception {
         ActiveMQDestination destination = info.getDestination();
-        synchronized (purgeInactiveDestinationsTask) {
+        inactiveDestinationsPurgeLock.readLock().lock();
+        try {
             switch (destination.getDestinationType()) {
 
             case ActiveMQDestination.QUEUE_TYPE:
@@ -482,13 +456,18 @@ public class RegionBroker extends EmptyBroker {
             default:
                 throw createUnknownDestinationTypeException(destination);
             }
+        } finally {
+            inactiveDestinationsPurgeLock.readLock().unlock();
         }
     }
 
     @Override
     public void removeSubscription(ConnectionContext context, RemoveSubscriptionInfo info) throws Exception {
-        synchronized (purgeInactiveDestinationsTask) {
+        inactiveDestinationsPurgeLock.readLock().lock();
+        try {
             topicRegion.removeSubscription(context, info);
+        } finally {
+            inactiveDestinationsPurgeLock.readLock().unlock();
         }
     }
 
@@ -496,10 +475,10 @@ public class RegionBroker extends EmptyBroker {
     public void send(ProducerBrokerExchange producerExchange, Message message) throws Exception {
         message.setBrokerInTime(System.currentTimeMillis());
         if (producerExchange.isMutable() || producerExchange.getRegion() == null
-                || (producerExchange.getRegion() != null && producerExchange.getRegion().getDestinationMap().get(message.getDestination()) == null)) {
+                || (producerExchange.getRegionDestination() != null && producerExchange.getRegionDestination().isDisposed())) {
             ActiveMQDestination destination = message.getDestination();
             // ensure the destination is registered with the RegionBroker
-            producerExchange.getConnectionContext().getBroker().addDestination(producerExchange.getConnectionContext(), destination,false);
+            producerExchange.getConnectionContext().getBroker().addDestination(producerExchange.getConnectionContext(), destination, isAllowTempAutoCreationOnSend());
             Region region;
             switch (destination.getDestinationType()) {
             case ActiveMQDestination.QUEUE_TYPE:
@@ -520,6 +499,7 @@ public class RegionBroker extends EmptyBroker {
             producerExchange.setRegion(region);
             producerExchange.setRegionDestination(null);
         }
+
         producerExchange.getRegion().send(producerExchange, message);
     }
 
@@ -650,8 +630,10 @@ public class RegionBroker extends EmptyBroker {
             brokerInfos.put(info.getBrokerId(), existing);
         }
         existing.incrementRefCount();
-        LOG.debug(getBrokerName() + " addBroker:" + info.getBrokerName() + " brokerInfo size : " + brokerInfos.size());
-        addBrokerInClusterUpdate();
+        if (LOG.isDebugEnabled()) {
+            LOG.debug(getBrokerName() + " addBroker:" + info.getBrokerName() + " brokerInfo size : " + brokerInfos.size());
+        }
+        addBrokerInClusterUpdate(info);
     }
 
     @Override
@@ -661,8 +643,10 @@ public class RegionBroker extends EmptyBroker {
             if (existing != null && existing.decrementRefCount() == 0) {
                brokerInfos.remove(info.getBrokerId());
             }
-            LOG.debug(getBrokerName() + " removeBroker:" + info.getBrokerName() + " brokerInfo size : " + brokerInfos.size());
-            removeBrokerInClusterUpdate();
+            if (LOG.isDebugEnabled()) {
+                LOG.debug(getBrokerName() + " removeBroker:" + info.getBrokerName() + " brokerInfo size : " + brokerInfos.size());
+            }
+            removeBrokerInClusterUpdate(info);
         }
     }
 
@@ -793,18 +777,18 @@ public class RegionBroker extends EmptyBroker {
         }
         return expired;
     }
-   
+
     private boolean stampAsExpired(Message message) throws IOException {
         boolean stamped=false;
         if (message.getProperty(ORIGINAL_EXPIRATION) == null) {
-            long expiration=message.getExpiration();     
+            long expiration=message.getExpiration();
             message.setProperty(ORIGINAL_EXPIRATION,new Long(expiration));
             stamped = true;
         }
         return stamped;
     }
 
-    
+
     @Override
     public void messageExpired(ConnectionContext context, MessageReference node, Subscription subscription) {
         if (LOG.isDebugEnabled()) {
@@ -812,51 +796,51 @@ public class RegionBroker extends EmptyBroker {
         }
         getRoot().sendToDeadLetterQueue(context, node, subscription);
     }
-    
+
     @Override
     public void sendToDeadLetterQueue(ConnectionContext context,
-	        MessageReference node, Subscription subscription){
-		try{
-			if(node!=null){
-				Message message=node.getMessage();
-				if(message!=null && node.getRegionDestination()!=null){
-					DeadLetterStrategy deadLetterStrategy=node
-					        .getRegionDestination().getDeadLetterStrategy();
-					if(deadLetterStrategy!=null){
-						if(deadLetterStrategy.isSendToDeadLetterQueue(message)){
-						    // message may be inflight to other subscriptions so do not modify
-						    message = message.copy();
-						    stampAsExpired(message);
-						    message.setExpiration(0);
-						    if(!message.isPersistent()){
-							    message.setPersistent(true);
-							    message.setProperty("originalDeliveryMode",
-								        "NON_PERSISTENT");
-							}
-							// The original destination and transaction id do
-							// not get filled when the message is first sent,
-							// it is only populated if the message is routed to
-							// another destination like the DLQ
-							ActiveMQDestination deadLetterDestination=deadLetterStrategy
-							        .getDeadLetterQueueFor(message, subscription);
-							if (context.getBroker()==null) {
-								context.setBroker(getRoot());
-							}
-							BrokerSupport.resendNoCopy(context,message,
-							        deadLetterDestination);
-						}
-					} else {
-					    if (LOG.isDebugEnabled()) {
-					        LOG.debug("Dead Letter message with no DLQ strategy in place, message id: "
+            MessageReference node, Subscription subscription){
+        try{
+            if(node!=null){
+                Message message=node.getMessage();
+                if(message!=null && node.getRegionDestination()!=null){
+                    DeadLetterStrategy deadLetterStrategy=node
+                            .getRegionDestination().getDeadLetterStrategy();
+                    if(deadLetterStrategy!=null){
+                        if(deadLetterStrategy.isSendToDeadLetterQueue(message)){
+                            // message may be inflight to other subscriptions so do not modify
+                            message = message.copy();
+                            stampAsExpired(message);
+                            message.setExpiration(0);
+                            if(!message.isPersistent()){
+                                message.setPersistent(true);
+                                message.setProperty("originalDeliveryMode",
+                                        "NON_PERSISTENT");
+                            }
+                            // The original destination and transaction id do
+                            // not get filled when the message is first sent,
+                            // it is only populated if the message is routed to
+                            // another destination like the DLQ
+                            ActiveMQDestination deadLetterDestination=deadLetterStrategy
+                                    .getDeadLetterQueueFor(message, subscription);
+                            if (context.getBroker()==null) {
+                                context.setBroker(getRoot());
+                            }
+                            BrokerSupport.resendNoCopy(context,message,
+                                    deadLetterDestination);
+                        }
+                    } else {
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("Dead Letter message with no DLQ strategy in place, message id: "
                                     + message.getMessageId() + ", destination: " + message.getDestination());
-					    }
-					}
-				}
-			}
-		}catch(Exception e){
-			LOG.warn("Caught an exception sending to DLQ: "+node,e);
-		}
-	}
+                        }
+                    }
+                }
+            }
+        }catch(Exception e){
+            LOG.warn("Caught an exception sending to DLQ: "+node,e);
+        }
+    }
 
     @Override
     public Broker getRoot() {
@@ -867,7 +851,7 @@ public class RegionBroker extends EmptyBroker {
             throw new RuntimeException("The broker from the BrokerService should not throw an exception");
         }
     }
-    
+
     /**
      * @return the broker sequence id
      */
@@ -877,17 +861,17 @@ public class RegionBroker extends EmptyBroker {
             return sequenceGenerator.getNextSequenceId();
         }
     }
-    
-    
+
+
     @Override
     public Scheduler getScheduler() {
         return this.scheduler;
     }
-    
+
     public ThreadPoolExecutor getExecutor() {
         return this.executor;
     }
-    
+
     @Override
     public void processConsumerControl(ConsumerBrokerExchange consumerExchange, ConsumerControl control) {
         ActiveMQDestination destination = control.getDestination();
@@ -899,69 +883,89 @@ public class RegionBroker extends EmptyBroker {
         case ActiveMQDestination.TOPIC_TYPE:
             topicRegion.processConsumerControl(consumerExchange, control);
             break;
-            
+
         case ActiveMQDestination.TEMP_QUEUE_TYPE:
             tempQueueRegion.processConsumerControl(consumerExchange, control);
             break;
-            
+
         case ActiveMQDestination.TEMP_TOPIC_TYPE:
             tempTopicRegion.processConsumerControl(consumerExchange, control);
             break;
-            
+
         default:
             LOG.warn("unmatched destination: " + destination + ", in consumerControl: "  + control);
         }
     }
-    
-    protected void addBrokerInClusterUpdate() {
+
+    protected void addBrokerInClusterUpdate(BrokerInfo info) {
         List<TransportConnector> connectors = this.brokerService.getTransportConnectors();
         for (TransportConnector connector : connectors) {
             if (connector.isUpdateClusterClients()) {
+                connector.addPeerBroker(info);
                 connector.updateClientClusterInfo();
             }
         }
     }
 
-    protected void removeBrokerInClusterUpdate() {
+    protected void removeBrokerInClusterUpdate(BrokerInfo info) {
         List<TransportConnector> connectors = this.brokerService.getTransportConnectors();
         for (TransportConnector connector : connectors) {
             if (connector.isUpdateClusterClients() && connector.isUpdateClusterClientsOnRemove()) {
+                connector.removePeerBroker(info);
                 connector.updateClientClusterInfo();
             }
         }
     }
-    
+
     protected void purgeInactiveDestinations() {
-        synchronized (purgeInactiveDestinationsTask) {
-            List<BaseDestination> list = new ArrayList<BaseDestination>();
+        inactiveDestinationsPurgeLock.writeLock().lock();
+        try {
+            List<Destination> list = new ArrayList<Destination>();
             Map<ActiveMQDestination, Destination> map = getDestinationMap();
+            if (isAllowTempAutoCreationOnSend()) {
+                map.putAll(tempQueueRegion.getDestinationMap());
+                map.putAll(tempTopicRegion.getDestinationMap());
+            }
+            long maxPurgedDests = this.brokerService.getMaxPurgedDestinationsPerSweep();
             long timeStamp = System.currentTimeMillis();
             for (Destination d : map.values()) {
-                if (d instanceof BaseDestination) {
-                    BaseDestination bd = (BaseDestination) d;
-                    bd.markForGC(timeStamp);
-                    if (bd.canGC()) {
-                        list.add(bd);
+                d.markForGC(timeStamp);
+                if (d.canGC()) {
+                    list.add(d);
+                    if (maxPurgedDests > 0 && list.size() == maxPurgedDests) {
+                        break;
                     }
                 }
             }
 
-            if (list.isEmpty() == false) {
-
+            if (!list.isEmpty()) {
                 ConnectionContext context = BrokerSupport.getConnectionContext(this);
                 context.setBroker(this);
 
-                for (BaseDestination dest : list) {
-                    dest.getLog().info(
-                            dest.getName() + " Inactive for longer than " + dest.getInactiveTimoutBeforeGC()
-                                    + " ms - removing ...");
+                for (Destination dest : list) {
+                    Logger log = LOG;
+                    if (dest instanceof BaseDestination) {
+                        log = ((BaseDestination) dest).getLog();
+                    }
+                    log.info(dest.getName() + " Inactive for longer than " +
+                             dest.getInactiveTimoutBeforeGC() + " ms - removing ...");
                     try {
-                        getRoot().removeDestination(context, dest.getActiveMQDestination(), 0);
+                        getRoot().removeDestination(context, dest.getActiveMQDestination(), isAllowTempAutoCreationOnSend() ? 1 : 0);
                     } catch (Exception e) {
                         LOG.error("Failed to remove inactive destination " + dest, e);
                     }
                 }
             }
+        } finally {
+            inactiveDestinationsPurgeLock.writeLock().unlock();
         }
     }
+
+    public boolean isAllowTempAutoCreationOnSend() {
+        return allowTempAutoCreationOnSend;
+    }
+
+    public void setAllowTempAutoCreationOnSend(boolean allowTempAutoCreationOnSend) {
+        this.allowTempAutoCreationOnSend = allowTempAutoCreationOnSend;
+    }
 }
