@@ -99,11 +99,14 @@ public class TopicSubscription extends AbstractSubscription {
             dispatch(node);
             setSlowConsumer(false);
         } else {
-            //we are slow
-            if(!isSlowConsumer()) {
-                setSlowConsumer(true);
-                for (Destination dest: destinations) {
-                    dest.slowConsumer(getContext(), this);
+            if ( info.getPrefetchSize() > 1 && matched.size() > info.getPrefetchSize() ) {
+                //we are slow
+                if(!isSlowConsumer()) {
+                    LOG.warn(toString() + ": has twice its prefetch limit pending, without an ack; it appears to be slow");
+                    setSlowConsumer(true);
+                    for (Destination dest: destinations) {
+                        dest.slowConsumer(getContext(), this);
+                    }
                 }
             }
             if (maximumPendingMessages != 0) {
@@ -175,7 +178,7 @@ public class TopicSubscription extends AbstractSubscription {
                             // eviction strategy
                             // for a bad strategy lets just not evict
                             if (messagesToEvict == 0) {
-                                LOG.warn("No messages to evict returned from eviction strategy: " + messageEvictionStrategy);
+                                LOG.warn("No messages to evict returned for "  + destination + " from eviction strategy: " + messageEvictionStrategy + " out of " + list.size() + " candidates");
                                 break;
                             }
                         }
@@ -192,7 +195,7 @@ public class TopicSubscription extends AbstractSubscription {
             duplicate = audit.isDuplicate(node);
             if (LOG.isDebugEnabled()) {
                 if (duplicate) {
-                    LOG.debug("ignoring duplicate add: " + node.getMessageId());
+                    LOG.debug(this + ", ignoring duplicate add: " + node.getMessageId());
                 }
             }
         }
@@ -537,7 +540,7 @@ public class TopicSubscription extends AbstractSubscription {
             destination.getDestinationStatistics().getDequeues().increment();
         }
         if (LOG.isDebugEnabled()) {
-            LOG.debug("Discarding message " + message);
+            LOG.debug(this + ", discarding message " + message);
         }
         Destination dest = message.getRegionDestination();
         if (dest != null) {
