@@ -24,6 +24,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import org.apache.activemq.broker.Broker;
 import org.apache.activemq.broker.ConnectionContext;
 import org.apache.activemq.broker.region.Destination;
+import org.apache.activemq.broker.region.IndirectMessageReference;
 import org.apache.activemq.broker.region.MessageReference;
 import org.apache.activemq.broker.region.QueueMessageReference;
 import org.apache.activemq.command.Message;
@@ -52,7 +53,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     protected Broker broker;
     private final PListStore store;
     private final String name;
-    private LinkedList<MessageReference> memoryList = new LinkedList<MessageReference>();
+    private PendingList memoryList;
     private PList diskList;
     private Iterator<MessageReference> iter;
     private Destination regionDestination;
@@ -67,6 +68,11 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
      */
     public FilePendingMessageCursor(Broker broker, String name, boolean prioritizedMessages) {
         super(prioritizedMessages);
+        if (this.prioritizedMessages) {
+            this.memoryList = new PrioritizedPendingList();
+        } else {
+            this.memoryList = new OrderedPendingList();
+        }
         this.broker = broker;
         // the store can be null if the BrokerService has persistence
         // turned off
@@ -133,9 +139,14 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     @Override
     public synchronized void release() {
         iterating = false;
+        if (iter instanceof DiskIterator) {
+           ((DiskIterator)iter).release();
+        };
         if (flushRequired) {
             flushRequired = false;
-            flushToDisk();
+            if (!hasSpace()) {
+                flushToDisk();
+            }
         }
     }
 
@@ -151,8 +162,9 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     }
 
     private void destroyDiskList() throws Exception {
-        if (!isDiskListEmpty()) {
+        if (diskList != null) {
             store.removePList(name);
+            diskList = null;
         }
     }
 
@@ -197,7 +209,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                 regionDestination = node.getMessage().getRegionDestination();
                 if (isDiskListEmpty()) {
                     if (hasSpace() || this.store == null) {
-                        memoryList.add(node);
+                        memoryList.addMessageLast(node);
                         node.incrementReferenceCount();
                         setCacheEnabled(true);
                         return true;
@@ -207,7 +219,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                     if (isDiskListEmpty()) {
                         expireOldMessages();
                         if (hasSpace()) {
-                            memoryList.add(node);
+                            memoryList.addMessageLast(node);
                             node.incrementReferenceCount();
                             return true;
                         } else {
@@ -227,7 +239,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                 throw new RuntimeException(e);
             }
         } else {
-            discard(node);
+            discardExpiredMessage(node);
         }
         //message expired
         return true;
@@ -245,7 +257,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                 regionDestination = node.getMessage().getRegionDestination();
                 if (isDiskListEmpty()) {
                     if (hasSpace()) {
-                        memoryList.addFirst(node);
+                        memoryList.addMessageFirst(node);
                         node.incrementReferenceCount();
                         setCacheEnabled(true);
                         return;
@@ -255,7 +267,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                     if (isDiskListEmpty()) {
                         expireOldMessages();
                         if (hasSpace()) {
-                            memoryList.addFirst(node);
+                            memoryList.addMessageFirst(node);
                             node.incrementReferenceCount();
                             return;
                         } else {
@@ -273,7 +285,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
                 throw new RuntimeException(e);
             }
         } else {
-            discard(node);
+            discardExpiredMessage(node);
         }
     }
 
@@ -290,15 +302,15 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
      */
     @Override
     public synchronized MessageReference next() {
-        Message message = (Message) iter.next();
-        last = message;
+        MessageReference reference = iter.next();
+        last = reference;
         if (!isDiskListEmpty()) {
             // got from disk
-            message.setRegionDestination(regionDestination);
-            message.setMemoryUsage(this.getSystemUsage().getMemoryUsage());
+            reference.getMessage().setRegionDestination(regionDestination);
+            reference.getMessage().setMemoryUsage(this.getSystemUsage().getMemoryUsage());
         }
-        message.incrementReferenceCount();
-        return message;
+        reference.incrementReferenceCount();
+        return reference;
     }
 
     /**
@@ -318,7 +330,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
      */
     @Override
     public synchronized void remove(MessageReference node) {
-        if (memoryList.remove(node)) {
+        if (memoryList.remove(node) != null) {
             node.decrementReferenceCount();
         }
         if (!isDiskListEmpty()) {
@@ -335,7 +347,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
      */
     @Override
     public synchronized int size() {
-        return memoryList.size() + (isDiskListEmpty() ? 0 : getDiskList().size());
+        return memoryList.size() + (isDiskListEmpty() ? 0 : (int)getDiskList().size());
     }
 
     /**
@@ -357,7 +369,7 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     @Override
     public synchronized boolean isFull() {
 
-        return super.isFull() || (systemUsage != null && systemUsage.getTempUsage().isFull());
+        return super.isFull() || (!isDiskListEmpty() && systemUsage != null && systemUsage.getTempUsage().isFull());
 
     }
 
@@ -374,12 +386,14 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     public void onUsageChanged(Usage usage, int oldPercentUsage, int newPercentUsage) {
         if (newPercentUsage >= getMemoryUsageHighWaterMark()) {
             synchronized (this) {
-                flushRequired = true;
-                if (!iterating) {
-                    expireOldMessages();
-                    if (!hasSpace()) {
-                        flushToDisk();
-                        flushRequired = false;
+                if (!flushRequired && size() != 0) {
+                    flushRequired =true;
+                    if (!iterating) {
+                        expireOldMessages();
+                        if (!hasSpace()) {
+                            flushToDisk();
+                            flushRequired = false;
+                        }
                     }
                 }
             }
@@ -397,25 +411,26 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
 
     protected synchronized void expireOldMessages() {
         if (!memoryList.isEmpty()) {
-            LinkedList<MessageReference> tmpList = new LinkedList<MessageReference>(this.memoryList);
-            this.memoryList = new LinkedList<MessageReference>();
-            while (!tmpList.isEmpty()) {
-                MessageReference node = tmpList.removeFirst();
+            for (Iterator<MessageReference> iterator = memoryList.iterator(); iterator.hasNext();) {
+                MessageReference node = iterator.next();
                 if (node.isExpired()) {
-                    discard(node);
-                } else {
-                    memoryList.add(node);
+                    node.decrementReferenceCount();
+                    discardExpiredMessage(node);
+                    iterator.remove();
                 }
             }
         }
-
     }
 
     protected synchronized void flushToDisk() {
-
-        if (!memoryList.isEmpty()) {
-            while (!memoryList.isEmpty()) {
-                MessageReference node = memoryList.removeFirst();
+        if (!memoryList.isEmpty() && store != null) {
+            long start = 0;
+             if (LOG.isTraceEnabled()) {
+                start = System.currentTimeMillis();
+                LOG.trace("" + name + ", flushToDisk() mem list size: " +memoryList.size()  + " " +  (systemUsage != null ? systemUsage.getMemoryUsage() : "") );
+             }
+            for (Iterator<MessageReference> iterator = memoryList.iterator(); iterator.hasNext();) {
+                MessageReference node = iterator.next();
                 node.decrementReferenceCount();
                 ByteSequence bs;
                 try {
@@ -429,6 +444,9 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
             }
             memoryList.clear();
             setCacheEnabled(false);
+             if (LOG.isTraceEnabled()) {
+                LOG.trace("" + name + ", flushToDisk() done - " + (System.currentTimeMillis() - start) + "ms " + (systemUsage != null ? systemUsage.getMemoryUsage() : ""));
+             }
         }
     }
 
@@ -448,14 +466,15 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
         return diskList;
     }
 
-    protected void discard(MessageReference message) {
-        message.decrementReferenceCount();
+    private void discardExpiredMessage(MessageReference reference) {
         if (LOG.isDebugEnabled()) {
-            LOG.debug("Discarding message " + message);
+            LOG.debug("Discarding expired message " + reference);
+        }
+        if (broker.isExpired(reference)) {
+            ConnectionContext context = new ConnectionContext(new NonCachedMessageEvaluationContext());
+            context.setBroker(broker);
+            reference.getRegionDestination().messageExpired(context, null, new IndirectMessageReference(reference.getMessage()));
         }
-        ConnectionContext ctx = new ConnectionContext(new NonCachedMessageEvaluationContext());
-        ctx.setBroker(broker);
-        broker.getRoot().sendToDeadLetterQueue(ctx, message, null);
     }
 
     protected ByteSequence getByteSequence(Message message) throws IOException {
@@ -471,35 +490,23 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
     }
 
     final class DiskIterator implements Iterator<MessageReference> {
-        private PListEntry next = null;
-        private PListEntry current = null;
-        PList list;
-
+        private final PList.PListIterator iterator;
         DiskIterator() {
             try {
-                this.list = getDiskList();
-                synchronized (this.list) {
-                    this.current = this.list.getFirst();
-                    this.next = this.current;
-                }
+                iterator = getDiskList().iterator();
             } catch (Exception e) {
                 throw new RuntimeException(e);
             }
         }
 
         public boolean hasNext() {
-            return this.next != null;
+            return iterator.hasNext();
         }
 
         public MessageReference next() {
-            this.current = next;
             try {
-                ByteSequence bs = this.current.getByteSequence();
-                synchronized (this.list) {
-                    this.current = this.list.refresh(this.current);
-                    this.next = this.list.getNext(this.current);
-                }
-                return getMessage(bs);
+                PListEntry entry = iterator.next();
+                return getMessage(entry.getByteSequence());
             } catch (IOException e) {
                 LOG.error("I/O error", e);
                 throw new RuntimeException(e);
@@ -507,18 +514,11 @@ public class FilePendingMessageCursor extends AbstractPendingMessageCursor imple
         }
 
         public void remove() {
-            try {
-                synchronized (this.list) {
-                    this.current = this.list.refresh(this.current);
-                    this.list.remove(this.current);
-                }
-
-            } catch (IOException e) {
-                LOG.error("I/O error", e);
-                throw new RuntimeException(e);
-            }
-
+            iterator.remove();
         }
 
+        public void release() {
+            iterator.release();
+        }
     }
 }
