@@ -17,8 +17,11 @@
 package org.apache.activemq.store.jdbc.adapter;
 
 import java.io.IOException;
+import java.io.PrintStream;
+import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
+import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
@@ -539,7 +542,7 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
         cleanupExclusiveLock.readLock().lock();
         try {
             s = c.getConnection().prepareStatement(this.statements.getFindDurableSubMessagesByPriorityStatement());
-            s.setMaxRows(maxRows);
+            s.setMaxRows(Math.max(maxReturned * 2, maxRows));
             s.setString(1, destination.getQualifiedName());
             s.setString(2, clientId);
             s.setString(3, subscriptionName);
@@ -739,20 +742,18 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
         }
     }
 
-    public void doDeleteOldMessages(TransactionContext c, boolean isPrioritizedMessages) throws SQLException, IOException {
+    int priorityIterator = 0;
+    public void doDeleteOldMessages(TransactionContext c) throws SQLException, IOException {
         PreparedStatement s = null;
         cleanupExclusiveLock.writeLock().lock();
         try {
-            if (isPrioritizedMessages) {
-                LOG.debug("Executing SQL: " + this.statements.getDeleteOldMessagesStatementWithPriority());
-                s = c.getConnection().prepareStatement(this.statements.getDeleteOldMessagesStatementWithPriority());
-            } else {
-                LOG.debug("Executing SQL: " + this.statements.getDeleteOldMessagesStatement());
-                s = c.getConnection().prepareStatement(this.statements.getDeleteOldMessagesStatement());
-            }
-            s.setLong(1, System.currentTimeMillis());
+            LOG.debug("Executing SQL: " + this.statements.getDeleteOldMessagesStatementWithPriority());
+            s = c.getConnection().prepareStatement(this.statements.getDeleteOldMessagesStatementWithPriority());
+            int priority = priorityIterator++%10;
+            s.setInt(1, priority);
+            s.setInt(2, priority);
             int i = s.executeUpdate();
-            LOG.debug("Deleted " + i + " old message(s).");
+            LOG.debug("Deleted " + i + " old message(s) at priority: " + priority);
         } finally {
             cleanupExclusiveLock.writeLock().unlock();
             close(s);
@@ -773,6 +774,9 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
             rs = s.executeQuery();
             if (rs.next()) {
                 result = rs.getLong(1);
+                if (result == 0 && rs.wasNull()) {
+                    result = -1;
+                }
             }
         } finally {
             cleanupExclusiveLock.readLock().unlock();
@@ -782,14 +786,14 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
         return result;
     }
 
-    private static void close(PreparedStatement s) {
+    protected static void close(PreparedStatement s) {
         try {
             s.close();
         } catch (Throwable e) {
         }
     }
 
-    private static void close(ResultSet rs) {
+    protected static void close(ResultSet rs) {
         try {
             rs.close();
         } catch (Throwable e) {
@@ -850,7 +854,30 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
 
     public void setMaxRows(int maxRows) {
         this.maxRows = maxRows;
-    }    
+    }
+
+    @Override
+    public void doRecordDestination(TransactionContext c, ActiveMQDestination destination) throws SQLException, IOException {
+        PreparedStatement s = null;
+        cleanupExclusiveLock.readLock().lock();
+        try {
+            s = c.getConnection().prepareStatement(this.statements.getCreateDurableSubStatement());
+            s.setString(1, destination.getQualifiedName());
+            s.setString(2, destination.getQualifiedName());
+            s.setString(3, destination.getQualifiedName());
+            s.setString(4, null);
+            s.setLong(5, 0);
+            s.setString(6, destination.getQualifiedName());
+            s.setLong(7, 11);  // entry out of priority range
+
+            if (s.executeUpdate() != 1) {
+                throw new IOException("Could not create ack record for destination: " + destination);
+            }
+        } finally {
+            cleanupExclusiveLock.readLock().unlock();
+            close(s);
+        }
+    }
 
     /**
      * @param c
@@ -1005,7 +1032,7 @@ public class DefaultJDBCAdapter implements JDBCAdapter {
             } catch (Throwable ignore) {
             }
         }
-    }  */  
+    }  */
 
     public long doGetLastProducerSequenceId(TransactionContext c, ProducerId id)
             throws SQLException, IOException {
