@@ -26,7 +26,7 @@ import java.util.concurrent.CancellationException;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
-import javax.transaction.xa.XAException;
+
 import org.apache.activemq.broker.ConnectionContext;
 import org.apache.activemq.command.Message;
 import org.apache.activemq.command.MessageAck;
@@ -56,8 +56,8 @@ import org.slf4j.LoggerFactory;
 /**
  * Provides a TransactionStore implementation that can create transaction aware
  * MessageStore objects from non transaction aware MessageStore objects.
- * 
- * 
+ *
+ *
  */
 public class KahaDBTransactionStore implements TransactionStore {
     static final Logger LOG = LoggerFactory.getLogger(KahaDBTransactionStore.class);
@@ -120,7 +120,7 @@ public class KahaDBTransactionStore implements TransactionStore {
                 cmd.run();
                 results.add(cmd.run());
             }
-            
+
             return results;
         }
     }
@@ -157,11 +157,21 @@ public class KahaDBTransactionStore implements TransactionStore {
                 KahaDBTransactionStore.this.addMessage(context, getDelegate(), send);
             }
 
+            @Override
+            public void addMessage(ConnectionContext context, final Message send, boolean canOptimize) throws IOException {
+                KahaDBTransactionStore.this.addMessage(context, getDelegate(), send);
+            }
+
             @Override
             public Future<Object> asyncAddQueueMessage(ConnectionContext context, Message message) throws IOException {
                 return KahaDBTransactionStore.this.asyncAddQueueMessage(context, getDelegate(), message);
             }
 
+            @Override
+            public Future<Object> asyncAddQueueMessage(ConnectionContext context, Message message, boolean canOptimize) throws IOException {
+                return KahaDBTransactionStore.this.asyncAddQueueMessage(context, getDelegate(), message);
+            }
+
             @Override
             public void removeMessage(ConnectionContext context, final MessageAck ack) throws IOException {
                 KahaDBTransactionStore.this.removeMessage(context, getDelegate(), ack);
@@ -181,11 +191,21 @@ public class KahaDBTransactionStore implements TransactionStore {
                 KahaDBTransactionStore.this.addMessage(context, getDelegate(), send);
             }
 
+            @Override
+            public void addMessage(ConnectionContext context, final Message send, boolean canOptimize) throws IOException {
+                KahaDBTransactionStore.this.addMessage(context, getDelegate(), send);
+            }
+
             @Override
             public Future<Object> asyncAddTopicMessage(ConnectionContext context, Message message) throws IOException {
                 return KahaDBTransactionStore.this.asyncAddTopicMessage(context, getDelegate(), message);
             }
 
+            @Override
+            public Future<Object> asyncAddTopicMessage(ConnectionContext context, Message message, boolean canOptimize) throws IOException {
+                return KahaDBTransactionStore.this.asyncAddTopicMessage(context, getDelegate(), message);
+            }
+
             @Override
             public void removeMessage(ConnectionContext context, final MessageAck ack) throws IOException {
                 KahaDBTransactionStore.this.removeMessage(context, getDelegate(), ack);
@@ -211,9 +231,15 @@ public class KahaDBTransactionStore implements TransactionStore {
      * @see org.apache.activemq.store.TransactionStore#prepare(TransactionId)
      */
     public void prepare(TransactionId txid) throws IOException {
-        inflightTransactions.remove(txid);
         KahaTransactionInfo info = getTransactionInfo(txid);
-        theStore.store(new KahaPrepareCommand().setTransactionInfo(info), true, null, null);
+        if (txid.isXATransaction() || theStore.isConcurrentStoreAndDispatchTransactions() == false) {
+            theStore.store(new KahaPrepareCommand().setTransactionInfo(info), true, null, null);
+        } else {
+            Tx tx = inflightTransactions.remove(txid);
+            if (tx != null) {
+               theStore.store(new KahaPrepareCommand().setTransactionInfo(info), true, null, null);
+            }
+        }
     }
 
     public Tx getTx(Object txid) {
@@ -265,10 +291,8 @@ public class KahaDBTransactionStore implements TransactionStore {
 
             } else {
                 KahaTransactionInfo info = getTransactionInfo(txid);
-                // ensure message order w.r.t to cursor and store for setBatch()
-                synchronized (this) {
-                    theStore.store(new KahaCommitCommand().setTransactionInfo(info), true, preCommit, postCommit);
-                }
+                theStore.store(new KahaCommitCommand().setTransactionInfo(info), true, preCommit, postCommit);
+                forgetRecoveredAcks(txid);
             }
         }else {
            LOG.error("Null transaction passed on commit");
@@ -283,11 +307,19 @@ public class KahaDBTransactionStore implements TransactionStore {
         if (txid.isXATransaction() || theStore.isConcurrentStoreAndDispatchTransactions() == false) {
             KahaTransactionInfo info = getTransactionInfo(txid);
             theStore.store(new KahaRollbackCommand().setTransactionInfo(info), false, null, null);
+            forgetRecoveredAcks(txid);
         } else {
             inflightTransactions.remove(txid);
         }
     }
 
+    protected void forgetRecoveredAcks(TransactionId txid) throws IOException {
+        if (txid.isXATransaction()) {
+            XATransactionId xaTid = ((XATransactionId) txid);
+            theStore.forgetRecoveredAcks(xaTid.getPreparedAcks());
+        }
+    }
+
     public void start() throws Exception {
     }
 
@@ -295,8 +327,6 @@ public class KahaDBTransactionStore implements TransactionStore {
     }
 
     public synchronized void recover(TransactionRecoveryListener listener) throws IOException {
-        // All the inflight transactions get rolled back..
-        // inflightTransactions.clear();
         for (Map.Entry<TransactionId, List<Operation>> entry : theStore.preparedTransactions.entrySet()) {
             XATransactionId xid = (XATransactionId) entry.getKey();
             ArrayList<Message> messageList = new ArrayList<Message>();
@@ -320,6 +350,8 @@ public class KahaDBTransactionStore implements TransactionStore {
             MessageAck[] acks = new MessageAck[ackList.size()];
             messageList.toArray(addedMessages);
             ackList.toArray(acks);
+            xid.setPreparedAcks(ackList);
+            theStore.trackRecoveredAcks(ackList);
             listener.recover(xid, addedMessages, acks);
         }
     }
@@ -396,7 +428,7 @@ public class KahaDBTransactionStore implements TransactionStore {
                         return message;
                     }
                     @Override
-                    public Future run(ConnectionContext ctx) throws IOException {
+                    public Future<Object> run(ConnectionContext ctx) throws IOException {
                         return destination.asyncAddTopicMessage(ctx, message);
                     }
 
@@ -490,7 +522,6 @@ public class KahaDBTransactionStore implements TransactionStore {
 
 
     private KahaTransactionInfo getTransactionInfo(TransactionId txid) {
-        return theStore.createTransactionInfo(txid);
+        return theStore.getTransactionIdTransformer().transform(txid);
     }
-
 }
