@@ -1,5 +1,5 @@
 /**
- * Licensed to the Apache Software Foundation (ASF) under one or more
+gxfdgvdfg * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
@@ -23,6 +23,7 @@ import java.io.InterruptedIOException;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.Socket;
+import java.net.SocketAddress;
 import java.net.SocketException;
 import java.net.SocketTimeoutException;
 import java.net.URI;
@@ -30,12 +31,11 @@ import java.net.UnknownHostException;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.SynchronousQueue;
-import java.util.concurrent.ThreadFactory;
-import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
+
 import javax.net.SocketFactory;
+
 import org.apache.activemq.Service;
 import org.apache.activemq.thread.DefaultThreadPools;
 import org.apache.activemq.transport.Transport;
@@ -50,9 +50,9 @@ import org.slf4j.LoggerFactory;
 
 /**
  * An implementation of the {@link Transport} interface using raw tcp/ip
- * 
+ *
  * @author David Martin Clavo david(dot)martin(dot)clavo(at)gmail.com (logging improvement modifications)
- * 
+ *
  */
 public class TcpTransport extends TransportThreadSupport implements Transport, Service, Runnable {
     private static final Logger LOG = LoggerFactory.getLogger(TcpTransport.class);
@@ -69,6 +69,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
     protected DataOutputStream dataOut;
     protected DataInputStream dataIn;
     protected TimeStampStream buffOut = null;
+
     /**
      * The Traffic Class to be set on the socket.
      */
@@ -129,6 +130,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
     protected final AtomicReference<CountDownLatch> stoppedLatch = new AtomicReference<CountDownLatch>();
 
     private Map<String, Object> socketOptions;
+    private int soLinger = Integer.MIN_VALUE;
     private Boolean keepAlive;
     private Boolean tcpNoDelay;
     private Thread runnerThread;
@@ -136,7 +138,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
 
     /**
      * Connect to a remote Node - e.g. a Broker
-     * 
+     *
      * @param wireFormat
      * @param socketFactory
      * @param remoteLocation
@@ -160,7 +162,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
 
     /**
      * Initialize from a server Socket
-     * 
+     *
      * @param wireFormat
      * @param socket
      * @throws IOException
@@ -206,7 +208,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
             onException(e);
         } catch (Throwable e){
             stoppedLatch.get().countDown();
-            IOException ioe=new IOException("Unexpected error occured");
+            IOException ioe=new IOException("Unexpected error occured: " + e);
             ioe.initCause(e);
             onException(ioe);
         }finally {
@@ -247,7 +249,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         // reflect the value returned by Socket.getTrafficClass().
         return this.trafficClass;
     }
-  
+
     public void setTypeOfService(int typeOfService) {
         this.trafficClass = QualityOfServiceUtils.getToS(typeOfService);
         this.typeOfServiceChosen = true;
@@ -260,7 +262,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
     public void setTrace(boolean trace) {
         this.trace = trace;
     }
-    
+
     public String getLogWriterName() {
         return logWriterName;
     }
@@ -292,7 +294,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
     public void setJmxPort(int jmxPort) {
         this.jmxPort = jmxPort;
     }
-    
+
     public int getMinmumWireFormatVersion() {
         return minmumWireFormatVersion;
     }
@@ -358,6 +360,18 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         this.keepAlive = keepAlive;
     }
 
+    /**
+     * Enable/disable soLinger
+     * @param soLinger enabled if > -1, disabled if == -1, system default otherwise
+     */
+    public void setSoLinger(int soLinger) {
+        this.soLinger = soLinger;
+    }
+
+    public int getSoLinger() {
+        return soLinger;
+    }
+
     public Boolean getTcpNoDelay() {
         return tcpNoDelay;
     }
@@ -382,7 +396,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
     public void setIoBufferSize(int ioBufferSize) {
         this.ioBufferSize = ioBufferSize;
     }
-    
+
     /**
      * @return the closeAsync
      */
@@ -411,7 +425,7 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
 
     /**
      * Configures the socket for use
-     * 
+     *
      * @param sock
      * @throws SocketException, IllegalArgumentException if setting the options
      *         on the socket failed.
@@ -434,6 +448,12 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         if (keepAlive != null) {
             sock.setKeepAlive(keepAlive.booleanValue());
         }
+
+        if (soLinger > -1) {
+            sock.setSoLinger(true, soLinger);
+        } else if (soLinger == -1) {
+            sock.setSoLinger(false, 0);
+        }
         if (tcpNoDelay != null) {
             sock.setTcpNoDelay(tcpNoDelay.booleanValue());
         }
@@ -513,31 +533,37 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         // closeStreams();
         if (socket != null) {
             if (closeAsync) {
-                //closing the socket can hang also 
+                //closing the socket can hang also
                 final CountDownLatch latch = new CountDownLatch(1);
-                
+
                 DefaultThreadPools.getDefaultTaskRunnerFactory().execute(new Runnable() {
-    
                     public void run() {
                         try {
                             socket.close();
                         } catch (IOException e) {
-                            LOG.debug("Caught exception closing socket",e);
-                        }finally {
+                            if (LOG.isDebugEnabled()) {
+                                LOG.debug("Caught exception closing socket", e);
+                            }
+                        } finally {
                             latch.countDown();
                         }
                     }
-                    
                 });
-                latch.await(1,TimeUnit.SECONDS);
-            }else {
+
+                try {
+                    latch.await(1,TimeUnit.SECONDS);
+                } catch (InterruptedException e) {
+                    Thread.currentThread().interrupt();
+                }
+
+            } else {
+
                 try {
                     socket.close();
                 } catch (IOException e) {
                     LOG.debug("Caught exception closing socket",e);
                 }
             }
-           
         }
     }
 
@@ -597,11 +623,16 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
 
     public String getRemoteAddress() {
         if (socket != null) {
-            return "" + socket.getRemoteSocketAddress();
+            SocketAddress address = socket.getRemoteSocketAddress();
+            if (address instanceof InetSocketAddress) {
+                return "tcp://" + ((InetSocketAddress)address).getAddress().getHostAddress() + ":" + ((InetSocketAddress)address).getPort();
+            } else {
+                return "" + socket.getRemoteSocketAddress();
+            }
         }
         return null;
     }
-    
+
     @Override
     public <T> T narrow(Class<T> target) {
         if (target == Socket.class) {
@@ -611,11 +642,10 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         }
         return super.narrow(target);
     }
-    
+
     public int getReceiveCounter() {
         return receiveCounter;
     }
-    
 
     /**
      * @param sock The socket on which to set the Traffic Class.
@@ -667,4 +697,8 @@ public class TcpTransport extends TransportThreadSupport implements Transport, S
         this.typeOfServiceChosen = false;
         return true;
     }
+
+    public WireFormat getWireFormat() {
+        return wireFormat;
+    }
 }
