@@ -48,15 +48,20 @@ import org.apache.activemq.advisory.ConsumerListener;
 import org.apache.activemq.broker.BrokerFactory;
 import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.broker.TransportConnector;
+import org.apache.activemq.broker.region.RegionBroker;
+import org.apache.activemq.broker.region.TopicRegion;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.command.ActiveMQTopic;
 import org.apache.activemq.network.DiscoveryNetworkConnector;
+import org.apache.activemq.network.NetworkBridge;
 import org.apache.activemq.network.NetworkConnector;
 import org.apache.activemq.util.IdGenerator;
 import org.apache.activemq.util.MessageIdList;
 import org.apache.activemq.util.Wait;
 import org.apache.activemq.xbean.BrokerFactoryBean;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.core.io.Resource;
 
 /**
@@ -66,6 +71,7 @@ import org.springframework.core.io.Resource;
  * 
  */
 public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
+    private static final Logger LOG = LoggerFactory.getLogger(JmsMultipleBrokersTestSupport.class);
     public static final String AUTO_ASSIGN_TRANSPORT = "tcp://localhost:0";
     public static int maxSetupTime = 5000;
 
@@ -81,11 +87,11 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
         return bridgeBrokers(localBrokerName, remoteBrokerName, false, 1, true);
     }
 
-    protected void bridgeBrokers(String localBrokerName, String remoteBrokerName, boolean dynamicOnly) throws Exception {
+    protected NetworkConnector bridgeBrokers(String localBrokerName, String remoteBrokerName, boolean dynamicOnly) throws Exception {
         BrokerService localBroker = brokers.get(localBrokerName).broker;
         BrokerService remoteBroker = brokers.get(remoteBrokerName).broker;
 
-        bridgeBrokers(localBroker, remoteBroker, dynamicOnly, 1, true, false);
+        return bridgeBrokers(localBroker, remoteBroker, dynamicOnly, 1, true, false);
     }
 
     protected NetworkConnector bridgeBrokers(String localBrokerName, String remoteBrokerName, boolean dynamicOnly, int networkTTL, boolean conduit) throws Exception {
@@ -108,6 +114,7 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
                 uri = "static:(failover:(" + remoteURI + "))";
             }
             NetworkConnector connector = new DiscoveryNetworkConnector(new URI(uri));
+            connector.setName("to-" + remoteBroker.getBrokerName());
             connector.setDynamicOnly(dynamicOnly);
             connector.setNetworkTTL(networkTTL);
             connector.setConduitSubscriptions(conduit);
@@ -170,12 +177,31 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
         if (!broker.getNetworkConnectors().isEmpty()) {
             result = Wait.waitFor(new Wait.Condition() {
                 public boolean isSatisified() throws Exception {
-                    return (broker.getNetworkConnectors().get(bridgeIndex).activeBridges().size() >= min);
+                    int activeCount = 0;
+                    for (NetworkBridge bridge : broker.getNetworkConnectors().get(bridgeIndex).activeBridges()) {
+                        if (bridge.getRemoteBrokerName() != null) {
+                            LOG.info("found bridge[" + bridge + "] to " + bridge.getRemoteBrokerName() + " on broker :" + broker.getBrokerName());
+                            activeCount++;
+                        }
+                    }
+                    return activeCount >= min;
                 }}, wait);
         }
         return result;
     }
 
+    protected void waitForMinTopicRegionConsumerCount(final String name, final int count) throws Exception {
+        final BrokerService broker = brokers.get(name).broker;
+        final TopicRegion topicRegion =  (TopicRegion) ((RegionBroker) broker.getRegionBroker()).getTopicRegion();
+        assertTrue("found expected consumers in topic region of" + name, Wait.waitFor(new Wait.Condition() {
+            @Override
+            public boolean isSatisified() throws Exception {
+                LOG.info("topic consumers: " + name +", " +  topicRegion.getSubscriptions().toString());
+                return topicRegion.getSubscriptions().size()  >= count;
+            }
+        }));
+    }
+
     protected void waitForBridgeFormation() throws Exception {
         waitForBridgeFormation(1);
     }
@@ -201,11 +227,15 @@ public class JmsMultipleBrokersTestSupport extends CombinationTestSupport {
 
     protected BrokerService createBroker(URI brokerUri) throws Exception {
         BrokerService broker = BrokerFactory.createBroker(brokerUri);
+        configureBroker(broker);
         brokers.put(broker.getBrokerName(), new BrokerItem(broker));
 
         return broker;
     }
 
+    protected void configureBroker(BrokerService broker) {
+    }
+
     protected BrokerService createBroker(Resource configFile) throws Exception {
         BrokerFactoryBean brokerFactory = new BrokerFactoryBean(configFile);
         brokerFactory.afterPropertiesSet();
