@@ -22,13 +22,17 @@ import java.net.URI;
 import java.net.URL;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Set;
 
 import javax.jms.BytesMessage;
 import javax.jms.Connection;
+import javax.jms.ConnectionFactory;
+import javax.jms.Destination;
 import javax.jms.Message;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
 import javax.jms.Session;
+import javax.jms.Topic;
 import javax.management.MBeanServer;
 import javax.management.MBeanServerInvocationHandler;
 import javax.management.MalformedObjectNameException;
@@ -47,7 +51,6 @@ import org.apache.activemq.broker.region.BaseDestination;
 import org.apache.activemq.broker.region.policy.PolicyEntry;
 import org.apache.activemq.broker.region.policy.PolicyMap;
 import org.apache.activemq.broker.region.policy.SharedDeadLetterStrategy;
-import org.apache.activemq.command.ActiveMQBlobMessage;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.command.ActiveMQTempQueue;
@@ -59,8 +62,6 @@ import org.slf4j.LoggerFactory;
  * A test case of the various MBeans in ActiveMQ. If you want to look at the
  * various MBeans after the test has been run then run this test case as a
  * command line application.
- * 
- * 
  */
 public class MBeanTest extends EmbeddedBrokerTestSupport {
     private static final Logger LOG = LoggerFactory.getLogger(MBeanTest.class);
@@ -85,12 +86,13 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         waitForKeyPress = true;
         TestRunner.run(MBeanTest.class);
     }
-    
+
     public void testConnectors() throws Exception{
         ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
         BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
-        assertEquals("openwire URL port doesn't equal bind Address", new URI(broker.getOpenWireURL()).getPort(), new URI(this.bindAddress).getPort());
-
+        assertEquals("openwire URL port doesn't equal bind Address",
+                     new URI(broker.getOpenWireURL()).getPort(),
+                     new URI(this.broker.getTransportConnectors().get(0).getPublishableConnectString()).getPort());
     }
 
     public void testMBeans() throws Exception {
@@ -103,6 +105,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertQueueBrowseWorks();
         assertCreateAndDestroyDurableSubscriptions();
         assertConsumerCounts();
+        assertProducerCounts();
     }
 
     public void testMoveMessages() throws Exception {
@@ -154,7 +157,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         long newQueuesize = queueNew.getQueueSize();
         echo("Second queue size: " + newQueuesize);
         assertEquals("Unexpected number of messages ",messageCount, newQueuesize);
-        
+
         // check memory usage migration
         assertTrue("new dest has some memory usage", queueNew.getMemoryPercentUsage() > 0);
         assertEquals("old dest has no memory usage", 0, queue.getMemoryPercentUsage());
@@ -208,7 +211,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         connection = connectionFactory.createConnection();
         useConnection(connection);
 
-
         ObjectName queueViewMBeanName = assertRegisteredObjectName(domain + ":Type=Queue,Destination=" + getDestinationString() + ",BrokerName=localhost");
         QueueViewMBean queue = (QueueViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, queueViewMBeanName, QueueViewMBean.class, true);
 
@@ -228,7 +230,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         consumer.close();
         session.close();
 
-
         // now lets get the dead letter queue
         Thread.sleep(1000);
 
@@ -256,7 +257,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         int dlqMemUsage = dlq.getMemoryPercentUsage();
         assertTrue("dlq has some memory usage", dlqMemUsage > 0);
         assertEquals("dest has no memory usage", 0, queue.getMemoryPercentUsage());
-        
 
         echo("About to retry " + messageCount + " messages");
 
@@ -277,7 +277,7 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("DLQ size", initialDlqSize - messageCount, dlqSize);
         assertEquals("queue size", initialQueueSize, queueSize);
         assertEquals("browse queue size", initialQueueSize, actualCount);
-        
+
         assertEquals("dest has some memory usage", dlqMemUsage, queue.getMemoryPercentUsage());
     }
 
@@ -315,10 +315,9 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
 
         String newDestination = getSecondDestinationString();
         long queueSize = queue.getQueueSize();
+        assertTrue(queueSize > 0);
         queue.copyMatchingMessagesTo("counter > 2", newDestination);
 
-        
-
         queueViewMBeanName = assertRegisteredObjectName(domain + ":Type=Queue,Destination=" + newDestination + ",BrokerName=localhost");
 
         queue = (QueueViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, queueViewMBeanName, QueueViewMBean.class, true);
@@ -332,7 +331,46 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("dest has no memory usage", 0, queue.getMemoryPercentUsage());
     }
 
+    public void testCreateDestinationWithSpacesAtEnds() throws Exception {
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
 
+        assertTrue("broker is not a slave", !broker.isSlave());
+        // create 2 topics
+        broker.addTopic(getDestinationString() + "1 ");
+        broker.addTopic(" " + getDestinationString() + "2");
+        broker.addTopic(" " + getDestinationString() + "3 ");
+
+        assertNotRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "1 ");
+        assertNotRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination= " + getDestinationString() + "2");
+        assertNotRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination= " + getDestinationString() + "3 ");
+
+        ObjectName topicObjName1 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "1");
+        ObjectName topicObjName2 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "2");
+        ObjectName topicObjName3 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "3");
+
+        TopicViewMBean topic1 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName1, TopicViewMBean.class, true);
+        TopicViewMBean topic2 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName2, TopicViewMBean.class, true);
+        TopicViewMBean topic3 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName3, TopicViewMBean.class, true);
+
+        assertEquals("topic1 Durable subscriber count", 0, topic1.getConsumerCount());
+        assertEquals("topic2 Durable subscriber count", 0, topic2.getConsumerCount());
+        assertEquals("topic3 Durable subscriber count", 0, topic3.getConsumerCount());
+
+        String topicName = getDestinationString();
+        String selector = null;
+
+        // create 1 subscriber for each topic
+        broker.createDurableSubscriber(clientID, "topic1.subscriber1", topicName + "1", selector);
+        broker.createDurableSubscriber(clientID, "topic2.subscriber1", topicName + "2", selector);
+        broker.createDurableSubscriber(clientID, "topic3.subscriber1", topicName + "3", selector);
+
+        assertEquals("topic1 Durable subscriber count", 1, topic1.getConsumerCount());
+        assertEquals("topic2 Durable subscriber count", 1, topic2.getConsumerCount());
+        assertEquals("topic3 Durable subscriber count", 1, topic3.getConsumerCount());
+    }
+
+    @SuppressWarnings("rawtypes")
     protected void assertSendViaMBean() throws Exception {
         String queueName = getDestinationString() + ".SendMBBean";
 
@@ -347,12 +385,12 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         QueueViewMBean proxy = (QueueViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, queueViewMBeanName, QueueViewMBean.class, true);
 
         proxy.purge();
-        
+
         int count = 5;
         for (int i = 0; i < count; i++) {
             String body = "message:" + i;
 
-            Map headers = new HashMap();
+            Map<String, Object> headers = new HashMap<String, Object>();
             headers.put("JMSCorrelationID", "MyCorrId");
             headers.put("JMSDeliveryMode", Boolean.FALSE);
             headers.put("JMSXGroupID", "MyGroupID");
@@ -364,12 +402,11 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
 
             proxy.sendTextMessage(headers, body);
         }
-        
+
         CompositeData[] compdatalist = proxy.browse();
         if (compdatalist.length == 0) {
             fail("There is no message in the queue:");
         }
-        String[] messageIDs = new String[compdatalist.length];
 
         for (int i = 0; i < compdatalist.length; i++) {
             CompositeData cdata = compdatalist[i];
@@ -406,7 +443,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
             assertComplexData(i, cdata, "JMSXGroupSeq", 1234);
             assertComplexData(i, cdata, "JMSXGroupID", "MyGroupID");
             assertComplexData(i, cdata, "Text", "message:" + i);
-
         }
     }
 
@@ -415,7 +451,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("Message " + messageIndex + " CData field: " + name, expected, value);
     }
 
-
     protected void assertQueueBrowseWorks() throws Exception {
         Integer mbeancnt = mbeanServer.getMBeanCount();
         echo("Mbean count :" + mbeancnt);
@@ -546,6 +581,80 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         assertEquals("topic2 Durable subscriber count", 0, topic2.getConsumerCount());
     }
 
+    protected void assertProducerCounts() throws Exception {
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        assertTrue("broker is not a slave", !broker.isSlave());
+        // create 2 topics
+        broker.addTopic(getDestinationString() + "1");
+        broker.addTopic(getDestinationString() + "2");
+
+        ObjectName topicObjName1 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "1");
+        ObjectName topicObjName2 = assertRegisteredObjectName(domain + ":Type=Topic,BrokerName=localhost,Destination=" + getDestinationString() + "2");
+        TopicViewMBean topic1 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName1, TopicViewMBean.class, true);
+        TopicViewMBean topic2 = (TopicViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, topicObjName2, TopicViewMBean.class, true);
+
+        assertEquals("topic1 Producer count", 0, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+        assertEquals("broker Topic Producer count", 0, broker.getTopicProducers().length);
+
+        // create 1 producer for each topic
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        Destination dest1 = session.createTopic(getDestinationString() + "1");
+        Destination dest2 = session.createTopic(getDestinationString() + "2");
+        MessageProducer producer1 = session.createProducer(dest1);
+        MessageProducer producer2 = session.createProducer(dest2);
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 2, broker.getTopicProducers().length);
+
+        // create 1 more producer for topic1
+        MessageProducer producer3 = session.createProducer(dest1);
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 2, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 3, broker.getTopicProducers().length);
+
+        // destroy topic1 producer
+        producer1.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 1, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 2, broker.getTopicProducers().length);
+
+        // destroy topic2 producer
+        producer2.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 1, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+
+        assertEquals("broker Topic Producer count", 1, broker.getTopicProducers().length);
+
+        // destroy remaining topic1 producer
+        producer3.close();
+        Thread.sleep(500);
+
+        assertEquals("topic1 Producer count", 0, topic1.getProducerCount());
+        assertEquals("topic2 Producer count", 0, topic2.getProducerCount());
+
+        MessageProducer producer4 = session.createProducer(null);
+        Thread.sleep(500);
+        assertEquals(1, broker.getDynamicDestinationProducers().length);
+        producer4.close();
+        Thread.sleep(500);
+
+        assertEquals("broker Topic Producer count", 0, broker.getTopicProducers().length);
+    }
+
     protected ObjectName assertRegisteredObjectName(String name) throws MalformedObjectNameException, NullPointerException {
         ObjectName objectName = new ObjectName(name);
         if (mbeanServer.isRegistered(objectName)) {
@@ -556,8 +665,18 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         return objectName;
     }
 
+    protected ObjectName assertNotRegisteredObjectName(String name) throws MalformedObjectNameException, NullPointerException {
+        ObjectName objectName = new ObjectName(name);
+        if (mbeanServer.isRegistered(objectName)) {
+            fail("Found the MBean!: " + objectName);
+        } else {
+            echo("Bean not registered Registered: " + objectName);
+        }
+        return objectName;
+    }
+
     protected void setUp() throws Exception {
-        bindAddress = "tcp://localhost:61616";
+        bindAddress = "tcp://localhost:0";
         useTopic = false;
         super.setUp();
         mbeanServer = broker.getManagementContext().getMBeanServer();
@@ -581,19 +700,24 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         super.tearDown();
     }
 
+    @Override
+    protected ConnectionFactory createConnectionFactory() throws Exception {
+        return new ActiveMQConnectionFactory(broker.getTransportConnectors().get(0).getPublishableConnectString());
+    }
+
     protected BrokerService createBroker() throws Exception {
         BrokerService answer = new BrokerService();
         answer.setPersistent(false);
         answer.setDeleteAllMessagesOnStartup(true);
         answer.setUseJmx(true);
-       
+
         // apply memory limit so that %usage is visible
         PolicyMap policyMap = new PolicyMap();
         PolicyEntry defaultEntry = new PolicyEntry();
         defaultEntry.setMemoryLimit(1024*1024*4);
         policyMap.setDefaultEntry(defaultEntry);
         answer.setDestinationPolicy(policyMap);
-        
+
         answer.addConnector(bindAddress);
         return answer;
     }
@@ -616,7 +740,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         Thread.sleep(1000);
     }
 
-    
     protected void useConnectionWithBlobMessage(Connection connection) throws Exception {
         connection.setClientID(clientID);
         connection.start();
@@ -658,22 +781,74 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         LOG.info(text);
     }
 
-
     protected String getSecondDestinationString() {
         return "test.new.destination." + getClass() + "." + getName();
     }
 
+    public void testDynamicProducerView() throws Exception {
+        connection = connectionFactory.createConnection();
+
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        assertTrue("broker is not a slave", !broker.isSlave());
+        assertEquals(0, broker.getDynamicDestinationProducers().length);
+
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = session.createProducer(null);
+
+        Destination dest1 = session.createTopic("DynamicDest-1");
+        Destination dest2 = session.createTopic("DynamicDest-2");
+        Destination dest3 = session.createQueue("DynamicDest-3");
+
+        // Wait a bit to let the producer get registered.
+        Thread.sleep(100);
+
+        assertEquals(1, broker.getDynamicDestinationProducers().length);
+
+        ObjectName viewName = broker.getDynamicDestinationProducers()[0];
+        assertNotNull(viewName);
+        ProducerViewMBean view = (ProducerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, viewName, ProducerViewMBean.class, true);
+        assertNotNull(view);
+
+        assertEquals("NOTSET", view.getDestinationName());
+
+        producer.send(dest1, session.createTextMessage("Test Message 1"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest1).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationTopic());
+        assertFalse(view.isDestinationQueue());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.send(dest2, session.createTextMessage("Test Message 2"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest2).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationTopic());
+        assertFalse(view.isDestinationQueue());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.send(dest3, session.createTextMessage("Test Message 3"));
+        Thread.sleep(200);
+        assertEquals(((ActiveMQDestination)dest3).getPhysicalName(), view.getDestinationName());
+        assertTrue(view.isDestinationQueue());
+        assertFalse(view.isDestinationTopic());
+        assertFalse(view.isDestinationTemporary());
+
+        producer.close();
+        Thread.sleep(200);
+        assertEquals(0, broker.getDynamicDestinationProducers().length);
+    }
 
     public void testTempQueueJMXDelete() throws Exception {
         connection = connectionFactory.createConnection();
-        
+
         connection.setClientID(clientID);
         connection.start();
         Session session = connection.createSession(transacted, authMode);
         ActiveMQTempQueue tQueue = (ActiveMQTempQueue) session.createTemporaryQueue();
         Thread.sleep(1000);
         ObjectName queueViewMBeanName = assertRegisteredObjectName(domain + ":Type="+  JMXSupport.encodeObjectNamePart(tQueue.getDestinationTypeAsString())+",Destination=" + JMXSupport.encodeObjectNamePart(tQueue.getPhysicalName()) + ",BrokerName=localhost");
-        
+
         // should not throw an exception
         mbeanServer.getObjectInstance(queueViewMBeanName);
 
@@ -687,7 +862,6 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
         } catch (Exception e) {
             // expected!
         }
-
     }
 
     // Test for AMQ-3029
@@ -713,13 +887,197 @@ public class MBeanTest extends EmbeddedBrokerTestSupport {
             CompositeData cdata = compdatalist[i];
             String messageID = (String) cdata.get("JMSMessageID");
             assertNotNull("Should have a message ID for message " + i, messageID);
-            
+
             messageIDs[i] = messageID;
         }
 
         assertTrue("dest has some memory usage", queue.getMemoryPercentUsage() > 0);
     }
 
+    public void testSubscriptionViewToConnectionMBean() throws Exception {
+
+        connection = connectionFactory.createConnection("admin", "admin");
+        connection.setClientID("MBeanTest");
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        Destination queue = session.createQueue(getDestinationString() + ".Queue");
+        MessageConsumer queueConsumer = session.createConsumer(queue);
+        MessageProducer producer = session.createProducer(queue);
+
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        Thread.sleep(100);
+
+        assertTrue(broker.getQueueSubscribers().length == 1);
+
+        ObjectName subscriptionName = broker.getQueueSubscribers()[0];
+        LOG.info("Looking for Subscription: " + subscriptionName);
+
+        SubscriptionViewMBean subscriberView =
+            (SubscriptionViewMBean)MBeanServerInvocationHandler.newProxyInstance(
+                    mbeanServer, subscriptionName, SubscriptionViewMBean.class, true);
+        assertNotNull(subscriberView);
+
+        ObjectName connectionName = subscriberView.getConnection();
+        LOG.info("Looking for Connection: " + connectionName);
+        assertNotNull(connectionName);
+        ConnectionViewMBean connectionView =
+            (ConnectionViewMBean)MBeanServerInvocationHandler.newProxyInstance(
+                    mbeanServer, connectionName, ConnectionViewMBean.class, true);
+        assertNotNull(connectionView);
+
+        // Our consumer plus one advisory consumer.
+        assertEquals(2, connectionView.getConsumers().length);
+
+        // Check that the subscription view we found earlier is in this list.
+        boolean found = false;
+        for (ObjectName name : connectionView.getConsumers()) {
+            if (name.equals(subscriptionName)) {
+                found = true;
+            }
+        }
+        assertTrue("We should have found: " + subscriptionName, found);
+
+        // Our producer and no others.
+        assertEquals(1, connectionView.getProducers().length);
+
+        // Bean should detect the updates.
+        queueConsumer.close();
+        producer.close();
+
+        Thread.sleep(200);
+
+        // Only an advisory consumers now.
+        assertEquals(1, connectionView.getConsumers().length);
+        assertEquals(0, connectionView.getProducers().length);
+    }
+
+    public void testCreateAndUnsubscribeDurableSubscriptions() throws Exception {
+
+        connection = connectionFactory.createConnection("admin", "admin");
+        connection.setClientID("MBeanTest");
+
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        String topicName = getDestinationString() + ".DurableTopic";
+        Topic topic = session.createTopic(topicName);
+
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        echo("Create QueueView MBean...");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        assertEquals("Durable subscriber count", 0, broker.getDurableTopicSubscribers().length);
+        assertEquals("Durable subscriber count", 0, broker.getInactiveDurableTopicSubscribers().length);
+
+        MessageConsumer durableConsumer1 = session.createDurableSubscriber(topic, "subscription1");
+        MessageConsumer durableConsumer2 = session.createDurableSubscriber(topic, "subscription2");
+
+        Thread.sleep(100);
+
+        assertEquals("Durable subscriber count", 2, broker.getDurableTopicSubscribers().length);
+        assertEquals("Durable subscriber count", 0, broker.getInactiveDurableTopicSubscribers().length);
+
+        durableConsumer1.close();
+        durableConsumer2.close();
+
+        Thread.sleep(100);
+
+        assertEquals("Durable subscriber count", 0, broker.getDurableTopicSubscribers().length);
+        assertEquals("Durable subscriber count", 2, broker.getInactiveDurableTopicSubscribers().length);
+
+        session.unsubscribe("subscription1");
+
+        Thread.sleep(100);
+
+        assertEquals("Inactive Durable subscriber count", 1, broker.getInactiveDurableTopicSubscribers().length);
+
+        session.unsubscribe("subscription2");
+
+        assertEquals("Inactive Durable subscriber count", 0, broker.getInactiveDurableTopicSubscribers().length);
+    }
+
+    public void testUserNamePopulated() throws Exception {
+        doTestUserNameInMBeans(true);
+    }
+
+    public void testUserNameNotPopulated() throws Exception {
+        doTestUserNameInMBeans(false);
+    }
+
+    @SuppressWarnings("unused")
+    private void doTestUserNameInMBeans(boolean expect) throws Exception {
+        broker.setPopulateUserNameInMBeans(expect);
+
+        connection = connectionFactory.createConnection("admin", "admin");
+        connection.setClientID("MBeanTest");
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        Destination queue = session.createQueue(getDestinationString() + ".Queue");
+        Topic topic = session.createTopic(getDestinationString() + ".Topic");
+        MessageProducer producer = session.createProducer(queue);
+        MessageConsumer queueConsumer = session.createConsumer(queue);
+        MessageConsumer topicConsumer = session.createConsumer(topic);
+        MessageConsumer durable = session.createDurableSubscriber(topic, "Durable");
+
+        ObjectName brokerName = assertRegisteredObjectName(domain + ":Type=Broker,BrokerName=localhost");
+        BrokerViewMBean broker = (BrokerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, brokerName, BrokerViewMBean.class, true);
+
+        Thread.sleep(100);
+
+        assertTrue(broker.getQueueProducers().length == 1);
+        assertTrue(broker.getTopicSubscribers().length == 2);
+        assertTrue(broker.getQueueSubscribers().length == 1);
+
+        ObjectName producerName = broker.getQueueProducers()[0];
+        ProducerViewMBean producerView = (ProducerViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, producerName, ProducerViewMBean.class, true);
+        assertNotNull(producerView);
+
+        if (expect) {
+            assertEquals("admin", producerView.getUserName());
+        } else {
+            assertNull(producerView.getUserName());
+        }
+
+        for (ObjectName name : broker.getTopicSubscribers()) {
+            SubscriptionViewMBean subscriberView = (SubscriptionViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, name, SubscriptionViewMBean.class, true);
+            if (expect) {
+                assertEquals("admin", subscriberView.getUserName());
+            } else {
+                assertNull(subscriberView.getUserName());
+            }
+        }
+
+        for (ObjectName name : broker.getQueueSubscribers()) {
+            SubscriptionViewMBean subscriberView = (SubscriptionViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, name, SubscriptionViewMBean.class, true);
+            if (expect) {
+                assertEquals("admin", subscriberView.getUserName());
+            } else {
+                assertNull(subscriberView.getUserName());
+            }
+        }
+
+        Set<ObjectName> names = mbeanServer.queryNames(null, null);
+        boolean found = false;
+        for (ObjectName name : names) {
+            if (name.toString().startsWith(domain + ":BrokerName=localhost,Type=Connection,ConnectorName=tcp") &&
+                name.toString().endsWith("Connection=MBeanTest")) {
+
+                ConnectionViewMBean connectionView =
+                    (ConnectionViewMBean)MBeanServerInvocationHandler.newProxyInstance(mbeanServer, name, ConnectionViewMBean.class, true);
+                assertNotNull(connectionView);
+
+                if (expect) {
+                    assertEquals("admin", connectionView.getUserName());
+                } else {
+                    assertNull(connectionView.getUserName());
+                }
+
+                found = true;
+                break;
+            }
+        }
+
+        assertTrue("Should find the connection's ManagedTransportConnection", found);
+    }
+
     public void testBrowseBytesMessages() throws Exception {
         connection = connectionFactory.createConnection();
         useConnectionWithByteMessage(connection);
