@@ -18,6 +18,7 @@
 package org.apache.activemq.store;
 
 import javax.jms.Connection;
+import javax.jms.DeliveryMode;
 import javax.jms.Message;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
@@ -33,10 +34,12 @@ import org.apache.activemq.broker.BrokerService;
 import org.apache.activemq.broker.region.policy.PendingDurableSubscriberMessageStoragePolicy;
 import org.apache.activemq.broker.region.policy.PolicyEntry;
 import org.apache.activemq.broker.region.policy.PolicyMap;
+import org.apache.activemq.broker.region.policy.SharedDeadLetterStrategy;
 import org.apache.activemq.broker.region.policy.StorePendingDurableSubscriberMessageStoragePolicy;
 import org.apache.activemq.command.ActiveMQDestination;
 import org.apache.activemq.command.ActiveMQQueue;
 import org.apache.activemq.command.ActiveMQTopic;
+import org.apache.activemq.util.Wait;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,6 +55,7 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
     protected Session sess;
     
     public boolean useCache = true;
+    public int deliveryMode = Message.DEFAULT_DELIVERY_MODE;
     public boolean dispatchAsync = true;
     public boolean prioritizeMessages = true;
     public boolean immediatePriorityDispatch = true;
@@ -75,10 +79,19 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
         StorePendingDurableSubscriberMessageStoragePolicy durableSubPending =
                 new StorePendingDurableSubscriberMessageStoragePolicy();
         durableSubPending.setImmediatePriorityDispatch(immediatePriorityDispatch);
+        durableSubPending.setUseCache(useCache);
         policy.setPendingDurableSubscriberPolicy(durableSubPending);
         PolicyMap policyMap = new PolicyMap();
         policyMap.put(new ActiveMQQueue("TEST"), policy);
         policyMap.put(new ActiveMQTopic("TEST"), policy);
+
+        // do not process expired for one test
+        PolicyEntry ignoreExpired = new PolicyEntry();
+        SharedDeadLetterStrategy ignoreExpiredStrategy = new SharedDeadLetterStrategy();
+        ignoreExpiredStrategy.setProcessExpired(false);
+        ignoreExpired.setDeadLetterStrategy(ignoreExpiredStrategy);
+        policyMap.put(new ActiveMQTopic("TEST_CLEANUP_NO_PRIORITY"), ignoreExpired);
+
         broker.setDestinationPolicy(policyMap);
         broker.start();
         broker.waitUntilStarted();
@@ -107,16 +120,26 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
     }
     
     public void testStoreConfigured() throws Exception {
-        Queue queue = sess.createQueue("TEST");
-        Topic topic = sess.createTopic("TEST");
+        final Queue queue = sess.createQueue("TEST");
+        final Topic topic = sess.createTopic("TEST");
         
         MessageProducer queueProducer = sess.createProducer(queue);
         MessageProducer topicProducer = sess.createProducer(topic);
-        
-        
-        Thread.sleep(500); // get it all propagated
-        
+
+        Wait.waitFor(new Wait.Condition() {
+            @Override
+            public boolean isSatisified() throws Exception {
+                return broker.getRegionBroker().getDestinationMap().get(queue) != null;
+            }
+        });
         assertTrue(broker.getRegionBroker().getDestinationMap().get(queue).getMessageStore().isPrioritizedMessages());
+
+        Wait.waitFor(new Wait.Condition(){
+            @Override
+            public boolean isSatisified() throws Exception {
+                return broker.getRegionBroker().getDestinationMap().get(topic) != null;
+            }
+        });
         assertTrue(broker.getRegionBroker().getDestinationMap().get(topic).getMessageStore().isPrioritizedMessages());
         
         queueProducer.close();
@@ -140,6 +163,7 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
             try {
                 MessageProducer producer = sess.createProducer(dest);
                 producer.setPriority(priority);
+                producer.setDeliveryMode(deliveryMode);
                 for (int i = 0; i < messageCount; i++) {
                     producer.send(sess.createTextMessage("message priority: " + priority));
                 }
@@ -160,6 +184,7 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
     
     public void initCombosForTestQueues() {
         addCombinationValues("useCache", new Object[] {new Boolean(true), new Boolean(false)});
+        addCombinationValues("deliveryMode", new Object[] {new Integer(DeliveryMode.NON_PERSISTENT), new Integer(DeliveryMode.PERSISTENT)});
     }
     
     public void testQueues() throws Exception {
@@ -305,7 +330,7 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
         int lowLowCount = 0;
         for (int i=0; i<numToProduce; i++) {
             Message msg = sub.receive(15000);
-            LOG.info("received i=" + i + ", " + (msg!=null? msg.getJMSMessageID() + "-" + msg.getJMSPriority() : null));
+            LOG.info("received i=" + i + ", " + (msg!=null? msg.getJMSMessageID() + ", priority:" + msg.getJMSPriority() : null));
             assertNotNull("Message " + i + " was null", msg);
             assertEquals("Message " + i + " has wrong priority", LOW_PRI+1, msg.getJMSPriority());
             assertTrue("not duplicate ", dups[i] == 0);
@@ -352,5 +377,178 @@ abstract public class MessagePriorityTest extends CombinationTestSupport {
             assertEquals("Message " + i + " has wrong priority", LOW_PRI, msg.getJMSPriority());
         }
     }
-    
+
+
+    public void initCombosForTestHighPriorityDeliveryInterleaved() {
+        addCombinationValues("useCache", new Object[] {Boolean.TRUE, Boolean.FALSE});
+    }
+
+    public void testHighPriorityDeliveryInterleaved() throws Exception {
+
+        // get zero prefetch
+        ActiveMQPrefetchPolicy prefetch = new ActiveMQPrefetchPolicy();
+        prefetch.setAll(0);
+        factory.setPrefetchPolicy(prefetch);
+        conn.close();
+        conn = factory.createConnection();
+        conn.setClientID("priority");
+        conn.start();
+        sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
+
+        ActiveMQTopic topic = (ActiveMQTopic)sess.createTopic("TEST");
+        final String subName = "priorityDisconnect";
+        TopicSubscriber sub = sess.createDurableSubscriber(topic, subName);
+        sub.close();
+
+        ProducerThread producerThread = new ProducerThread(topic, 1, HIGH_PRI);
+        producerThread.run();
+
+        producerThread.setMessagePriority(HIGH_PRI -1);
+        producerThread.setMessageCount(1);
+        producerThread.run();
+
+        producerThread.setMessagePriority(LOW_PRI);
+        producerThread.setMessageCount(1);
+        producerThread.run();
+        LOG.info("Ordered priority messages sent");
+
+        sub = sess.createDurableSubscriber(topic, subName);
+        int count = 0;
+
+        Message msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        LOG.info("received " + msg.getJMSMessageID() + ", priority:" + msg.getJMSPriority());
+        assertEquals("Message has wrong priority", HIGH_PRI, msg.getJMSPriority());
+
+        producerThread.setMessagePriority(LOW_PRI+1);
+        producerThread.setMessageCount(1);
+        producerThread.run();
+
+        msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        LOG.info("received " + msg.getJMSMessageID() + ", priority:" + msg.getJMSPriority());
+        assertEquals("high priority", HIGH_PRI -1, msg.getJMSPriority());
+
+        msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        LOG.info("received hi? : " + msg);
+        assertEquals("high priority", LOW_PRI +1, msg.getJMSPriority());
+
+        msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        LOG.info("received hi? : " + msg);
+        assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+
+        msg = sub.receive(4000);
+        assertNull("Message was null", msg);
+    }
+
+    // immediatePriorityDispatch is only relevant when cache is exhausted
+    public void initCombosForTestHighPriorityDeliveryThroughBackLog() {
+        addCombinationValues("useCache", new Object[] {Boolean.FALSE});
+        addCombinationValues("immediatePriorityDispatch", new Object[] {Boolean.TRUE});
+    }
+
+    public void testHighPriorityDeliveryThroughBackLog() throws Exception {
+
+        // get zero prefetch
+        ActiveMQPrefetchPolicy prefetch = new ActiveMQPrefetchPolicy();
+        prefetch.setAll(0);
+        factory.setPrefetchPolicy(prefetch);
+        conn.close();
+        conn = factory.createConnection();
+        conn.setClientID("priority");
+        conn.start();
+        sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
+
+        ActiveMQTopic topic = (ActiveMQTopic)sess.createTopic("TEST");
+        final String subName = "priorityDisconnect";
+        TopicSubscriber sub = sess.createDurableSubscriber(topic, subName);
+        sub.close();
+
+        ProducerThread producerThread = new ProducerThread(topic, 600, LOW_PRI);
+        producerThread.run();
+
+
+        sub = sess.createDurableSubscriber(topic, subName);
+        int count = 0;
+
+        for (;count < 300; count++) {
+            Message msg = sub.receive(15000);
+            assertNotNull("Message was null", msg);
+            assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+        }
+
+        producerThread.setMessagePriority(HIGH_PRI);
+        producerThread.setMessageCount(1);
+        producerThread.run();
+
+        Message msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        assertEquals("high priority", HIGH_PRI, msg.getJMSPriority());
+
+        for (;count < 600; count++) {
+            msg = sub.receive(15000);
+            assertNotNull("Message was null", msg);
+            assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+        }
+    }
+
+
+    public void initCombosForTestHighPriorityNonDeliveryThroughBackLog() {
+        addCombinationValues("useCache", new Object[] {Boolean.FALSE});
+        addCombinationValues("immediatePriorityDispatch", new Object[] {Boolean.FALSE});
+    }
+
+    public void testHighPriorityNonDeliveryThroughBackLog() throws Exception {
+
+        // get zero prefetch
+        ActiveMQPrefetchPolicy prefetch = new ActiveMQPrefetchPolicy();
+        prefetch.setAll(0);
+        factory.setPrefetchPolicy(prefetch);
+        conn.close();
+        conn = factory.createConnection();
+        conn.setClientID("priority");
+        conn.start();
+        sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
+
+        ActiveMQTopic topic = (ActiveMQTopic)sess.createTopic("TEST");
+        final String subName = "priorityDisconnect";
+        TopicSubscriber sub = sess.createDurableSubscriber(topic, subName);
+        sub.close();
+
+        ProducerThread producerThread = new ProducerThread(topic, 600, LOW_PRI);
+        producerThread.run();
+
+
+        sub = sess.createDurableSubscriber(topic, subName);
+        int count = 0;
+
+        for (;count < 300; count++) {
+            Message msg = sub.receive(15000);
+            assertNotNull("Message was null", msg);
+            assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+        }
+
+        producerThread.setMessagePriority(HIGH_PRI);
+        producerThread.setMessageCount(1);
+        producerThread.run();
+
+        for (;count < 400; count++) {
+            Message msg = sub.receive(15000);
+            assertNotNull("Message was null", msg);
+            assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+        }
+
+        Message msg = sub.receive(15000);
+        assertNotNull("Message was null", msg);
+        assertEquals("high priority", HIGH_PRI, msg.getJMSPriority());
+
+        for (;count < 600; count++) {
+            msg = sub.receive(15000);
+            assertNotNull("Message was null", msg);
+            assertEquals("high priority", LOW_PRI, msg.getJMSPriority());
+        }
+    }
+
 }
