@@ -16,7 +16,12 @@
  */
 package org.apache.activemq.usecases;
 
+import java.util.Random;
+import java.util.HashSet;
 import java.util.Vector;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.TimeUnit;
 import javax.jms.Connection;
 import javax.jms.JMSException;
 import javax.jms.Message;
@@ -24,13 +29,17 @@ import javax.jms.MessageConsumer;
 import javax.jms.MessageListener;
 import javax.jms.MessageProducer;
 import javax.jms.Session;
+import javax.management.ObjectName;
 import junit.framework.Test;
 import org.apache.activemq.ActiveMQConnectionFactory;
 import org.apache.activemq.broker.BrokerFactory;
 import org.apache.activemq.broker.BrokerService;
+import org.apache.activemq.broker.jmx.DurableSubscriptionViewMBean;
+import org.apache.activemq.broker.jmx.TopicViewMBean;
 import org.apache.activemq.broker.region.policy.PolicyEntry;
 import org.apache.activemq.broker.region.policy.PolicyMap;
 import org.apache.activemq.command.ActiveMQTopic;
+import org.apache.activemq.command.MessageId;
 import org.apache.activemq.store.jdbc.JDBCPersistenceAdapter;
 import org.apache.activemq.store.kahadb.KahaDBPersistenceAdapter;
 import org.apache.activemq.util.Wait;
@@ -43,12 +52,15 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
     private static final Logger LOG = LoggerFactory.getLogger(DurableSubscriptionOfflineTest.class);
     public boolean usePrioritySupport = Boolean.TRUE;
     public int journalMaxFileLength = Journal.DEFAULT_MAX_FILE_LENGTH;
+    public boolean keepDurableSubsActive = true;
     private BrokerService broker;
     private ActiveMQTopic topic;
-    private Vector<Exception> exceptions = new Vector<Exception>();
+    private Vector<Throwable> exceptions = new Vector<Throwable>();
 
     protected ActiveMQConnectionFactory createConnectionFactory() throws Exception {
-        return new ActiveMQConnectionFactory("vm://" + getName(true));
+        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory("vm://" + getName(true));
+        connectionFactory.setWatchTopicAdvisories(false);
+        return connectionFactory;
     }
 
     @Override
@@ -88,6 +100,9 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
         broker.setBrokerName(getName(true));
         broker.setDeleteAllMessagesOnStartup(deleteAllMessages);
         broker.getManagementContext().setCreateConnector(false);
+        broker.setAdvisorySupport(false);
+        broker.setKeepDurableSubsActive(keepDurableSubsActive);
+        broker.addConnector("tcp://0.0.0.0:0");
 
         if (usePrioritySupport) {
             PolicyEntry policy = new PolicyEntry();
@@ -321,6 +336,119 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
         assertEquals("offline consumer got all", sent, listener.count);
     }
 
+    public void initCombosForTestJMXCountersWithOfflineSubs() throws Exception {
+        this.addCombinationValues("keepDurableSubsActive",
+                new Object[]{Boolean.TRUE, Boolean.FALSE});
+    }
+
+    public void testJMXCountersWithOfflineSubs() throws Exception {
+        // create durable subscription 1
+        Connection con = createConnection("cliId1");
+        Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        session.createDurableSubscriber(topic, "SubsId", null, true);
+        session.close();
+        con.close();
+
+        // restart broker
+        broker.stop();
+        createBroker(false /*deleteAllMessages*/);
+
+        // send messages
+        con = createConnection();
+        session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = session.createProducer(null);
+
+        int sent = 0;
+        for (int i = 0; i < 10; i++) {
+            sent++;
+            Message message = session.createMessage();
+            producer.send(topic, message);
+        }
+        session.close();
+        con.close();
+
+        // consume some messages
+        con = createConnection("cliId1");
+        session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageConsumer consumer = session.createDurableSubscriber(topic, "SubsId", null, true);
+
+        for (int i=0; i<sent/2; i++) {
+            Message m =  consumer.receive(4000);
+            assertNotNull("got message: " + i, m);
+            LOG.info("Got :" + i + ", " + m);
+        }
+
+        // check some counters while active
+        ObjectName activeDurableSubName = broker.getAdminView().getDurableTopicSubscribers()[0];
+        LOG.info("active durable sub name: " + activeDurableSubName);
+        final DurableSubscriptionViewMBean durableSubscriptionView = (DurableSubscriptionViewMBean)
+                broker.getManagementContext().newProxyInstance(activeDurableSubName, DurableSubscriptionViewMBean.class, true);
+
+        assertTrue("is active", durableSubscriptionView.isActive());
+        assertEquals("all enqueued", keepDurableSubsActive ? 10 : 0, durableSubscriptionView.getEnqueueCounter());
+        assertTrue("correct waiting acks", Wait.waitFor(new Wait.Condition() {
+            @Override
+            public boolean isSatisified() throws Exception {
+                return 5 == durableSubscriptionView.getMessageCountAwaitingAcknowledge();
+            }
+        }));
+        assertEquals("correct dequeue", 5, durableSubscriptionView.getDequeueCounter());
+
+
+        ObjectName destinationName = broker.getAdminView().getTopics()[0];
+        TopicViewMBean topicView = (TopicViewMBean) broker.getManagementContext().newProxyInstance(destinationName, TopicViewMBean.class, true);
+        assertEquals("correct enqueue", 10, topicView.getEnqueueCount());
+        assertEquals("still zero dequeue, we don't decrement on each sub ack to stop exceeding the enqueue count with multiple subs", 0, topicView.getDequeueCount());
+        assertEquals("inflight", 5, topicView.getInFlightCount());
+
+        session.close();
+        con.close();
+
+        // check some counters when inactive
+        ObjectName inActiveDurableSubName = broker.getAdminView().getInactiveDurableTopicSubscribers()[0];
+        LOG.info("inactive durable sub name: " + inActiveDurableSubName);
+        DurableSubscriptionViewMBean durableSubscriptionView1 = (DurableSubscriptionViewMBean)
+                broker.getManagementContext().newProxyInstance(inActiveDurableSubName, DurableSubscriptionViewMBean.class, true);
+
+        assertTrue("is not active", !durableSubscriptionView1.isActive());
+        assertEquals("all enqueued", keepDurableSubsActive ? 10 : 0, durableSubscriptionView1.getEnqueueCounter());
+        assertEquals("correct awaiting ack", 0, durableSubscriptionView1.getMessageCountAwaitingAcknowledge());
+        assertEquals("correct dequeue", keepDurableSubsActive ? 5 : 0, durableSubscriptionView1.getDequeueCounter());
+
+        // destination view
+        assertEquals("correct enqueue", 10, topicView.getEnqueueCount());
+        assertEquals("still zero dequeue, we don't decrement on each sub ack to stop exceeding the enqueue count with multiple subs", 0, topicView.getDequeueCount());
+        assertEquals("inflight back to 0 after deactivate", 0, topicView.getInFlightCount());
+
+        // consume the rest
+        con = createConnection("cliId1");
+        session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        consumer = session.createDurableSubscriber(topic, "SubsId", null, true);
+
+        for (int i=0; i<sent/2;i++) {
+            Message m =  consumer.receive(30000);
+            assertNotNull("got message: " + i, m);
+            LOG.info("Got :" + i + ", " + m);
+        }
+
+        activeDurableSubName = broker.getAdminView().getDurableTopicSubscribers()[0];
+        LOG.info("durable sub name: " + activeDurableSubName);
+        final DurableSubscriptionViewMBean durableSubscriptionView2 = (DurableSubscriptionViewMBean)
+                broker.getManagementContext().newProxyInstance(activeDurableSubName, DurableSubscriptionViewMBean.class, true);
+
+        assertTrue("is active", durableSubscriptionView2.isActive());
+        assertEquals("all enqueued", keepDurableSubsActive ? 10 : 0, durableSubscriptionView2.getEnqueueCounter());
+        assertTrue("correct dequeue", Wait.waitFor(new Wait.Condition() {
+            @Override
+            public boolean isSatisified() throws Exception {
+                long val = durableSubscriptionView2.getDequeueCounter();
+                LOG.info("dequeue count:" + val);
+                return 10 == val;
+            }
+        }));
+
+    }
+
     public void initCombosForTestOfflineSubscriptionCanConsumeAfterOnlineSubs() throws Exception {
         this.addCombinationValues("defaultPersistenceAdapter",
                 new Object[]{ PersistenceAdapterChoice.KahaDB, PersistenceAdapterChoice.JDBC});
@@ -839,6 +967,208 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
         assertEquals("offline consumer got all", sent, listener.count);
     }
 
+    public void testNoDuplicateOnConcurrentSendTranCommitAndActivate() throws Exception {
+        final int messageCount = 1000;
+        Connection con = null;
+        Session session = null;
+        final int numConsumers = 10;
+        for (int i = 0; i <= numConsumers; i++) {
+            con = createConnection("cli" + i);
+            session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            session.createDurableSubscriber(topic, "SubsId", null, true);
+            session.close();
+            con.close();
+        }
+
+        class CheckForDupsClient implements Runnable {
+            HashSet<Long> ids = new HashSet<Long>();
+            final int id;
+
+            public CheckForDupsClient(int id) {
+                this.id = id;
+            }
+
+            @Override
+            public void run() {
+                try {
+                    Connection con = createConnection("cli" + id);
+                    Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+                    for (int j=0;j<2;j++) {
+                        MessageConsumer consumer = session.createDurableSubscriber(topic, "SubsId", null, true);
+                        for (int i = 0; i < messageCount/2; i++) {
+                            Message message = consumer.receive(4000);
+                            assertNotNull(message);
+                            long producerSequenceId = new MessageId(message.getJMSMessageID()).getProducerSequenceId();
+                            assertTrue("ID=" + id + " not a duplicate: " + producerSequenceId, ids.add(producerSequenceId));
+                        }
+                        consumer.close();
+                    }
+
+                    // verify no duplicates left
+                    MessageConsumer consumer = session.createDurableSubscriber(topic, "SubsId", null, true);
+                    Message message = consumer.receive(4000);
+                    if (message != null) {
+                        long producerSequenceId = new MessageId(message.getJMSMessageID()).getProducerSequenceId();
+                        assertTrue("ID=" + id + " not a duplicate: " + producerSequenceId, ids.add(producerSequenceId));
+                    }
+                    assertNull(message);
+
+
+                    session.close();
+                    con.close();
+                } catch (Throwable e) {
+                    e.printStackTrace();
+                    exceptions.add(e);
+                }
+            }
+        }
+
+
+        final String payLoad = new String(new byte[1000]);
+        con = createConnection();
+        final Session sendSession = con.createSession(true, Session.SESSION_TRANSACTED);
+        MessageProducer producer = sendSession.createProducer(topic);
+        for (int i = 0; i < messageCount; i++) {
+            producer.send(sendSession.createTextMessage(payLoad));
+        }
+
+        ExecutorService executorService = Executors.newCachedThreadPool();
+
+        // concurrent commit and activate
+        executorService.execute(new Runnable() {
+            @Override
+            public void run() {
+                try {
+                    sendSession.commit();
+                } catch (JMSException e) {
+                    e.printStackTrace();
+                    exceptions.add(e);
+                }
+            }
+        });
+        for (int i = 0; i < numConsumers; i++) {
+            executorService.execute(new CheckForDupsClient(i));
+        }
+
+        executorService.shutdown();
+        executorService.awaitTermination(5, TimeUnit.MINUTES);
+        con.close();
+
+        assertTrue("no exceptions: " + exceptions, exceptions.isEmpty());
+    }
+
+    public void testOrderOnActivateDeactivate() throws Exception {
+        for (int i=0;i<10;i++) {
+            LOG.info("Iteration: " + i);
+            doTestOrderOnActivateDeactivate();
+            broker.stop();
+            createBroker(true /*deleteAllMessages*/);
+        }
+    }
+
+    public void doTestOrderOnActivateDeactivate() throws Exception {
+        final int messageCount = 1000;
+        Connection con = null;
+        Session session = null;
+        final int numConsumers = 4;
+        for (int i = 0; i <= numConsumers; i++) {
+            con = createConnection("cli" + i);
+            session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            session.createDurableSubscriber(topic, "SubsId", null, true);
+            session.close();
+            con.close();
+        }
+
+        final String url = "failover:(tcp://localhost:"
+            + (broker.getTransportConnectors().get(1).getConnectUri()).getPort()
+            + "?wireFormat.maxInactivityDuration=0)?"
+            + "jms.watchTopicAdvisories=false&"
+            + "jms.alwaysSyncSend=true&jms.dispatchAsync=true&"
+            + "jms.sendAcksAsync=true&"
+            + "initialReconnectDelay=100&maxReconnectDelay=30000&"
+            + "useExponentialBackOff=true";
+        final ActiveMQConnectionFactory clientFactory = new ActiveMQConnectionFactory(url);
+
+        class CheckOrderClient implements Runnable {
+            final int id;
+            int runCount = 0;
+
+            public CheckOrderClient(int id) {
+                this.id = id;
+            }
+
+            @Override
+            public void run() {
+                try {
+                    synchronized (this) {
+                        Connection con = clientFactory.createConnection();
+                        con.setClientID("cli" + id);
+                        con.start();
+                        Session session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                        MessageConsumer consumer = session.createDurableSubscriber(topic, "SubsId", null, true);
+                        int nextId = 0;
+
+                        ++runCount;
+                        int i=0;
+                        for (; i < messageCount/2; i++) {
+                            Message message = consumer.receiveNoWait();
+                            if (message == null) {
+                                break;
+                            }
+                            long producerSequenceId = new MessageId(message.getJMSMessageID()).getProducerSequenceId();
+                            assertEquals(id + " expected order: runCount: " + runCount  + " id: " + message.getJMSMessageID(), ++nextId, producerSequenceId);
+                        }
+                        LOG.info(con.getClientID() + " peeked " + i);
+                        session.close();
+                        con.close();
+                    }
+                } catch (Throwable e) {
+                    e.printStackTrace();
+                    exceptions.add(e);
+                }
+            }
+        }
+
+        Runnable producer = new Runnable() {
+            final String payLoad = new String(new byte[600]);
+
+            @Override
+            public void run() {
+                try {
+                    Connection con = createConnection();
+                    final Session sendSession = con.createSession(true, Session.SESSION_TRANSACTED);
+                    MessageProducer producer = sendSession.createProducer(topic);
+                    for (int i = 0; i < messageCount; i++) {
+                        producer.send(sendSession.createTextMessage(payLoad));
+                    }
+                    LOG.info("About to commit: " + messageCount);
+                    sendSession.commit();
+                    LOG.info("committed: " + messageCount);
+                    con.close();
+                } catch (Exception e) {
+                    e.printStackTrace();
+                    exceptions.add(e);
+                }
+            }
+        };
+
+        ExecutorService executorService = Executors.newCachedThreadPool();
+
+        // concurrent commit and activate
+        for (int i = 0; i < numConsumers; i++) {
+            final CheckOrderClient client = new CheckOrderClient(i);
+            for (int j=0; j<100; j++) {
+                executorService.execute(client);
+            }
+        }
+        executorService.execute(producer);
+
+        executorService.shutdown();
+        executorService.awaitTermination(5, TimeUnit.MINUTES);
+        con.close();
+
+        assertTrue("no exceptions: " + exceptions, exceptions.isEmpty());
+    }
 
     public void testUnmatchedSubUnsubscribeDeletesAll() throws Exception {
         // create offline subs 1
@@ -1059,7 +1389,9 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
     // use very small journal to get lots of files to cleanup
     public void initCombosForTestCleanupDeletedSubAfterRestart() throws Exception {
         this.addCombinationValues("journalMaxFileLength",
-                new Object[]{new Integer(64*1024)});
+                new Object[]{new Integer(64 * 1024)});
+        this.addCombinationValues("keepDurableSubsActive",
+                new Object[]{Boolean.TRUE, Boolean.FALSE});
     }
 
     // https://issues.apache.org/jira/browse/AMQ-3206
@@ -1121,6 +1453,166 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
         assertEquals("only one journal file left after restart", 1, pa.getStore().getJournal().getFileMap().size());
     }
 
+    // https://issues.apache.org/jira/browse/AMQ-3768
+    public void testPageReuse() throws Exception {
+        Connection con = null;
+        Session session = null;
+
+        final int numConsumers = 115;
+        for (int i=0; i<=numConsumers;i++) {
+            con = createConnection("cli" + i);
+            session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            session.createDurableSubscriber(topic, "SubsId", null, true);
+            session.close();
+            con.close();
+        }
+
+
+        // populate ack locations
+        con = createConnection();
+        session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = session.createProducer(null);
+        Message message = session.createTextMessage(new byte[10].toString());
+        producer.send(topic, message);
+        con.close();
+
+        // we have a split, remove all but the last so that
+        // the head pageid changes in the acklocations listindex
+        for (int i=0; i<=numConsumers -1; i++) {
+            con = createConnection("cli" + i);
+            session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            session.unsubscribe("SubsId");
+            session.close();
+            con.close();
+        }
+
+        destroyBroker();
+        createBroker(false);
+
+        // create a bunch more subs to reuse the freed page and get us in a knot
+        for (int i=1; i<=numConsumers;i++) {
+            con = createConnection("cli" + i);
+            session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            session.createDurableSubscriber(topic, "SubsId", filter, true);
+            session.close();
+            con.close();
+        }
+    }
+
+
+    public void testRedeliveryFlag() throws Exception {
+
+        Connection con;
+        Session session;
+        final int numClients = 2;
+        for (int i=0; i<numClients; i++) {
+            con = createConnection("cliId" + i);
+            session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+            session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+            session.close();
+            con.close();
+        }
+
+        final Random random = new Random();
+
+        // send messages
+        con = createConnection();
+        session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        MessageProducer producer = session.createProducer(null);
+
+        final int count = 1000;
+        for (int i = 0; i < count; i++) {
+            Message message = session.createMessage();
+            message.setStringProperty("filter", "true");
+            producer.send(topic, message);
+        }
+        session.close();
+        con.close();
+
+        class Client implements Runnable {
+            Connection con;
+            Session session;
+            String clientId;
+            Client(String id) {
+                this.clientId = id;
+            }
+
+            @Override
+            public void run() {
+                MessageConsumer consumer = null;
+                Message message = null;
+
+                try {
+                    for (int i = -1; i < random.nextInt(10); i++) {
+                        // go online and take none
+                        con = createConnection(clientId);
+                        session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                        consumer = session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+                        session.close();
+                        con.close();
+                    }
+
+                    // consume 1
+                    con = createConnection(clientId);
+                    session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                    consumer = session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+                    message = consumer.receive(4000);
+                    assertNotNull("got message", message);
+                    // it is not reliable as it depends on broker dispatch rather than client receipt
+                    // and delivered ack
+                    //  assertFalse("not redelivered", message.getJMSRedelivered());
+                    message.acknowledge();
+                    session.close();
+                    con.close();
+
+                    // peek all
+                    for (int j = 0; j < random.nextInt(10); j++) {
+                        con = createConnection(clientId);
+                        session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                        consumer = session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+
+                        for (int i = 0; i < count - 1; i++) {
+                            assertNotNull("got message", consumer.receive(4000));
+                        }
+                        // no ack
+                        session.close();
+                        con.close();
+                    }
+
+
+                    // consume remaining
+                    con = createConnection(clientId);
+                    session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                    consumer = session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+
+                    for (int i = 0; i < count - 1; i++) {
+                        message = consumer.receive(4000);
+                        assertNotNull("got message", message);
+                        assertTrue("is redelivered", message.getJMSRedelivered());
+                    }
+                    message.acknowledge();
+                    session.close();
+                    con.close();
+
+                    con = createConnection(clientId);
+                    session = con.createSession(false, Session.CLIENT_ACKNOWLEDGE);
+                    consumer = session.createDurableSubscriber(topic, "SubsId", "filter = 'true'", true);
+                    assertNull("no message left", consumer.receive(2000));
+                } catch (Throwable throwable) {
+                    throwable.printStackTrace();
+                    exceptions.add(throwable);
+                }
+            }
+        }
+        ExecutorService executorService = Executors.newCachedThreadPool();
+        for (int i=0; i<numClients; i++) {
+            executorService.execute(new Client("cliId" + i));
+        }
+        executorService.shutdown();
+        executorService.awaitTermination(10, TimeUnit.MINUTES);
+        assertTrue("No exceptions  " + exceptions.elements(), exceptions.isEmpty());
+    }
+
     public static class Listener implements MessageListener {
         int count = 0;
         String id = null;
@@ -1157,6 +1649,7 @@ public class DurableSubscriptionOfflineTest extends org.apache.activemq.TestSupp
                 }
             }
             catch (JMSException e) {
+                e.printStackTrace();
                 exceptions.add(e);
             }
         }
